@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-16 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -29,16 +29,37 @@ class ElasticSearch {
   pthread_t esThreadLoop;
   u_int num_queued_elems;
   struct string_list *head, *tail;
-  pthread_rwlock_t listMutex;
+  Mutex listMutex;
   bool reportDrops;
+  struct timeval lastUpdateTime;
   u_int32_t elkDroppedFlowsQueueTooLong;
+  u_int64_t elkExportedFlows, elkLastExportedFlows;
+  float elkExportRate;
+  u_int64_t checkpointDroppedFlows, checkpointExportedFlows; /* Those will hold counters at checkpoints */
+
+  char *es_template_push_url, *es_version_query_url;
+  char *es_version;
+  const char * const get_es_version();
  public:
   ElasticSearch();
   ~ElasticSearch();
+  void checkPointCounters(bool drops_only) {
+    if(!drops_only)
+      checkpointExportedFlows = elkExportedFlows;
+    checkpointDroppedFlows = elkDroppedFlowsQueueTooLong;
+  };
+  inline bool atleast_version_6() {
+    const char * const ver = get_es_version();
+    return ver && strcmp(ver, "6") >= 0;
+  };
+  inline u_int32_t numDroppedFlows() const { return elkDroppedFlowsQueueTooLong; };
   int sendToES(char* msg);
   void pushEStemplate();
   void indexESdata();
   void startFlowDump();
+
+  void updateStats(const struct timeval *tv);
+  void lua(lua_State* vm, bool since_last_checkpoint) const;
 };
 
 
