@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,7 @@
 #include "ntop_includes.h"
 
 typedef struct {
-  u_int32_t pktRetr, pktOOO, pktLost;
+  u_int32_t pktRetr, pktOOO, pktLost, pktKeepAlive;
   u_int64_t last, next;
 } TCPPacketStats;
 
@@ -69,25 +69,34 @@ class Flow : public GenericHashEntry {
   u_int32_t vrfId;
   u_int8_t protocol, src2dst_tcp_flags, dst2src_tcp_flags;
   struct ndpi_flow_struct *ndpiFlow;
-  bool detection_completed, protocol_processed, blacklist_alarm_emitted,
-    cli2srv_direction, twh_over, dissect_next_http_packet, passVerdict,
+  bool detection_completed, protocol_processed,
+    cli2srv_direction, twh_over, twh_ok, dissect_next_http_packet, passVerdict,
     check_tor, l7_protocol_guessed, flow_alerted, flow_dropped_counts_increased,
     good_low_flow_detected, good_ssl_hs,
-    quota_exceeded, cli_quota_app_proto, cli_quota_is_category, srv_quota_app_proto, srv_quota_is_category;
+    quota_exceeded;
   u_int16_t diff_num_http_requests;
 #ifdef NTOPNG_PRO
+  bool counted_in_aggregated_flow;
   bool ingress2egress_direction;
   u_int8_t routing_table_id;
 #ifndef HAVE_NEDGE
   FlowProfile *trafficProfile;
+#else
+  u_int16_t cli2srv_in, cli2srv_out, srv2cli_in, srv2cli_out;
+  L7PolicySource_t cli_quota_source, srv_quota_source;
 #endif
   CounterTrend throughputTrend, goodputTrend, thptRatioTrend;
 #endif
   ndpi_protocol ndpiDetectedProtocol;
+  custom_app_t custom_app;
   void *cli_id, *srv_id;
   char *json_info, *host_server_name, *bt_hash;
-  bool dump_flow_traffic;
-
+  char *community_id_flow_hash;
+#ifdef HAVE_NEDGE
+  u_int32_t last_conntrack_update; 
+  u_int32_t marker;
+#endif
+ 
   union {
     struct {
       char *last_url, *last_method;
@@ -160,7 +169,7 @@ class Flow : public GenericHashEntry {
     u_int32_t last_dump;
   } last_db_dump;
 
-#ifdef NTOPNG_PRO
+#ifdef HAVE_NEDGE
   struct {
     struct {
       TrafficShaper *ingress, *egress;
@@ -190,22 +199,31 @@ class Flow : public GenericHashEntry {
   char* intoaV4(unsigned int addr, char* buf, u_short bufLen);
   void processLua(lua_State* vm, ProcessInfo *proc, bool client);
   void processJson(bool is_src, json_object *my_object, ProcessInfo *proc);
-  void checkBlacklistedFlow();
   void allocDPIMemory();
   bool checkTor(char *hostname);
   void setBittorrentHash(char *hash);
   bool isLowGoodput();
   void updatePacketStats(InterarrivalStats *stats, const struct timeval *when);
   void dumpPacketStats(lua_State* vm, bool cli2srv_direction);
-  inline u_int32_t getCurrentInterArrivalTime(time_t now, bool cli2srv_direction) {
-    return(1000 /* msec */
-	   * (now - (cli2srv_direction ? cli2srvStats.pktTime.lastTime.tv_sec : srv2cliStats.pktTime.lastTime.tv_sec)));
+  bool isReadyToPurge();
+  inline bool isBlacklistedFlow() {
+    return(cli_host && srv_host && (cli_host->isBlacklisted()
+				    || srv_host->isBlacklisted()
+				    || (get_protocol_category() == CUSTOM_CATEGORY_MALWARE)));
+  };
+  inline bool isDeviceAllowedProtocol() {
+      return(!cli_host || !srv_host ||
+        ((cli_host->getDeviceAllowedProtocolStatus(ndpiDetectedProtocol, true) == device_proto_allowed) &&
+         (srv_host->getDeviceAllowedProtocolStatus(ndpiDetectedProtocol, false) == device_proto_allowed)));
   }
   char* printTCPflags(u_int8_t flags, char *buf, u_int buf_len);
   inline bool isProto(u_int16_t p ) { return((ndpi_get_lower_proto(ndpiDetectedProtocol) == p) ? true : false); }
 #ifdef NTOPNG_PRO
-  bool updateDirectionShapers(bool src2dst_direction, TrafficShaper **ingress_shaper, TrafficShaper **egress_shaper);
+  void update_pools_stats(const struct timeval *tv,
+			  u_int64_t diff_sent_packets, u_int64_t diff_sent_bytes,
+			  u_int64_t diff_rcvd_packets, u_int64_t diff_rcvd_bytes);
 #endif
+  bool triggerAlerts() const;
   void dumpFlowAlert();
 
  public:
@@ -216,6 +234,16 @@ class Flow : public GenericHashEntry {
        time_t _first_seen, time_t _last_seen);
   ~Flow();
 
+  virtual void set_to_purge() {
+    /* not called from the datapath for flows, so it is only
+       safe to touch low goodput uses */
+    if(good_low_flow_detected) {
+      if(cli_host) cli_host->decLowGoodputFlows(true);
+      if(srv_host) srv_host->decLowGoodputFlows(false);
+    }
+    GenericHashEntry::set_to_purge();
+  };
+
   FlowStatus getFlowStatus();
   struct site_categories* getFlowCategory(bool force_categorization);
   void freeDPIMemory();
@@ -233,18 +261,23 @@ class Flow : public GenericHashEntry {
   char* serialize(bool es_json = false);
   json_object* flow2json();
   json_object* flow2es(json_object *flow_object);
+  json_object* flow2statusinfojson();
   inline u_int8_t getTcpFlags()        { return(src2dst_tcp_flags | dst2src_tcp_flags);  };
   inline u_int8_t getTcpFlagsCli2Srv() { return(src2dst_tcp_flags);                      };
   inline u_int8_t getTcpFlagsSrv2Cli() { return(dst2src_tcp_flags);                      };
-#ifdef NTOPNG_PRO
+#ifdef HAVE_NEDGE
   bool checkPassVerdict(const struct tm *now);
-  inline bool isPassVerdict()            { return passVerdict;  };
+  bool isPassVerdict();
+  inline void setConntrackMarker(u_int32_t marker) 	{ this->marker = marker; }
+  inline u_int32_t getConntrackMarker() 		{ return(marker); }
 #endif
-  inline void setDropVerdict()           { passVerdict = false; };
+  void setDropVerdict();
   void incFlowDroppedCounters();
 
   u_int32_t getPid(bool client);
   u_int32_t getFatherPid(bool client);
+  u_int32_t get_uid(bool client) const;
+  u_int32_t get_pid(bool client) const;
   char* get_username(bool client);
   char* get_proc_name(bool client);
   u_int32_t getNextTcpSeq(u_int8_t tcpFlags, u_int32_t tcpSeqNum, u_int32_t payloadLen) ;
@@ -268,15 +301,27 @@ class Flow : public GenericHashEntry {
 		       u_int16_t payload_len, bool src2dst_direction);
 
   void updateSeqNum(time_t when, u_int32_t sN, u_int32_t aN);
+  inline void updateCommunityIdFlowHash() {
+    if(!community_id_flow_hash)
+      community_id_flow_hash = CommunityIdFlowHash::get_community_id_flow_hash(this);
+  }
   void processDetectedProtocol();
   void setDetectedProtocol(ndpi_protocol proto_id, bool forceDetection);
+  void setCustomApp(custom_app_t ca) {
+    memcpy(&custom_app, &ca, sizeof(custom_app));
+  };
   void setJSONInfo(const char *json);
+#ifdef NTOPNG_PRO
+  inline bool is_counted_in_aggregated_flow()          { return(counted_in_aggregated_flow); };
+  inline void set_counted_in_aggregated_flow(bool val) { counted_in_aggregated_flow  = val;  };
+#endif
   bool isFlowPeer(char *numIP, u_int16_t vlanId);
   void incStats(bool cli2srv_direction, u_int pkt_len,
 		u_int8_t *payload, u_int payload_len, u_int8_t l4_proto,
 		const struct timeval *when);
   void addFlowStats(bool cli2srv_direction, u_int in_pkts, u_int in_bytes, u_int in_goodput_bytes,
 		    u_int out_pkts, u_int out_bytes, u_int out_goodput_bytes, time_t last_seen);
+  inline bool isThreeWayHandshakeOK()             { return(twh_ok);                          };
   inline bool isDetectionCompleted()              { return(detection_completed);             };
   inline struct ndpi_flow_struct* get_ndpi_flow() { return(ndpiFlow);                        };
   inline void* get_cli_id()                       { return(cli_id);                          };
@@ -285,10 +330,10 @@ class Flow : public GenericHashEntry {
   inline u_int32_t get_srv_ipv4()                 { return(srv_host->get_ip()->get_ipv4());  };
   inline struct ndpi_in6_addr* get_cli_ipv6()     { return(cli_host->get_ip()->get_ipv6());  };
   inline struct ndpi_in6_addr* get_srv_ipv6()     { return(srv_host->get_ip()->get_ipv6());  };
-  inline u_int16_t get_cli_port()                 { return(ntohs(cli_port));                 };
-  inline u_int16_t get_srv_port()                 { return(ntohs(srv_port));                 };
+  inline u_int16_t get_cli_port() const           { return(ntohs(cli_port));                 };
+  inline u_int16_t get_srv_port() const           { return(ntohs(srv_port));                 };
   inline u_int16_t get_vlan_id()                  { return(vlanId);                          };
-  inline u_int8_t  get_protocol()                 { return(protocol);                        };
+  inline u_int8_t  get_protocol() const           { return(protocol);                        };
   inline u_int64_t get_bytes()                    { return(cli2srv_bytes+srv2cli_bytes);     };
   inline u_int64_t get_bytes_cli2srv()            { return(cli2srv_bytes);                   };
   inline u_int64_t get_bytes_srv2cli()            { return(srv2cli_bytes);                   };
@@ -311,15 +356,22 @@ class Flow : public GenericHashEntry {
   inline u_int32_t get_duration()                 { return((u_int32_t)(get_last_seen()-get_first_seen())); };
   inline char* get_protocol_name()                { return(Utils::l4proto2name(protocol));   };
   inline ndpi_protocol get_detected_protocol()    { return(ndpiDetectedProtocol);          };
-  void fixAggregatedFlowFields();
-  inline ndpi_protocol_category_t get_detected_protocol_category() { return ndpi_get_proto_category(iface->get_ndpi_struct(), ndpiDetectedProtocol); };
+
   inline Host* get_cli_host()                     { return(cli_host);                        };
   inline Host* get_srv_host()                     { return(srv_host);                        };
   inline char* get_json_info()			  { return(json_info);                       };
-  inline ndpi_protocol_breed_t get_protocol_breed() { return(ndpi_get_proto_breed(iface->get_ndpi_struct(), ndpiDetectedProtocol.app_protocol)); };
-  inline char* get_protocol_breed_name()            { return(ndpi_get_proto_breed_name(iface->get_ndpi_struct(),
-										       ndpi_get_proto_breed(iface->get_ndpi_struct(),
-													    ndpiDetectedProtocol.app_protocol))); };
+  inline ndpi_protocol_breed_t get_protocol_breed() {
+    return(ndpi_get_proto_breed(iface->get_ndpi_struct(), ndpiDetectedProtocol.app_protocol));
+  };
+  inline const char * const get_protocol_breed_name() {
+    return(ndpi_get_proto_breed_name(iface->get_ndpi_struct(), get_protocol_breed()));
+  };
+  inline ndpi_protocol_category_t get_protocol_category() {
+    return(ndpi_get_proto_category(iface->get_ndpi_struct(), ndpiDetectedProtocol));
+};
+  inline const char * const get_protocol_category_name() {
+    return(ndpi_category_get_name(iface->get_ndpi_struct(), get_protocol_category()));
+  };
   char* get_detected_protocol_name(char *buf, u_int buf_len) {
     return(ndpi_protocol2name(iface->get_ndpi_struct(), ndpiDetectedProtocol, buf, buf_len));
   }
@@ -335,16 +387,10 @@ class Flow : public GenericHashEntry {
   u_int64_t get_current_packets_cli2srv();
   u_int64_t get_current_packets_srv2cli();
   void handle_process(ProcessInfo *pinfo, bool client_process);
-  inline bool idle() { return(false); } /* Idle flows are checked in Flow::update_hosts_stats */
-  bool isReadyToPurge();
+  inline bool idle() { return(is_ready_to_be_purged()); }
   inline bool is_l7_protocol_guessed() { return(l7_protocol_guessed); };
   char* print(char *buf, u_int buf_len);
-  void update_hosts_stats(struct timeval *tv);
-#ifdef NTOPNG_PRO
-  void update_pools_stats(const struct timeval *tv,
-			  u_int64_t diff_sent_packets, u_int64_t diff_sent_bytes,
-			  u_int64_t diff_rcvd_packets, u_int64_t diff_rcvd_bytes);
-#endif
+  void update_hosts_stats(struct timeval *tv, bool dump_alert);
   u_int32_t key();
   static u_int32_t key(Host *cli, u_int16_t cli_port,
 		       Host *srv, u_int16_t srv_port,
@@ -355,14 +401,11 @@ class Flow : public GenericHashEntry {
 	     u_int16_t _cli_port, u_int16_t _srv_port,
 	     u_int16_t _vlanId, u_int8_t _protocol,
 	     bool *src2srv_direction);
+  bool clientLessThanServer() const;
   void sumStats(nDPIStats *stats);
   void guessProtocol();
-  bool dumpFlow(bool idle_flow);
-  bool dumpFlowTraffic(void);
+  bool dumpFlow(bool dump_alert);
   bool match(AddressTree *ptree);
-  inline Host* get_real_client() { return(cli2srv_direction ? cli_host : srv_host); }
-  inline Host* get_real_server() { return(cli2srv_direction ? srv_host : cli_host); }
-  inline bool isSuspiciousFlowThpt();
   void dissectSSL(u_int8_t *payload, u_int16_t payload_len, const struct bpf_timeval *when, bool cli2srv);
   void dissectHTTP(bool src2dst_direction, char *payload, u_int16_t payload_len);
   void dissectSSDP(bool src2dst_direction, char *payload, u_int16_t payload_len);
@@ -391,24 +434,16 @@ class Flow : public GenericHashEntry {
   inline bool isSSLHandshake()         { return(isSSLProto() && good_ssl_hs && !protos.ssl.is_data); }
   inline bool hasSSLHandshakeEnded()   { return(getSSLEncryptionStatus() == SSL_ENCRYPTION_BOTH);    }
   FlowSSLEncryptionStatus getSSLEncryptionStatus();
-  void setDumpFlowTraffic(bool what)   { dump_flow_traffic = what; }
-  bool getDumpFlowTraffic(void)        { return dump_flow_traffic; }
-#ifdef NTOPNG_PRO
-#ifndef HAVE_NEDGE
+
+#if defined(NTOPNG_PRO) && !defined(HAVE_NEDGE)
   inline void updateProfile()     { trafficProfile = iface->getFlowProfile(this); }
   inline char* get_profile_name() { return(trafficProfile ? trafficProfile->getName() : (char*)"");}
-#endif
-  void updateFlowShapers();
-  void recheckQuota(const struct tm *now);
 #endif
   inline float getFlowRTT() { return(rttSec); }
   /* http://bradhedlund.com/2008/12/19/how-to-calculate-tcp-throughput-for-long-distance-links/ */
   inline float getCli2SrvMaxThpt() { return(rttSec ? ((float)(cli2srv_window*8)/rttSec) : 0); }
   inline float getSrv2CliMaxThpt() { return(rttSec ? ((float)(srv2cli_window*8)/rttSec) : 0); }
 
-  inline u_int32_t getCli2SrvCurrentInterArrivalTime(time_t now) { return(getCurrentInterArrivalTime(now, true));  }
-  inline u_int32_t getSrv2CliCurrentInterArrivalTime(time_t now) { return(getCurrentInterArrivalTime(now, false)); }
-
   inline u_int32_t getCli2SrvMinInterArrivalTime()  { return(cli2srvStats.pktTime.min_ms); }
   inline u_int32_t getCli2SrvMaxInterArrivalTime()  { return(cli2srvStats.pktTime.max_ms); }
   inline u_int32_t getCli2SrvAvgInterArrivalTime()  { return((cli2srv_packets < 2) ? 0 : cli2srvStats.pktTime.total_delta_ms / (cli2srv_packets-1)); }
@@ -435,9 +470,13 @@ class Flow : public GenericHashEntry {
   inline u_int32_t getFlowDeviceIp()       { return flow_device.device_ip; };
   inline u_int16_t getFlowDeviceInIndex()  { return flow_device.in_index;  };
   inline u_int16_t getFlowDeviceOutIndex() { return flow_device.out_index; };
-  void setPacketsBytes(u_int32_t s2d_pkts, u_int32_t d2s_pkts, u_int32_t s2d_bytes, u_int32_t d2s_bytes);  
-
-#ifdef NTOPNG_PRO
+#ifdef HAVE_NEDGE
+  inline void setLastConntrackUpdate(u_int32_t when) { last_conntrack_update = when; }
+  bool isNetfilterIdleFlow();
+#endif
+  
+#ifdef HAVE_NEDGE
+  void setPacketsBytes(time_t now, u_int32_t s2d_pkts, u_int32_t d2s_pkts, u_int64_t s2d_bytes, u_int64_t d2s_bytes);  
   void getFlowShapers(bool src2dst_direction, TrafficShaper **shaper_ingress, TrafficShaper **shaper_egress) {
     if(src2dst_direction) {
       *shaper_ingress = flowShaperIds.cli2srv.ingress,
@@ -447,10 +486,16 @@ class Flow : public GenericHashEntry {
 	*shaper_egress = flowShaperIds.srv2cli.egress;
     }
   }
-
+  bool updateDirectionShapers(bool src2dst_direction, TrafficShaper **ingress_shaper, TrafficShaper **egress_shaper);
+  void updateFlowShapers(bool first_update=false);
+  void recheckQuota(const struct tm *now);
   inline u_int8_t getFlowRoutingTableId() { return(routing_table_id); }
   inline void setIngress2EgressDirection(bool _ingress2egress) { ingress2egress_direction = _ingress2egress; }
   inline bool isIngress2EgressDirection() { return(ingress2egress_direction); }
+#endif
+  void housekeep();
+#ifdef HAVE_EBPF
+  void setProcessInfo(eBPFevent *event, bool client_process);
 #endif
 };
 
