@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -25,27 +25,36 @@
 #include "ntop_includes.h"
 
 extern "C" {
-#ifdef HAVE_GEOIP
-#include "GeoIP.h"
-#include "GeoIPCity.h"
+#ifdef HAVE_MAXMINDDB
+#include <maxminddb.h>
 #endif
 };
 
 class Geolocation {
  private:
-#ifdef HAVE_GEOIP
+#ifdef HAVE_MAXMINDDB
+  MMDB_s geo_ip_asn_mmdb, geo_ip_city_mmdb;
+  bool loadMaxMindDB(const char * const base_path, const char * const db_name, MMDB_s * const mmdb) const;
+  bool mmdbs_ok;
+
+#elif defined(HAVE_GEOIP)
   GeoIP *geo_ip_asn_db, *geo_ip_asn_db_v6;
   GeoIP *geo_ip_city_db, *geo_ip_city_db_v6;
 
   GeoIP* loadGeoDB(char *base_path, const char *db_name);
 #endif
 
+#define TEST_GEOLOCATION 1
+#ifdef TEST_GEOLOCATION
+  void testme();
+#endif
  public:
   Geolocation(char *db_home);
   ~Geolocation();
 
   void getAS(IpAddress *addr, u_int32_t *asn, char **asname);
-  void getInfo(IpAddress *addr, char **continent_code, char **country_code, char **city, float *latitude, float *longitude);  
+  void getInfo(IpAddress *addr, char **continent_code, char **country_code, char **city, float *latitude, float *longitude);
+  static void freeInfo(char **continent_code, char **country_code, char **city);
 };
 
 #endif /* _GEOLOCATION_H_ */
