@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -38,8 +38,11 @@ class Vlan;
 class VlanHash;
 class AutonomousSystem;
 class AutonomousSystemHash;
+class Country;
+class CountriesHash;
 class DB;
 class Paginator;
+class NetworkInterfaceTsPoint;
 
 #ifdef NTOPNG_PRO
 class AggregatedFlow;
@@ -47,6 +50,7 @@ class AggregatedFlowHash;
 class L7Policer;
 class FlowInterfacesStats;
 class TrafficShaper;
+class NIndexFlowDB;
 #endif
 
 typedef struct {
@@ -70,35 +74,55 @@ class NetworkInterface : public Checkpointable {
   u_int8_t alertLevel, purgeRuns;
   u_int32_t bridge_lan_interface_id, bridge_wan_interface_id;
   u_int32_t num_hashes;
-  
+
   /* Disaggregations */
   u_int16_t numVirtualInterfaces;
+  set<u_int32_t>  flowHashingIgnoredInterfaces;
   FlowHashingEnum flowHashingMode;
   FlowHashing *flowHashing;
 
   /* Network Discovery */
   NetworkDiscovery *discovery;
   MDNS *mdns;
-  SNMP *snmp;
+
+#ifdef HAVE_EBPF
+  /* eBPF */
+  u_int16_t next_insert_idx, next_remove_idx;
+  eBPFevent **ebpfEvents; 
+#endif
+  
+  /* Live Capture */
+  Mutex active_captures_lock;
+  u_int8_t num_live_captures;
+  struct ntopngLuaContext *live_captures[MAX_NUM_PCAP_CAPTURES];
+  static bool matchLiveCapture(struct ntopngLuaContext * const luactx,
+			       const struct pcap_pkthdr * const h,
+			       const u_char * const packet,
+			       Flow * const f);
+  void deliverLiveCapture(const struct pcap_pkthdr * const h, const u_char * const packet, Flow * const f);
   
   string ip_addresses;
   int id;
-  bool bridge_interface, is_dynamic_interface;
+  bool bridge_interface, is_dynamic_interface, is_traffic_mirrored, is_loopback;
+  bool reload_custom_categories;
 #ifdef NTOPNG_PRO
   L7Policer *policer;
 #ifndef HAVE_NEDGE
   FlowProfiles  *flow_profiles, *shadow_flow_profiles;
 #endif
+  CustomAppStats *custom_app_stats;
   FlowInterfacesStats *flow_interfaces_stats;
   AggregatedFlowHash *aggregated_flows_hash; /**< Hash used to store aggregated flows information. */
 #endif
   EthStats ethStats;
+  std::map<u_int32_t, u_int64_t> ip_mac; /* IP (network byte order) <-> MAC association [2 bytes are unused] */
   u_int32_t arp_requests, arp_replies;
   ICMPstats icmp_v4, icmp_v6;
   LocalTrafficStats localStats;
   int pcap_datalink_type; /**< Datalink type of pcap. */
   pthread_t pollLoop;
   bool pollLoopCreated, has_too_many_hosts, has_too_many_flows, mtuWarningShown;
+  bool too_many_drops, slow_stats_update;
   u_int32_t ifSpeed, numL2Devices, numHosts, numLocalHosts, scalingFactor;
   u_int64_t checkpointPktCount, checkpointBytesCount, checkpointPktDropCount; /* Those will hold counters at checkpoints */
   u_int16_t ifMTU;
@@ -107,15 +131,14 @@ class NetworkInterface : public Checkpointable {
   PacketStats pktStats;
   FlowHash *flows_hash; /**< Hash used to store flows information. */
   u_int32_t last_remote_pps, last_remote_bps;
+  u_int8_t packet_drops_alert_perc;
+  TimeseriesExporter *tsExporter;
+  TimeseriesRing *ts_ring;
 
   /* Sub-interface views */
   u_int8_t numSubInterfaces;
   NetworkInterface *subInterfaces[MAX_NUM_VIEW_INTERFACES];
 
-  /* Lua */
-  bool user_scripts_reload_inline, user_scripts_reload_periodic;
-  lua_State *L_user_scripts_inline, *L_user_scripts_periodic;
-
   u_int nextFlowAggregation;
   TcpFlowStats tcpFlowStats;
   TcpPacketStats tcpPacketStats;
@@ -131,31 +154,31 @@ class NetworkInterface : public Checkpointable {
   /* Autonomous Systems */
   AutonomousSystemHash *ases_hash; /**< Hash used to store Autonomous Systems information. */
 
+  /* Countries */
+  CountriesHash *countries_hash;
+
   /* Vlans */
   VlanHash *vlans_hash; /**< Hash used to store Vlans information. */
 
   /* Hosts */
   HostHash *hosts_hash; /**< Hash used to store hosts information. */
-  bool purge_idle_flows_hosts, sprobe_interface, inline_interface,
-    dump_all_traffic, dump_to_tap, dump_to_disk, dump_unknown_traffic;
+  bool purge_idle_flows_hosts, sprobe_interface, inline_interface;
   DB *db;
-  u_int dump_sampling_rate, dump_max_pkts_file, dump_max_duration, dump_max_files;
   StatsManager  *statsManager;
   AlertsManager *alertsManager;
   HostPools *host_pools;
-  bool has_vlan_packets, has_mac_addresses;
+  VlanAddressTree *hide_from_top, *hide_from_top_shadow;
+  bool has_vlan_packets, has_ebpf_events, has_mac_addresses;
   struct ndpi_detection_module_struct *ndpi_struct;
   time_t last_pkt_rcvd, last_pkt_rcvd_remote, /* Meaningful only for ZMQ interfaces */
     next_idle_flow_purge, next_idle_host_purge;
   bool running, is_idle;
-  PacketDumper *pkt_dumper;
-  PacketDumperTuntap *pkt_dumper_tap;
   NetworkStats *networkStats;
   InterfaceStatsHash *interfaceStats;
   char checkpoint_compression_buffer[CONST_MAX_NUM_CHECKPOINTS][MAX_CHECKPOINT_COMPRESSION_BUFFER_SIZE];
 
-  lua_State* initUserScriptsInterpreter(const char *lua_file, const char *context);
-  void termLuaInterpreter();
+  PROFILING_DECLARE(24);
+
   void init();
   void deleteDataStructures();
   NetworkInterface* getSubInterface(u_int32_t criteria, bool parser_interface);
@@ -167,7 +190,7 @@ class NetworkInterface : public Checkpointable {
 		bool *src2dst_direction,
 		time_t first_seen, time_t last_seen,
 		u_int32_t rawsize,
-		bool *new_flow);
+		bool *new_flow, bool create_if_missing);
   int sortHosts(u_int32_t *begin_slot,
 		bool walk_all,
 		struct flowHostRetriever *retriever,
@@ -178,18 +201,22 @@ class NetworkInterface : public Checkpointable {
 		char *countryFilter, char *mac_filter,
 		u_int16_t vlan_id, char *osFilter,
 		u_int32_t asnFilter, int16_t networkFilter,
-		u_int16_t pool_filter, bool filtered_hosts, u_int8_t ipver_filter, int proto_filter,
-		bool hostMacsOnly, char *sortColumn);
+		u_int16_t pool_filter, bool filtered_hosts,
+		bool blacklisted_hosts, bool hide_top_hidden, bool anomalousOnly,
+		u_int8_t ipver_filter, int proto_filter,
+		TrafficType traffic_type_filter,
+		char *sortColumn);
   int sortASes(struct flowHostRetriever *retriever,
 	       char *sortColumn);
+  int sortCountries(struct flowHostRetriever *retriever,
+	       char *sortColumn);
   int sortVLANs(struct flowHostRetriever *retriever,
 		char *sortColumn);
   int sortMacs(u_int32_t *begin_slot,
 	       bool walk_all,
 	       struct flowHostRetriever *retriever,
 	       u_int8_t bridge_iface_idx,
-	       u_int16_t vlan_id, bool sourceMacsOnly,	       
-	       bool hostMacsOnly, bool dhcpMacsOnly,
+	       bool sourceMacsOnly,	bool dhcpMacsOnly,
 	       const char *manufacturer,
 	       char *sortColumn, u_int16_t pool_filter, u_int8_t devtype_filter,
 	       u_int8_t location_filter);
@@ -200,13 +227,9 @@ class NetworkInterface : public Checkpointable {
 		Host *host,
 		Paginator *p,
 		const char *sortColumn);
-  
+
   bool isNumber(const char *str);
-  bool validInterface(char *name);
-  bool isInterfaceUp(char *name);
   bool checkIdle();
-  void dumpPacketDisk(const struct pcap_pkthdr *h, const u_char *packet, dump_reason reason);
-  void dumpPacketTap(const struct pcap_pkthdr *h, const u_char *packet, dump_reason reason);
 
   void disablePurge(bool on_flows);
   void enablePurge(bool on_flows);
@@ -215,7 +238,9 @@ class NetworkInterface : public Checkpointable {
 		PacketStats *_pktStats, TcpPacketStats *_tcpPacketStats);
 
   void topItemsCommit(const struct timeval *when);
-  
+  void checkMacIPAssociation(bool triggerEvent, u_char *_mac, u_int32_t ipv4);
+  void pollQueuedeBPFEvents();
+  void reloadCustomCategories();
  public:
   /**
   * @brief A Constructor
@@ -227,8 +252,9 @@ class NetworkInterface : public Checkpointable {
   NetworkInterface(const char *name, const char *custom_interface_type = NULL);
   virtual ~NetworkInterface();
 
-  void finishInitialization();
+  void finishInitialization(u_int8_t num_defined_interfaces);
   virtual u_int32_t getASesHashSize();
+  virtual u_int32_t getCountriesHashSize();
   virtual u_int32_t getVLANsHashSize();
   virtual u_int32_t getMacsHashSize();
   virtual u_int32_t getHostsHashSize();
@@ -246,7 +272,6 @@ class NetworkInterface : public Checkpointable {
   virtual void startPacketPolling();
   virtual void shutdown();
   virtual void cleanup();
-  virtual char *getScriptName()                { return NULL;   };
   virtual char *getEndpoint(u_int8_t id)       { return NULL;   };
   virtual bool set_packet_filter(char *filter) { return(false); };
   virtual void incrDrops(u_int32_t num)        { ; }
@@ -254,21 +279,28 @@ class NetworkInterface : public Checkpointable {
      See C++ FAQ Lite covers this in section 23.7
   */
   inline virtual bool isPacketInterface()      { return(getIfType() != interface_type_FLOW); }
-  inline virtual bool isDiscoverableInterface(){ return(false);                              }
+#if defined(linux) && !defined(HAVE_LIBCAP) && !defined(HAVE_NEDGE)
+  /* Note: if we miss the capabilities, we block the overriding of this method. */
+  inline bool
+#else
+  virtual bool
+#endif
+                      isDiscoverableInterface(){ return(false);                              }
   inline virtual char* altDiscoverableName()   { return(NULL);                               }
   inline virtual const char* get_type()        { return(customIftype ? customIftype : CONST_INTERFACE_TYPE_UNKNOWN); }
-  inline virtual InterfaceType getIfType()     { return(interface_type_UNKNOWN); }
+  virtual InterfaceType getIfType()            { return(interface_type_UNKNOWN); }
   inline FlowHash *get_flows_hash()            { return flows_hash;     }
   inline TcpFlowStats* getTcpFlowStats()       { return(&tcpFlowStats); }
   inline virtual bool is_ndpi_enabled()        { return(true);          }
   inline u_int  getNumnDPIProtocols()          { return(ndpi_get_num_supported_protocols(ndpi_struct)); };
   inline time_t getTimeLastPktRcvd()           { return(last_pkt_rcvd ? last_pkt_rcvd : last_pkt_rcvd_remote); };
-  inline void  setTimeLastPktRcvd(time_t t)    { last_pkt_rcvd = t; };
+  inline void  setTimeLastPktRcvd(time_t t)    { if(t > last_pkt_rcvd) last_pkt_rcvd = t; };
   inline ndpi_protocol_category_t get_ndpi_proto_category(ndpi_protocol proto) { return(ndpi_get_proto_category(ndpi_struct, proto)); };
   inline const char* get_ndpi_category_name(ndpi_protocol_category_t category) { return(ndpi_category_get_name(ndpi_struct, category)); };
   ndpi_protocol_category_t get_ndpi_proto_category(u_int protoid);
   inline char* get_ndpi_proto_name(u_int id)   { return(ndpi_get_proto_name(ndpi_struct, id));   };
   inline int   get_ndpi_proto_id(char *proto)  { return(ndpi_get_protocol_id(ndpi_struct, proto));   };
+  inline int   get_ndpi_category_id(char *cat) { return(ndpi_get_category_id(ndpi_struct, cat));     };
   inline char* get_ndpi_proto_breed_name(u_int id) {
     return(ndpi_get_proto_breed_name(ndpi_struct, ndpi_get_proto_breed(ndpi_struct, id))); };
   inline u_int get_flow_size()                 { return(ndpi_detection_get_sizeof_ndpi_flow_struct()); };
@@ -280,6 +312,8 @@ class NetworkInterface : public Checkpointable {
   inline bool get_inline_interface()           { return inline_interface;  }
   inline bool hasSeenVlanTaggedPackets()       { return(has_vlan_packets); }
   inline void setSeenVlanTaggedPackets()       { has_vlan_packets = true;  }
+  inline bool hasSeenEBPFEvents()              { return(has_ebpf_events);  }
+  inline void setSeenEBPFEvents()              { has_ebpf_events = true;   }
   inline bool hasSeenMacAddresses()            { return(has_mac_addresses); }
   inline void setSeenMacAddresses()            { has_mac_addresses = true;  }
   inline struct ndpi_detection_module_struct* get_ndpi_struct() { return(ndpi_struct);         };
@@ -288,19 +322,22 @@ class NetworkInterface : public Checkpointable {
   inline void enable_sprobe()                  { sprobe_interface = true; };
   int dumpFlow(time_t when, Flow *f);
 #ifdef NTOPNG_PRO
-  int  dumpAggregatedFlow(AggregatedFlow *f);
+  void dumpAggregatedFlow(time_t when, AggregatedFlow *f);
   void flushFlowDump();
 #endif
   int dumpDBFlow(time_t when, Flow *f);
   int dumpEsFlow(time_t when, Flow *f);
   int dumpLsFlow(time_t when, Flow *f);
+#if defined(HAVE_NINDEX) && defined(NTOPNG_PRO)
+  inline bool dumpnIndexFlow(time_t when, Flow *f)  { return(db ? db->dumpFlow(when, f, NULL) : false); };
+#endif
   int dumpLocalHosts2redis(bool disable_purge);
   inline void incRetransmittedPkts(u_int32_t num)   { tcpPacketStats.incRetr(num); };
   inline void incOOOPkts(u_int32_t num)             { tcpPacketStats.incOOO(num);  };
   inline void incLostPkts(u_int32_t num)            { tcpPacketStats.incLost(num); };
   bool checkPointHostCounters(lua_State* vm, u_int8_t checkpoint_id, char *host_ip, u_int16_t vlan_id, DetailsLevel details_level);
   bool checkPointNetworkCounters(lua_State* vm, u_int8_t checkpoint_id, u_int8_t network_id, DetailsLevel details_level);
-  bool checkPointHostTalker(lua_State* vm, char *host_ip, u_int16_t vlan_id);
+  bool checkPointHostTalker(lua_State* vm, char *host_ip, u_int16_t vlan_id, bool saveCheckpoint);
   inline bool checkPointInterfaceCounters(lua_State* vm, u_int8_t checkpoint_id, DetailsLevel details_level) { return checkpoint(vm, this, checkpoint_id, details_level); }
   inline char* getCheckpointCompressionBuffer(u_int8_t checkpoint_id) { return (checkpoint_id<CONST_MAX_NUM_CHECKPOINTS) ? checkpoint_compression_buffer[checkpoint_id] : NULL; };
   void checkPointCounters(bool drops_only);
@@ -310,20 +347,27 @@ class NetworkInterface : public Checkpointable {
   virtual u_int64_t getCheckPointNumBytes();
   virtual u_int32_t getCheckPointNumPacketDrops();
 
-  inline void incFlagsStats(u_int8_t flags) { pktStats.incFlagStats(flags); };
-  inline void incStats(bool ingressPacket, time_t when, u_int16_t eth_proto, u_int16_t ndpi_proto,		       
-		       u_int pkt_len, u_int num_pkts, u_int pkt_overhead, bool conntrack_update=false) {
-#ifdef HAVE_NEDGE
-    if(! conntrack_update)
-      return;
-#endif
+  inline void _incStats(bool ingressPacket, time_t when,
+			u_int16_t eth_proto, u_int16_t ndpi_proto,
+		       u_int pkt_len, u_int num_pkts, u_int pkt_overhead) {
     ethStats.incStats(ingressPacket, eth_proto, num_pkts, pkt_len, pkt_overhead);
-    ndpiStats.incStats(when, ndpi_proto, 0, 0, 1, pkt_len);
+    ndpiStats.incStats(when, ndpi_proto, 0, 0, num_pkts, pkt_len);
     // Note: here we are not currently interested in packet direction, so we tell it is receive
     ndpiStats.incCategoryStats(when, get_ndpi_proto_category(ndpi_proto), 0 /* see above comment */, pkt_len);
     pktStats.incStats(pkt_len);
   };
 
+  inline void incFlagsStats(u_int8_t flags) { pktStats.incFlagStats(flags); };
+  inline void incStats(bool ingressPacket, time_t when, u_int16_t eth_proto, u_int16_t ndpi_proto,
+		       u_int pkt_len, u_int num_pkts, u_int pkt_overhead) {
+#ifdef HAVE_NEDGE
+    /* In nedge, we only update the stats periodically with conntrack */
+    return;
+#endif
+
+    _incStats(ingressPacket, when, eth_proto, ndpi_proto, pkt_len, num_pkts, pkt_overhead);
+  };
+
   inline void incLocalStats(u_int num_pkts, u_int pkt_len, bool localsender, bool localreceiver) {
     localStats.incStats(num_pkts, pkt_len, localsender, localreceiver);
   };
@@ -332,6 +376,8 @@ class NetworkInterface : public Checkpointable {
   inline int get_datalink()        { return(pcap_datalink_type); };
   inline void set_datalink(int l)  { pcap_datalink_type = l;     };
   inline int isRunning()	   { return running;             };
+  inline bool isTrafficMirrored()  { return is_traffic_mirrored; };
+  void  updateTrafficMirrored();
   bool restoreHost(char *host_ip, u_int16_t vlan_id);
   u_int printAvailableInterfaces(bool printHelp, int idx, char *ifname, u_int ifname_len);
   void findFlowHosts(u_int16_t vlan_id,
@@ -361,12 +407,10 @@ class NetworkInterface : public Checkpointable {
 		     Host **srcHost, Host **dstHost, Flow **flow);
   void processFlow(ZMQ_Flow *zflow);
   void processInterfaceStats(sFlowInterfaceStats *stats);
-  void dumpFlows();
   void getnDPIStats(nDPIStats *stats, AddressTree *allowed_hosts, const char *host_ip, u_int16_t vlan_id);
   void periodicStatsUpdate();
   virtual void lua(lua_State* vm);
-  void getnDPIProtocols(lua_State *vm);
-  void getnDPIProtocols(lua_State *vm, ndpi_protocol_category_t filter);
+  void getnDPIProtocols(lua_State *vm, ndpi_protocol_category_t filter, bool skip_critical);
   void setnDPIProtocolCategory(u_int16_t protoId, ndpi_protocol_category_t protoCategory);
 
   int getActiveHostsList(lua_State* vm,
@@ -378,7 +422,9 @@ class NetworkInterface : public Checkpointable {
 			 char *countryFilter, char *mac_filter,
 			 u_int16_t vlan_id, char *osFilter,
 			 u_int32_t asnFilter, int16_t networkFilter,
-			 u_int16_t pool_filter, bool filtered_hosts, u_int8_t ipver_filter, int proto_filter,
+			 u_int16_t pool_filter, bool filtered_hosts, bool blacklisted_hosts, bool hide_top_hidden,
+       u_int8_t ipver_filter, int proto_filter,
+       TrafficType traffic_type_filter, bool tsLua, bool anomalousOnly,
 			 char *sortColumn, u_int32_t maxHits,
 			 u_int32_t toSkip, bool a2zSortOrder);
   int getActiveHostsGroup(lua_State* vm,
@@ -390,8 +436,9 @@ class NetworkInterface : public Checkpointable {
 			  u_int16_t vlan_id, char *osFilter,
 			  u_int32_t asnFilter, int16_t networkFilter,
 			  u_int16_t pool_filter, bool filtered_hosts, u_int8_t ipver_filter,
-			  bool hostsOnly, char *groupColumn);
+			  char *groupColumn);
   int getActiveASList(lua_State* vm, const Paginator *p);
+  int getActiveCountriesList(lua_State* vm, const Paginator *p);
   int getActiveVLANList(lua_State* vm,
 			char *sortColumn, u_int32_t maxHits,
 			u_int32_t toSkip, bool a2zSortOrder,
@@ -400,9 +447,7 @@ class NetworkInterface : public Checkpointable {
 		       u_int32_t *begin_slot,
 		       bool walk_all,
 		       u_int8_t bridge_iface_idx,
-		       u_int16_t vlan_id,
-		       bool sourceMacsOnly,
-		       bool hostMacsOnly, bool dhcpMacsOnly,
+		       bool sourceMacsOnly, bool dhcpMacsOnly,
 		       const char *manufacturer,
 		       char *sortColumn, u_int32_t maxHits,
 		       u_int32_t toSkip, bool a2zSortOrder,
@@ -410,16 +455,12 @@ class NetworkInterface : public Checkpointable {
 		       u_int8_t location_filter);
   int getActiveMacManufacturers(lua_State* vm,
 				u_int8_t bridge_iface_idx,
-				u_int16_t vlan_id,
-				bool sourceMacsOnly,
-				bool hostMacsOnly, bool dhcpMacsOnly,
+				bool sourceMacsOnly, bool dhcpMacsOnly,
 				u_int32_t maxHits, u_int8_t devtype_filter,
 			        u_int8_t location_filter);
   int getActiveDeviceTypes(lua_State* vm,
 			   u_int8_t bridge_iface_idx,
-			   u_int16_t vlan_id,
-			   bool sourceMacsOnly,
-			   bool hostMacsOnly, bool dhcpMacsOnly,
+			   bool sourceMacsOnly, bool dhcpMacsOnly,
 			   u_int32_t maxHits, const char *manufacturer,
 			   u_int8_t location_filter);
   int getMacsIpAddresses(lua_State *vm, int idx);
@@ -455,10 +496,16 @@ class NetworkInterface : public Checkpointable {
   virtual u_int     getNumMacs();
   virtual u_int     getNumHTTPHosts();
 
+  inline u_int64_t  getNumPacketsSinceReset()     { return getNumPackets() - getCheckPointNumPackets(); }
+  inline u_int64_t  getNumBytesSinceReset()       { return getNumBytes() - getCheckPointNumBytes(); }
+  inline u_int64_t  getNumPacketDropsSinceReset() { return getNumPacketDrops() - getCheckPointNumPacketDrops(); }
+
   void runHousekeepingTasks();
+  void runShutdownTasks();
   Vlan* getVlan(u_int16_t vlanId, bool createIfNotPresent);
   AutonomousSystem *getAS(IpAddress *ipa, bool createIfNotPresent);
-  virtual Mac*  getMac(u_int8_t _mac[6], u_int16_t vlanId, bool createIfNotPresent);
+  Country* getCountry(const char *country_name, bool createIfNotPresent);
+  virtual Mac*  getMac(u_int8_t _mac[6], bool createIfNotPresent);
   virtual Host* getHost(char *host_ip, u_int16_t vlan_id);
   bool getHostInfo(lua_State* vm, AddressTree *allowed_hosts, char *host_ip, u_int16_t vlan_id);
   void findUserFlows(lua_State *vm, char *username);
@@ -475,18 +522,21 @@ class NetworkInterface : public Checkpointable {
 #ifdef NTOPNG_PRO
   void refreshL7Rules();
   void refreshShapers();
-  inline L7Policer* getL7Policer()                     { return(policer);     }
+  inline L7Policer* getL7Policer()                     { return(policer);                }
   inline FlowInterfacesStats* getFlowInterfacesStats() { return(flow_interfaces_stats);  }
 #endif
-  inline HostPools* getHostPools()         { return(host_pools);  }
-
-  PacketDumper *getPacketDumper(void)      { return pkt_dumper; }
-  PacketDumperTuntap *getPacketDumperTap(void)      { return pkt_dumper_tap; }
+  inline HostPools* getHostPools()                     { return(host_pools);    }
 
+  bool registerLiveCapture(struct ntopngLuaContext * const luactx, int *id);
+  bool deregisterLiveCapture(struct ntopngLuaContext * const luactx);
+  void dumpLiveCaptures(lua_State* vm);
+  bool stopLiveCapture(int capture_id);
 #ifdef NTOPNG_PRO
+#ifdef HAVE_NEDGE
   void updateHostsL7Policy(u_int16_t host_pool_id);
   void updateFlowsL7Policy();
-  void resetPoolsStats();
+#endif
+  void resetPoolsStats(u_int16_t pool_filter);
   inline void luaHostPoolsStats(lua_State *vm)           { if (host_pools) host_pools->luaStats(vm);           };
   inline void luaHostPoolsVolatileMembers(lua_State *vm) { if (host_pools) host_pools->luaVolatileMembers(vm); };
 #endif
@@ -494,32 +544,19 @@ class NetworkInterface : public Checkpointable {
   inline u_int16_t getHostPool(Host *h) { if(h && host_pools) return host_pools->getPool(h); return NO_HOST_POOL_ID; };
   inline u_int16_t getHostPool(Mac *m)  { if(m && host_pools) return host_pools->getPool(m); return NO_HOST_POOL_ID; };
 
-  bool updateDumpAllTrafficPolicy(void);
-  bool updateDumpTrafficDiskPolicy();
-  bool updateDumpTrafficTapPolicy();
-  int updateDumpTrafficSamplingRate();
-  int updateDumpTrafficMaxPktsPerFile();
-  int updateDumpTrafficMaxSecPerFile();
-  int updateDumpTrafficMaxFiles(void);
-  inline bool getDumpTrafficDiskPolicy()      { return(dump_to_disk);       }
-  inline bool getDumpTrafficTapPolicy()       { return(dump_to_tap);        }
-  inline u_int getDumpTrafficSamplingRate()   { return(dump_sampling_rate); }
-  inline u_int getDumpTrafficMaxPktsPerFile() { return(dump_max_pkts_file); }
-  inline u_int getDumpTrafficMaxSecPerFile()  { return(dump_max_duration);  }
-  inline u_int getDumpTrafficMaxFiles()       { return(dump_max_files);     }
-  inline char* getDumpTrafficTapName()        { return(pkt_dumper_tap ? pkt_dumper_tap->getName() : (char*)""); }
-  void loadDumpPrefs();
   void loadScalingFactorPrefs();
+  void loadPacketsDropsAlertPrefs();
   void getnDPIFlowsCount(lua_State *vm);
 
   inline void setBridgeLanInterfaceId(u_int32_t v) { bridge_lan_interface_id = v;     };
   inline u_int32_t getBridgeLanInterfaceId()       { return(bridge_lan_interface_id); };
   inline void setBridgeWanInterfaceId(u_int32_t v) { bridge_wan_interface_id = v;     };
-  inline u_int32_t getBridgeWanInterfaceId()       { return(bridge_wan_interface_id); };  
+  inline u_int32_t getBridgeWanInterfaceId()       { return(bridge_wan_interface_id); };
   inline HostHash* get_hosts_hash()                { return(hosts_hash);              }
   inline MacHash*  get_macs_hash()                 { return(macs_hash);               }
   inline VlanHash*  get_vlans_hash()               { return(vlans_hash);              }
   inline AutonomousSystemHash* get_ases_hash()     { return(ases_hash);               }
+  inline CountriesHash* get_countries_hash()       { return(countries_hash);          }
   inline bool is_bridge_interface()                { return(bridge_interface);        }
   inline const char* getLocalIPAddresses()         { return(ip_addresses.c_str());    }
   void addInterfaceAddress(char *addr);
@@ -536,13 +573,13 @@ class NetworkInterface : public Checkpointable {
   inline FlowProfile* getFlowProfile(Flow *f)  { return(flow_profiles ? flow_profiles->getFlowProfile(f) : NULL);           }
   inline bool checkProfileSyntax(char *filter) { return(flow_profiles ? flow_profiles->checkProfileSyntax(filter) : false); }
 #endif
-  
+
   bool passShaperPacket(TrafficShaper *a_shaper, TrafficShaper *b_shaper, struct pcap_pkthdr *h);
   void initL7Policer();
 #endif
 
   void getFlowsStatus(lua_State *vm);
-  void startDBLoop() { if(db) db->startDBLoop(); };
+  void startDBLoop()               { if(db) db->startDBLoop();                 };
   inline bool createDBSchema()     { if(db) { return db->createDBSchema();     } return false;   };
   inline bool createNprobeDBView() { if(db) { return db->createNprobeDBView(); } return false;   };
 #ifdef NTOPNG_PRO
@@ -562,17 +599,16 @@ class NetworkInterface : public Checkpointable {
 
   void refreshHostsAlertPrefs(bool full_refresh);
   int updateHostTrafficPolicy(AddressTree* allowed_networks, char *host_ip, u_int16_t host_vlan);
-  int setHostDumpTrafficPolicy(AddressTree* allowed_networks, char *host_ip, u_int16_t host_vlan, bool dump_traffic_to_disk);
-  int engageReleaseHostAlert(AddressTree* allowed_networks, char *host_ip, u_int16_t host_vlan, bool engage,
-			     AlertEngine alert_engine,
-			     char *engaged_alert_id, AlertType alert_type, AlertLevel alert_severity, const char *alert_json);
 
-  int luaEvalFlow(Flow *f, const LuaCallback cb);
-  inline void forceLuaInterpreterReload() { user_scripts_reload_inline = user_scripts_reload_periodic = true; };
+  void reloadHideFromTop(bool refreshHosts=true);
+  inline void requestReloadCustomCategories()       { reload_custom_categories = true; }
+  inline bool customCategoriesReloadRequested()     { return reload_custom_categories; }
+  void reloadHostsBlacklist();
+  bool isHiddenFromTop(Host *host);
   inline virtual bool areTrafficDirectionsSupported() { return(false); };
   inline virtual bool isView() { return(false); };
-  bool getMacInfo(lua_State* vm, char *mac, u_int16_t vlan_id);
-  bool setMacDeviceType(char *strmac, u_int16_t vlanId, DeviceType dtype, bool alwaysOverwrite);
+  bool getMacInfo(lua_State* vm, char *mac);
+  bool setMacDeviceType(char *strmac, DeviceType dtype, bool alwaysOverwrite);
   bool setMacOperatingSystem(lua_State* vm, char *mac, OperatingSystem os);
   bool getASInfo(lua_State* vm, u_int32_t asn);
   bool getVLANInfo(lua_State* vm, u_int16_t vlan_id);
@@ -587,6 +623,8 @@ class NetworkInterface : public Checkpointable {
   inline void decAlertLevel()               { if(--alertLevel < 0) alertLevel = 0; }
   inline int8_t getAlertLevel()             { return(alertLevel);                  }
 #ifdef NTOPNG_PRO
+  virtual bool getCustomAppDetails(u_int32_t remapped_app_id, u_int32_t *const pen, u_int32_t *const app_field, u_int32_t *const app_id) { return false; };
+  virtual void addToNotifiedInformativeCaptivePortal(u_int32_t client_ip) { ; };
   virtual void addIPToLRUMatches(u_int32_t client_ip, u_int16_t user_pool_id,
 				 char *label, int32_t lifetime_sec) { ; };
   void aggregatePartialFlow(Flow *flow);
@@ -607,9 +645,9 @@ class NetworkInterface : public Checkpointable {
   }
 
   inline bool mdnsQueueResolveIPv4(u_int32_t ipv4addr, bool alsoUseGatewayDNS) {
-    return(mdns ? mdns->queueResolveIPv4(ipv4addr, alsoUseGatewayDNS) : false);    
+    return(mdns ? mdns->queueResolveIPv4(ipv4addr, alsoUseGatewayDNS) : false);
   }
-  
+
   inline void mdnsFetchResolveResponses(lua_State* vm, int32_t timeout_sec = 2) {
     if(mdns) mdns->fetchResolveResponses(vm, timeout_sec);
   }
@@ -620,7 +658,6 @@ class NetworkInterface : public Checkpointable {
   inline bool isDynamicInterface()                { return(is_dynamic_interface);            };
   inline void setDynamicInterface()               { is_dynamic_interface = true;             };
   inline void luaTopMacsProtos(lua_State *vm) { frequentMacs->luaTopMacsProtocols(vm); }
-  inline SNMP* getSNMP() { return(snmp); }
   inline MDNS* getMDNS() { return(mdns); }
   inline NetworkDiscovery* getNetworkDiscovery() { return(discovery); }
   inline void incPoolNumHosts(u_int16_t id, bool isInlineCall) {
@@ -635,12 +672,29 @@ class NetworkInterface : public Checkpointable {
   inline void decPoolNumL2Devices(u_int16_t id, bool isInlineCall) {
     if (host_pools) host_pools->decNumL2Devices(id, isInlineCall);
   };
-  void updateFlowStats(u_int8_t protocol,
-		       u_int32_t srcHost, u_int16_t sport,
-		       u_int32_t dstHost, u_int16_t dport,
-		       u_int32_t s2d_pkts, u_int32_t d2s_pkts,
-		       u_int32_t s2d_bytes, u_int32_t d2s_bytes);
   Host* findHostByIP(AddressTree *allowed_hosts, char *host_ip, u_int16_t vlan_id);
+#ifdef HAVE_NINDEX
+  NIndexFlowDB* getNindex();
+#endif
+  inline TimeseriesExporter* getTSExporter() { if(!tsExporter) tsExporter = new TimeseriesExporter(this); return(tsExporter); }
+  inline uint32_t getMaxSpeed()              { return(ifSpeed);     }
+  inline bool isLoopback()                   { return(is_loopback); }
+
+  virtual void sendTermination()             { ; }
+  virtual bool read_from_pcap_dump()         { return(false); };
+  void makeTsPoint(NetworkInterfaceTsPoint *pt);
+  void tsLua(lua_State* vm);
+#ifdef HAVE_EBPF
+  inline bool iseBPFEventAvailable() { return((ebpfEvents && (ebpfEvents[next_remove_idx] != NULL)) ? true : false); }
+  bool enqueueeBPFEvent(eBPFevent *event);
+  bool dequeueeBPFEvent(eBPFevent **event);
+  void delivereBPFEvent(eBPFevent *event);
+#endif
+
+#ifdef PROFILING
+  inline void profiling_section_enter(const char *label, int id) { PROFILING_SECTION_ENTER(label, id); };
+  inline void profiling_section_exit(int id) { PROFILING_SECTION_EXIT(id); };
+#endif
 };
 
 #endif /* _NETWORK_INTERFACE_H_ */
