@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -40,9 +40,9 @@ class Ntop {
  private:
   AddressTree local_interface_addresses;
   char epoch_buf[11];
-  char working_dir[MAX_PATH]; /**< Array of working directory. */
-  char install_dir[MAX_PATH]; /**< Array of install directory. */
-  char startup_dir[MAX_PATH]; /**< Array of startup directory. */
+  char working_dir[MAX_PATH]; /**< Working directory. */
+  char install_dir[MAX_PATH]; /**< Install directory. */
+  char startup_dir[MAX_PATH]; /**< Startup directory. */
   char *custom_ndpi_protos; /**< Pointer of a custom protocol for nDPI. */
   NetworkInterface **iface; /**< Array of network interfaces. */
   u_int8_t num_defined_interfaces; /**< Number of defined interfaces. */
@@ -55,13 +55,13 @@ class Ntop {
   Logstash *logstash; /**< Pointer of Logstash. */
   ExportInterface *export_interface;
 #endif
+  TimelineExtract *extract;
   PeriodicActivities *pa; /**< Instance of periodical activities. */
   AddressResolution *address;
   Prefs *prefs;
   Geolocation *geo;
   MacManufacturers *mac_manufacturers;
   void *trackers_automa;
-  HTTPBL *httpbl;
   long time_offset;
   time_t start_time; /**< Time when start() was called */
   int udp_socket;
@@ -74,14 +74,11 @@ class Ntop {
 #endif
   FlowChecker *flow_checker;
 #endif
-#ifdef HAVE_NDB
-  Nseries *nseries[NUM_NSERIES];
-#endif
-  AddressTree *hostBlacklist, *hostBlacklistShadow;
 
   void loadLocalInterfaceAddress();
   void initAllowedProtocolPresets();
-  
+  bool checkUserPassword(const char * const user, const char * const password, char *group, bool *localuser) const;
+
  public:
   /**
    * @brief A Constructor
@@ -111,8 +108,8 @@ class Ntop {
    *
    * @param logFile A valid path to a log file
    */
-  inline void registerLogFile(const char* logFile)    { getTrace()->set_log_file(logFile);       };
-  inline void rotateLogs(bool force_rotation = false) { getTrace()->rotate_logs(force_rotation); };
+  inline void registerLogFile(const char* logFile) { getTrace()->set_log_file(logFile); };
+  inline void rotateLogs(bool mode)                { getTrace()->rotate_logs(mode);     };
 #ifdef NTOPNG_PRO
   void registerNagios(void);
   inline FlowChecker *getFlowChecker() { return(flow_checker); };
@@ -171,7 +168,10 @@ class Ntop {
   inline void getMacManufacturer(const char *mac, lua_State *vm) {
     u_int8_t mac_bytes[6];
     Utils::parseMac(mac_bytes, mac);
-    mac_manufacturers->getMacManufacturer(mac_bytes, vm);
+    if(mac_manufacturers)
+      mac_manufacturers->getMacManufacturer(mac_bytes, vm);
+    else
+      lua_pushnil(vm);
   }
 
   /**
@@ -238,13 +238,6 @@ class Ntop {
    */
   inline char* get_docs_dir()                     { return(prefs->get_docs_dir());      };
 
-  /**
-   * @brief Get httpbl.
-   *
-   * @return Current httpbl instance.
-   */
-  inline HTTPBL* get_httpbl()                        { return(httpbl);             };
-
   /**
    * @brief Register the network interface.
    * @details Check for duplicated interface and add the network interface in to @ref iface.
@@ -270,7 +263,7 @@ class Ntop {
    * @return The network interface instance if exists, NULL otherwise.
    */
   inline NetworkInterface* getInterfaceAtId(lua_State *vm, int i) const {
-    if((i < num_defined_interfaces) && iface[i]) {
+    if(i >= 0 && i < num_defined_interfaces && iface[i]) {
       return isInterfaceAllowed(vm, iface[i]->get_name()) ? iface[i] : NULL;
     }
     return NULL;
@@ -295,7 +288,7 @@ class Ntop {
    * @param name Name of network interface.
    * @return The network interface Id if exists, -1 otherwise.
    */
-  int getInterfaceIdByName(char *name);
+  int getInterfaceIdByName(lua_State *vm, const char * const name);
 
   /**
    * @brief Get the network interface with the specified Id
@@ -311,12 +304,6 @@ class Ntop {
    * @param h HTTP server instance.
    */
   inline void registerHTTPserver(HTTPserver *h)      { httpd = h;              };
-  /**
-   * @brief Set httpbl.
-   *
-   * @param h The categorization instance.
-   */
-  inline void setHTTPBL(HTTPBL *h)                   { httpbl = h; };
 
   /**
    * @brief Get the network interface identified by name or Id.
@@ -377,6 +364,7 @@ class Ntop {
   inline NtopGlobals*      getGlobals()              { return(globals); };
   inline Trace*            getTrace()                { return(globals->getTrace()); };
   inline Redis*            getRedis()                { return(redis);               };
+  inline TimelineExtract*  getTimelineExtract()      { return(extract); };
 #ifndef HAVE_NEDGE
   inline ElasticSearch*    getElasticSearch()        { return(elastic_search);      };
   inline Logstash*         getLogstash()             { return(logstash);            };
@@ -394,12 +382,16 @@ class Ntop {
 #endif
 
   void getUsers(lua_State* vm);
-  void getUserGroup(lua_State* vm);
+  bool isUserAdministrator(lua_State* vm);
   void getAllowedNetworks(lua_State* vm);
-  bool getInterfaceAllowed(lua_State* vm, char *ifname)      const;
-  bool isInterfaceAllowed(lua_State* vm, const char *ifname) const;
-  bool isInterfaceAllowed(lua_State* vm, int ifid)           const;
-  bool checkUserPassword(const char *user, const char *password);
+  bool getInterfaceAllowed(lua_State* vm, char *ifname)         const;
+  bool isInterfaceAllowed(lua_State* vm, const char *ifname)    const;
+  bool isInterfaceAllowed(lua_State* vm, int ifid)              const;
+  bool isLocalUser(lua_State* vm);
+  bool checkCaptiveUserPassword(const char * const user, const char * const password, char *group) const;
+  bool checkGuiUserPassword(struct mg_connection *conn, const char * const user, const char * const password, char *group, bool *localuser) const;
+  bool isBlacklistedLogin(struct mg_connection *conn) const;
+  bool checkUserInterfaces(const char * const user)             const;
   bool resetUserPassword(char *username, char *old_password, char *new_password);
   bool mustChangePassword(const char *user);
   bool changeUserRole(char *username, char *user_role) const;
@@ -415,19 +407,23 @@ class Ntop {
   bool isCaptivePortalUser(const char * const username);
   bool deleteUser(char *username);
   bool getUserHostPool(char *username, u_int16_t *host_pool_id);
-  bool getUserAllowedIfname(char *username, char *buf, size_t buflen);
+  bool getUserAllowedIfname(const char * const username, char *buf, size_t buflen) const;
   bool hasUserLimitedLifetime(const char * const username, int32_t *lifetime_secs);
   void setWorkingDir(char *dir);
   void fixPath(char *str, bool replaceDots = true);
   void removeTrailingSlash(char *str);
   void daemonize();
+  void shutdownPeriodicActivities();
   void shutdown();
+  void shutdownAll();
   void runHousekeepingTasks();
+  void runShutdownTasks();
   bool isLocalInterfaceAddress(int family, void *addr)       { return(local_interface_addresses.findAddress(family, addr) == -1 ? false : true);    };
   inline char* getLocalNetworkName(int16_t local_network_id) {
     return(address->get_local_network((u_int8_t)local_network_id));
   };
-  IpAddress* getLocalNetworkIp(int16_t local_network_id);
+  void getLocalNetworkIp(int16_t local_network_id, IpAddress **network_ip, u_int8_t *network_prefix);
+  inline void addLocalNetwork(const char *network)           { address->setLocalNetwork((char*)network); }
   void createExportInterface();
   void initNetworkInterfaces();
   void initElasticSearch();
@@ -445,32 +441,25 @@ class Ntop {
 
   inline void getLocalNetworks(lua_State* vm) { address->getLocalNetworks(vm);          };
   inline u_int8_t getNumLocalNetworks()       { return(address->getNumLocalNetworks()); };
-  void reloadInterfacesLuaInterpreter();
   void loadTrackers();
   bool isATrackerHost(char *host);
-  void allocHostBlacklist();
-  void swapHostBlacklist();
-  void addToHostBlacklist(char *net);
-  bool isBlacklistedIP(IpAddress *ip);
-  bool isExistingInterface(char *name);
+  bool isExistingInterface(const char * const name) const;
   inline NetworkInterface* getFirstInterface() { return(iface[0]);         }
   inline NetworkInterface* getInterface(int i) { return(((i < num_defined_interfaces) && iface[i]) ? iface[i] : NULL); }
 #ifdef NTOPNG_PRO
+  bool addToNotifiedInformativeCaptivePortal(u_int32_t client_ip);
   bool addIPToLRUMatches(u_int32_t client_ip, u_int16_t user_pool_id,
 			 char *label, int32_t lifetime_secs, char *ifname);
-#ifdef HAVE_NDB
-  inline void tsSet(u_int8_t series_id, u_int32_t tsSec, bool isCounter,
-		    u_int8_t ifaceId, u_int16_t step, const char *label,
-		    const char *key, const char *metric, u_int64_t sent, u_int64_t rcvd) {
-    nseries[series_id]->set(tsSec, isCounter, ifaceId, step, label ? label : "",
-			    key ? key : "", metric ? metric : "", sent, rcvd);
-  }
-  
-  inline void tsFlush(u_int8_t series_id) { nseries[series_id]->flush(); }
-#endif /* HAVE_NDB */
+#ifdef HAVE_EBPF
+  void deliverEventToInterfaces(eBPFevent *event);
+#endif
 #endif /* NTOPNG_PRO */
   
   DeviceProtocolBitmask* getDeviceAllowedProtocols(DeviceType t) { return(&deviceProtocolPresets[t]); }
+  void refreshAllowedProtocolPresets(DeviceType t, bool client, lua_State *L, int index);
+  DeviceProtoStatus getDeviceAllowedProtocolStatus(DeviceType dev_type, ndpi_protocol proto, u_int16_t pool_id, bool as_client);
+
+  void sendNetworkInterfacesTermination();
 };
 
 extern Ntop *ntop;
