@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,13 +30,14 @@ class Paginator {
   bool a2z_sort_order;
   bool detailed_results /* deprecated, use DetailsLevel instead */;
   char *sort_column, *country_filter, *host_filter;
-  int l7proto_filter;
+  int l7proto_filter, l7category_filter;
   u_int16_t port_filter;
   int16_t local_network_filter;
   u_int16_t vlan_id_filter;
   u_int8_t ip_version /* Either 4 or 6 */;
   int8_t unicast_traffic, unidirectional_traffic, alerted_flows, filtered_flows;
   u_int32_t asn_filter;
+  u_int32_t uid_filter, pid_filter;
   u_int32_t deviceIP;
   u_int16_t inIndex, outIndex;
   u_int16_t pool_filter;
@@ -73,6 +74,10 @@ class Paginator {
     if(l7proto_filter >= 0) { (*f) = l7proto_filter; return true; } return false;
   }
 
+  inline bool l7categoryFilter(int *f) const {
+    if(l7category_filter >= 0) { (*f) = l7category_filter; return true; } return false;
+  }
+
   inline bool portFilter(u_int16_t *f) const {
     if(port_filter) { (*f) = port_filter; return true; } return false;
   }
@@ -121,6 +126,14 @@ class Paginator {
     if(asn_filter != (u_int32_t)-1) { (*f) = asn_filter; return true; } return false;
   }
 
+  inline bool uidFilter(u_int32_t *f) const {
+    if(uid_filter != NO_UID) { (*f) = uid_filter; return true; } return false;
+  }
+
+  inline bool pidFilter(u_int32_t *f) const {
+    if(pid_filter != NO_PID) { (*f) = pid_filter; return true; } return false;
+  }
+
   inline bool unidirectionalTraffic(bool *f) const {
     if(unidirectional_traffic != -1) { (*f) = (unidirectional_traffic==1) ? true : false; return true; } return false;
   }
