@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,27 +30,32 @@ class Redis {
  private:
   redisContext *redis;
   Mutex *l;
-  char *redis_host;
-  char *redis_password;
-  u_int32_t num_requests, num_reconnections;
+  char *redis_host, *redis_password, *redis_version;
+#ifdef __linux__
+  bool is_socket_connection;
+#endif
+  u_int32_t num_requests, num_reconnections, num_redis_version;
   u_int16_t redis_port;
   u_int8_t redis_db_id;
   pthread_t esThreadLoop;
   pthread_t lsThreadLoop;
   bool operational;
+  bool initializationCompleted;
   StringCache_t *stringCache;
   u_int numCached;
-  
+
+  char* getRedisVersion();
   void reconnectRedis();
-  int msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors = true);
-  int oneOperator(const char *operation, char *key);
-  int twoOperators(const char *operation, char *op1, char *op2);
+  int msg_push(const char * const cmd, const char * const queue_name, const char * const msg, u_int queue_trim_size,
+	       bool trace_errors = true, bool head_trim = true);
   int pushHost(const char* ns_cache, const char* ns_list, char *hostname,
 	       bool dont_check_for_existence, bool localHost);
   int popHost(const char* ns_list, char *hostname, u_int hostname_len);
-  void addToCache(char *key, char *value, u_int expire_secs);
-  bool isCacheable(char *key);
+  void addToCache(const char * const key, const char * const value, u_int expire_secs);
+  bool isCacheable(const char * const key);
   bool expireCache(char *key, u_int expire_sec);
+
+  void checkDumpable(const char * const key);
 	  
  public:
   Redis(const char *redis_host = (char*)"127.0.0.1",
@@ -58,69 +63,49 @@ class Redis {
 	u_int16_t redis_port = 6379, u_int8_t _redis_db_id = 0);
   ~Redis();
 
-  char* getVersion(char *str, u_int str_len);
+  inline char* getVersion()        { return(redis_version);     }
+  inline u_int32_t getNumVersion() { return(num_redis_version); }
+  inline bool haveRedisDump()      { return((num_redis_version >= 0x020600) ? true : false); }
   void setDefaults();
   inline bool isOperational() { return(operational); };
+  inline void setInitializationComplete() { initializationCompleted = true; };
   int expire(char *key, u_int expire_sec);
   int get(char *key, char *rsp, u_int rsp_len, bool cache_it = false);
-  int hashGet(char *key, char *member, char *rsp, u_int rsp_len);
-  int hashDel(char *key, char *field);
-  int hashSet(char *key, char *field, char *value);
+  int hashGet(const char * const key, const char * const member, char * const rsp, u_int rsp_len);
+  int hashDel(const char * const key, const char * const field);
+  int hashSet(const char * const key, const char * const field, const char * const value);
   int delHash(char *key, char *member);
-  int set(char *key, char *value, u_int expire_secs=0);
-  char* popSet(char *pop_name, char *rsp, u_int rsp_len);
+  int set(const char * const key, const char * const value, u_int expire_secs=0);
   int keys(const char *pattern, char ***keys_p);
   int hashKeys(const char *pattern, char ***keys_p);
   int hashGetAll(const char *key, char ***keys_p, char ***values_p);
   int del(char *key);
-  int zIncr(char *key, char *member);
-  int zTrim(char *key, u_int trim_len);
-  int zRevRange(const char *pattern, char ***keys_p);
   int pushHostToResolve(char *hostname, bool dont_check_for_existence, bool localHost);
   int popHostToResolve(char *hostname, u_int hostname_len);
 
-  int pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existence, bool localHost);
-  int popHostToTrafficFiltering(char *hostname, u_int hostname_len);
-
-  char* getTrafficFilteringCategory(char *numeric_ip, char *buf, u_int buf_len, bool query_httpbl_if_unknown);
-  int popDomainToCategorize(char *domainname, u_int domainname_len);
-
   int getAddress(char *numeric_ip, char *rsp, u_int rsp_len, bool queue_if_not_found);
-  int getAddressTrafficFiltering(char *numeric_ip, NetworkInterface *iface,
-		       char *rsp, u_int rsp_len, bool queue_if_not_found);
   int setResolvedAddress(char *numeric_ip, char *symbolic_ip);
-  int setTrafficFilteringAddress(char* numeric_ip, char* httpbl);
-
-  int hashIncr(char *key, char *field, u_int32_t value);
-
-  int addHostToDBDump(NetworkInterface *iface, IpAddress *ip, char *name);
 
   int sadd(const char *set_name, char *item);
   int srem(const char *set_name, char *item);
   int smembers(lua_State* vm, char *setName);
   int smembers(const char *set_name, char ***members);
 
-  void setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_int32_t id);
-  u_int32_t host_to_id(NetworkInterface *iface, char *daybuf, char *host_name, bool *new_key);
-  int id_to_host(char *daybuf, char *host_idx, char *buf, u_int buf_len);
-  int lpush(const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors = true);
-  int rpush(const char *queue_name, char *msg, u_int queue_trim_size);
+  int lpush(const char * const queue_name, const char * const msg, u_int queue_trim_size, bool trace_errors = true);
+  int rpush(const char * const queue_name, const char * const msg, u_int queue_trim_size);
   int lindex(const char *queue_name, int idx, char *buf, u_int buf_len);
+  int ltrim(const char *queue_name, int start_idx, int end_idx);
+  u_int hstrlen(const char * const key, const char * const value);
+  u_int len(const char * const key);
   u_int llen(const char *queue_name);
   int lset(const char *queue_name, u_int32_t idx, const char *value);
   int lrem(const char *queue_name, const char *value);
   int lrange(const char *list_name, char ***elements, int start_offset, int end_offset);
   int lpop(const char *queue_name, char *buf, u_int buf_len);
   int lpop(const char *queue_name, char ***elements, u_int num_elements);
+  u_int incr(const char *key);
   int flushDb();
   void flushCache();
-  /**
-   * @brief Increment a redis key and return its new value
-   *
-   * @param key The key whose value will be incremented.
-   */
-  u_int32_t incrKey(char *key);
-  int rename(char *oldk, char *newk) { return(twoOperators("RENAME", oldk, newk)); };
   void lua(lua_State *vm);
   char* dump(char *key);
   int restore(char *key, char *buf);
