@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -18,6 +18,7 @@
  * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  */
+#ifndef HAVE_NEDGE
 
 #ifndef _SNMP_H_
 #define _SNMP_H_
@@ -31,7 +32,8 @@
 class SNMP {
  private:
   int udp_sock;
-
+  u_int snmp_version;
+  
   int _get(char *agentIP, char *community, char *oid, u_int8_t snmp_version);
   int _getnext(char *agentIP, char *community, char *oid, u_int8_t snmp_version);
   int snmp_get_fctn(lua_State* vm, bool isGetNext);  
@@ -50,3 +52,5 @@ class SNMP {
 };
   
 #endif /* _SNMP_H_ */
+
+#endif /* HAVE_NEDGE */
