@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,7 +37,7 @@ class StoreManager {
 		 void *payload);
  public:
   StoreManager(int interface_id);
-  ~StoreManager();
+  virtual ~StoreManager();
 
   NetworkInterface* getNetworkInterface();
   
