@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2016-17 - ntop.org
+ * (C) 2016-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,7 +24,7 @@
 
 #include "ntop_includes.h"
 
-#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__))
+#if defined(HAVE_PF_RING) && (!defined(NTOPNG_EMBEDDED_EDITION))
 
 class ZCCollectorInterface : public ParserInterface {
  private:
