@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -67,11 +67,12 @@ const char *win_inet_ntop(int af, const void *src, char *dst,socklen_t size);
 #define inet_ntop win_inet_ntop
 
 #define realpath(N,R) _fullpath((R),(N),_MAX_PATH)
+#define PCAP_NETMASK_UNKNOWN	0xffffffff
 
 typedef HANDLE pthread_mutex_t;
 typedef struct { HANDLE signal, broadcast; } pthread_cond_t;
 typedef HANDLE pthread_t;
-
+typedef int mode_t;
 
 #ifdef __cplusplus
 extern "C" {
@@ -184,6 +185,11 @@ struct win_in6_addr
 
 #define in6_addr win_in6_addr
 
+#if defined(_MSC_VER)
+#include <BaseTsd.h>
+#define SSIZE_T ssize_t
+#endif
+
 struct ip6_hdr
 {
   union
