@@ -7,12 +7,11 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 require "alert_utils"
-require "rrd_utils"
 local callback_utils = require "callback_utils"
 
 if (ntop.isPro()) then
   package.path = dirs.installdir .. "/pro/scripts/callbacks/interface/?.lua;" .. package.path
-  pcall(require, 'hourly')
+  require('hourly')
 end
 
 -- ########################################################
