@@ -1,14 +1,17 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
+local ts_utils = require("ts_utils")
+local info = ntop.getInfo() 
+local page_utils = require("page_utils")
 
 sendHTTPContentTypeHeader('text/html')
 
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
+page_utils.print_header(i18n("about.about_x", { product=info.product }))
 
 active_page = "about"
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
@@ -19,11 +22,11 @@ if(_POST["ntopng_license"] ~= nil) then
 end
 
 info = ntop.getInfo()
-print("<hr /><h2>"..i18n("about.about").." "..info["product"].."</h2>")
+print("<hr /><h2>"..i18n("about.about_x", {product=info["product"]}).."</h2>")
 
 print("<table class=\"table table-bordered table-striped\">\n")
-print("<tr><th>Copyright</th><td>"..info["copyright"].."</td></tr>\n")
-print("<tr><th>License</th><td>")
+print("<tr><th>") print(i18n("about.copyright")) print("</th><td>"..info["copyright"].."</td></tr>\n")
+print("<tr><th>") print(i18n("about.licence")) print("</th><td>")
 
 info["ntopng.license"] = ntop.getCache('ntopng.license')
 if(info["pro.release"] == false) then
@@ -119,17 +122,53 @@ if (ndpi_ver ~= nil) then
   end
 end
 
+print("<tr><th><a href=\"https://curl.haxx.se\" target=\"_blank\">cURL</A></th><td>"..info["version.curl"].."</td></tr>\n")
+
 print("<tr><th><a href=\"http://twitter.github.io/\" target=\"_blank\"><i class=\'fa fa-twitter fa-lg'></i> Twitter Bootstrap</A></th><td>3.x</td></tr>\n")
 print("<tr><th><a href=\"http://fortawesome.github.io/Font-Awesome/\" target=\"_blank\"><i class=\'fa fa-flag fa-lg'></i> Font Awesome</A></th><td>4.x</td></tr>\n")
 print("<tr><th><a href=\"http://www.rrdtool.org/\" target=\"_blank\">RRDtool</A></th><td>"..info["version.rrd"].."</td></tr>\n")
+
+if(info["version.nindex"] ~= nil) then
+   print("<tr><th>nIndex</th><td>"..info["version.nindex"].."</td></tr>\n")
+end
+
+local l7_resolution = "5m"
+
+if ts_utils.getDriverName() == "influxdb" then
+   print("<tr><th><a href=\"http://www.influxdata.com\" target=\"_blank\">InfluxDB</A></th><td><img id=\"influxdb-info-load\" border=0 src=".. ntop.getHttpPrefix() .. "/img/throbber.gif style=\"vertical-align:text-top;\" id=throbber><span id=\"influxdb-info-text\"></span></td></tr>\n")
+   print[[<script>
+$(function() {
+   $.get("]] print(ntop.getHttpPrefix()) print[[/lua/get_influxdb_info.lua", function(info) {
+      $("#influxdb-info-load").hide();
+      $("#influxdb-info-text").html(info.version + " ");
+   }).fail(function() {
+      $("#influxdb-info-load").hide();
+   });
+});
+</script>
+]]
+
+   -- NOTE: need to calculate this dynamically as it can be temporary disabled at runtime
+   local steps = tonumber(ntop.getPref("ntopng.prefs.ts_write_steps"))
+
+   if steps and steps > 0 then
+      l7_resolution = (steps * 5) .. "s"
+   else
+      l7_resolution = "1m"
+   end
+end
+
+print("<tr><th>".. i18n("prefs.timeseries_resolution_resolution_title") .."</th><td>"..l7_resolution.."</td></tr>\n")
 print("<tr><th><a href=\"http://www.redis.io\" target=\"_blank\">Redis</A> Server</th><td>"..info["version.redis"].."</td></tr>\n")
 print("<tr><th><a href=\"https://github.com/valenok/mongoose\" target=\"_blank\">Mongoose web server</A></th><td>"..info["version.httpd"].."</td></tr>\n")
 print("<tr><th><a href=\"http://www.luajit.org\" target=\"_blank\">LuaJIT</A></th><td>"..info["version.luajit"].."</td></tr>\n")
-print("<tr><th><a href=\"http://www.zeromq.org\" target=\"_blank\">ØMQ</A></th><td>"..info["version.zmq"].."</td></tr>\n")
+if info["version.zmq"] ~= nil then
+   print("<tr><th><a href=\"http://www.zeromq.org\" target=\"_blank\">ØMQ</A></th><td>"..info["version.zmq"].."</td></tr>\n")
+end
 if(info["version.geoip"] ~= nil) then
-print("<tr><th><a href=\"http://www.maxmind.com\" target=\"_blank\">GeoIP</A></th><td>"..info["version.geoip"])
+print("<tr><th><a href=\"http://www.maxmind.com\" target=\"_blank\">GeoLite</A></th><td>"..info["version.geoip"])
 
-print [[ <br>&nbsp;<br><small>]] print(i18n("about.maxmind", {maxmind_url="http://www.maxmind.com/"})) print[[</small>
+print [[ <br><small>]] print(i18n("about.maxmind", {maxmind_url="http://www.maxmind.com/"})) print[[</small>
 ]]
 
 print("</td></tr>\n")
