@@ -1,19 +1,19 @@
 --
--- (C) 2017 - ntop.org
+-- (C) 2017-18 - ntop.org
 --
 
-dirs = ntop.getDirs()
+local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
-json = require("dkjson")
+local json = require("dkjson")
 
 sendHTTPHeader('application/json')
 
 local latest_version = ntop.getCache("ntopng.cache.version")
 
 if isEmptyString(latest_version) then
-  local rsp = ntop.httpGet("http://www.ntop.org/ntopng.version", "", "", 10 --[[ seconds ]])
+  local rsp = ntop.httpGet("https://www.ntop.org/ntopng.version", "", "", 10 --[[ seconds ]])
 
   if(not isEmptyString(rsp)) and (not isEmptyString(rsp["CONTENT"])) then
      latest_version = trimSpace(string.gsub(rsp["CONTENT"], "\n", ""))
