@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -61,9 +61,11 @@ else
     if(flow["proto.l4"] == "TCP") then
        print(', "c2sOOO":'.. flow["cli2srv.out_of_order"] )
        print(', "c2slost":'..flow["cli2srv.lost"] )
+       print(', "c2skeep_alive":'..flow["cli2srv.keep_alive"] )
        print(', "c2sretr":'..flow["cli2srv.retransmissions"] )
        print(', "s2cOOO":'.. flow["srv2cli.out_of_order"] )
        print(', "s2clost":'..flow["srv2cli.lost"] )
+       print(', "s2ckeep_alive":'..flow["srv2cli.keep_alive"] )
        print(', "s2cretr":'..flow["srv2cli.retransmissions"] )
     end
 
@@ -78,7 +80,7 @@ else
       proc = flow.client_process
       print ('"'..proc.pid..'": {')
 
-      if(proc.actual_memory > 0) then
+      if proc.actual_memory and proc.actual_memory > 0 then
         -- average_cpu_load
         print('"average_cpu_load": "')
 	load = round(proc.average_cpu_load, 2)
@@ -111,7 +113,7 @@ else
       proc = flow.server_process
       print ('"'..proc.pid..'": {')
 
-      if(proc.actual_memory > 0) then
+      if((proc.actual_memory ~= nil) and (proc.actual_memory > 0)) then
         -- average_cpu_load
         load = round(proc.average_cpu_load, 2)
         print('"average_cpu_load": "')
