@@ -1,14 +1,16 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
-dirs = ntop.getDirs()
+local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 require "flow_utils"
 local json = require "dkjson"
 
+local have_nedge = ntop.isnEdge()
+
 sendHTTPContentTypeHeader('text/html')
 local debug = false
 local debug_process = false -- Show flow processed information
@@ -25,8 +27,11 @@ local sortOrder   = _GET["sortOrder"]
 local host_info   = url2hostinfo(_GET)
 local port        = _GET["port"]
 local application = _GET["application"]
+local category    = _GET["category"]
 local network_id  = _GET["network"]
 local vlan        = _GET["vlan"]
+local uid         = _GET["uid"]
+local pid         = _GET["pid"]
 
 local deviceIP    = _GET["deviceIP"]
 local inIfIdx     = _GET["inIfIdx"]
@@ -104,6 +109,10 @@ if application ~= nil and application ~= "" then
    pageinfo["l7protoFilter"] = interface.getnDPIProtoId(application)
 end
 
+if category ~= nil and category ~= "" then
+   pageinfo["l7categoryFilter"] = interface.getnDPICategoryId(category)
+end
+
 if not isEmptyString(flowhosts_type) then
    if flowhosts_type == "local_origin_remote_target" then
       pageinfo["clientMode"] = "local"
@@ -151,6 +160,14 @@ if not isEmptyString(vlan) then
    pageinfo["vlanIdFilter"] = tonumber(vlan)
 end
 
+if not isEmptyString(uid) then
+   pageinfo["uidFilter"] = tonumber(uid)
+end
+
+if not isEmptyString(pid) then
+   pageinfo["pidFilter"] = tonumber(pid)
+end
+
 if not isEmptyString(deviceIP) then
    pageinfo["deviceIpFilter"] = deviceIP
 
@@ -186,15 +203,28 @@ if(flows_stats == nil) then flows_stats = { } end
 
 for key, value in ipairs(flows_stats) do
    local info = ""
-   if(flows_stats[key]["info"] ~= nil) then
-      info = shortenString(flows_stats[key]["info"])
-   elseif(flows_stats[key]["icmp"] ~= nil) then
+   -- use an italic font to indicate extra information added after sorting
+   local italic = true
+   if(not isEmptyString(flows_stats[key]["info"])) then
+      info = flows_stats[key]["info"]
+      italic = false
+   elseif(not isEmptyString(flows_stats[key]["icmp"])) then
       info = getICMPTypeCode(flows_stats[key]["icmp"])
    elseif(flows_stats[key]["proto.ndpi"] == "SIP") then
       info = getSIPInfo(flows_stats[key])
    elseif(flows_stats[key]["proto.ndpi"] == "RTP") then
       info = getRTPInfo(flows_stats[key])
    end
+
+   -- safety checks against injections
+   info = noHtml(info) 
+   info = info:gsub('"', '')
+
+   if italic then
+      info = string.format("<i>%s</i>", info)
+   end
+   
+   info = shortenString(info)
    flows_stats[key]["info"] = info
 
    if(flows_stats[key]["profile"] ~= nil) then
@@ -211,6 +241,9 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
    local srv_name = flowinfo2hostname(value, "srv")
    local cli_name = flowinfo2hostname(value, "cli")
 
+   local src_port, dst_port = '', ''
+   local src_process, dst_process = '', ''
+
    if(cli_name == nil) then cli_name = "???" end
    if(srv_name == nil) then srv_name = "???" end
 
@@ -229,23 +262,15 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
       src_key="<A HREF='"..ntop.getHttpPrefix().."/lua/host_details.lua?" .. hostinfo2url(value,"cli").. "' data-toggle='tooltip' title='" ..cli_tooltip.. "' >".. shortenString(stripVlan(cli_name))
       if(value["cli.systemhost"] == true) then src_key = src_key .. "&nbsp;<i class='fa fa-flag'></i>" end
 
-      -- Flow username
-      local i, j
-      if(value["moreinfo.json"] ~= nil) then
-	 i, j = string.find(value["moreinfo.json"], '"57593":')
-      end
-      if(i ~= nil) then
-	 has_user = string.sub(value["moreinfo.json"], j+2, j+3)
-	 if(has_user == '""') then has_user = nil end
-      end
-      if(has_user ~= nil) then src_key = src_key .. " <i class='fa fa-user'></i>" end
       src_key = src_key .. "</A>"
 
       if(value["cli.port"] > 0) then
-	 src_port=":<A HREF='"..ntop.getHttpPrefix().."/lua/port_details.lua?port=" .. value["cli.port"] .. "'>"..ntop.getservbyport(value["cli.port"], string.lower(value["proto.l4"])).."</A>"
+	 src_port="<A HREF='"..ntop.getHttpPrefix().."/lua/port_details.lua?port=" .. value["cli.port"] .. "'>"..ntop.getservbyport(value["cli.port"], string.lower(value["proto.l4"])).."</A>"
       else
 	 src_port=""
       end
+
+      src_process = flowinfo2process(value["client_process"], hostinfo2url(value,"cli"))
    else
       src_key = shortenString(stripVlan(cli_name))
       src_port=":"..value["cli.port"]
@@ -257,19 +282,29 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
       dst_key = dst_key .. "</A>"
 
       if(value["srv.port"] > 0) then
-	 dst_port=":<A HREF='"..ntop.getHttpPrefix().."/lua/port_details.lua?port=" .. value["srv.port"] .. "'>"..ntop.getservbyport(value["srv.port"], string.lower(value["proto.l4"])).."</A>"
+	 dst_port="<A HREF='"..ntop.getHttpPrefix().."/lua/port_details.lua?port=" .. value["srv.port"] .. "'>"..ntop.getservbyport(value["srv.port"], string.lower(value["proto.l4"])).."</A>"
       else
 	 dst_port=""
       end
+
+      dst_process = flowinfo2process(value["server_process"], hostinfo2url(value,"srv"))
    else
       dst_key = shortenString(stripVlan(srv_name))
       dst_port=":"..value["srv.port"]
    end
 
-   record["column_key"] = "<A HREF='"
+   local column_key = "<A HREF='"
       ..ntop.getHttpPrefix().."/lua/flow_details.lua?flow_key="
       ..value["ntopng.key"]
       .."'><span class='label label-info'>Info</span></A>"
+   if(have_nedge) then
+     if (value["verdict.pass"]) then
+       column_key = column_key.." <span title='"..i18n("flow_details.drop_flow_traffic_btn").."' class='label label-default block-badge' "..(ternary(isAdministrator(), "onclick='block_flow("..value["ntopng.key"]..");' style='cursor: pointer;'", "")).."><i class='fa fa-ban' /></span>"
+     else
+       column_key = column_key.." <span title='"..i18n("flow_details.flow_traffic_is_dropped").."' class='label label-danger block-badge'><i class='fa fa-ban' /></span>"
+     end
+   end
+   record["column_key"] = column_key
    record["key"] = value["ntopng.key"]
 
    local column_client = src_key
@@ -279,7 +314,15 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
       column_client = column_client..getFlag(info["country"])
    end
 
-   column_client = column_client..src_port
+   column_client = string.format("%s%s%s %s",
+				 column_client,
+				 ternary(src_port ~= '', ':', ''),
+				 src_port,
+				 src_process)
+   if(value["verdict.pass"] == false) then
+     column_client = "<strike>"..column_client.."</strike>"
+   end
+
    record["column_client"] = column_client
 
    local column_server = dst_key
@@ -288,7 +331,14 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
       column_server = column_server..getFlag(info["country"])
    end
 
-   column_server = column_server..dst_port
+   column_server = string.format("%s%s%s %s",
+				 column_server,
+				 ternary(dst_port ~= '', ':', ''),
+				 dst_port,
+				 dst_process)
+   if(value["verdict.pass"] == false) then
+     column_server = "<strike>"..column_server.."</strike>"
+   end
    record["column_server"] = column_server
 
    record["column_vlan"] = ''
@@ -301,7 +351,7 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
    if(interface.isPacketInterface()) then
       if(value["flow.status"] ~= 0) then
 	 column_proto_l4 = "<i class='fa fa-warning fa-lg' style='color: orange;'"
-	    .." title='"..string.gsub(getFlowStatus(value["flow.status"]), "<[^>]*>([^<]+)<.*", "%1")
+	    .." title='"..string.gsub(getFlowStatus(value["flow.status"], flow2statusinfo(value)), "<[^>]*>([^<]+)<.*", "%1")
 	    .."'></i> "
       end
    end
@@ -328,6 +378,9 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
    else
       column_proto_l4 = column_proto_l4..value["proto.l4"]
    end
+   if(value["verdict.pass"] == false) then
+     column_proto_l4 = "<strike>"..column_proto_l4.."</strike>"
+   end
    record["column_proto_l4"] = column_proto_l4
 
    local app = getApplicationLabel(value["proto.ndpi"])
@@ -340,14 +393,14 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
    record["column_bytes"] = bytesToSize(value["bytes"])..""
 
    local column_thpt = ''
+   if(throughput_type == "pps") then
+      column_thpt = column_thpt..pktsToSize(value["throughput_pps"]).. " "
+   else
+      column_thpt = column_thpt..bitsToSize(8*value["throughput_bps"]).. " "
+   end
+
    if((value["throughput_trend_"..throughput_type] ~= nil)
       and (value["throughput_trend_"..throughput_type] > 0)) then
-      if(throughput_type == "pps") then
-	 column_thpt = column_thpt..pktsToSize(value["throughput_pps"]).. " "
-      else
-	 column_thpt = column_thpt..bitsToSize(8*value["throughput_bps"]).. " "
-      end
-
       if(value["throughput_trend_"..throughput_type] == 1) then
 	 column_thpt = column_thpt.."<i class='fa fa-arrow-up'></i>"
       elseif(value["throughput_trend_"..throughput_type] == 2) then
@@ -355,16 +408,12 @@ for _key, value in ipairs(flows_stats) do -- pairsByValues(vals, funct) do
       elseif(value["throughput_trend_"..throughput_type] == 3) then
 	 column_thpt = column_thpt.."<i class='fa fa-minus'></i>"
       end
-
-   else
-      column_thpt = "0 "..throughput_type
    end
    record["column_thpt"] = column_thpt
 
    local cli2srv = round((value["cli2srv.bytes"] * 100) / value["bytes"], 0)
 
    record["column_breakdown"] = "<div class='progress'><div class='progress-bar progress-bar-warning' style='width: " .. cli2srv .."%;'>Client</div><div class='progress-bar progress-bar-info' style='width: " .. (100-cli2srv) .. "%;'>Server</div></div>"
-   record["column_info"] = "ciao"
 
    local info = value["info"]
 
