@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -17,7 +17,7 @@ local host_info = url2hostinfo(_GET)
 
 interface.select(ifname)
 
-local host = interface.getMacInfo(host_info["host"], host_info["vlan"])
+local host = interface.getMacInfo(host_info["host"])
 
 local res = {}
 if host ~= nil then
