@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -21,7 +21,7 @@ local vlan = interface.getVLANInfo(vlan_id)
 
 local res = {}
 if vlan ~= nil then
-   res = vlan2record(vlan)
+   res = vlan2record(getInterfaceId(ifname), vlan)
 end
 
 print(json.encode(res, nil))
