@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -7,6 +7,8 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
 local json = require ("dkjson")
+local page_utils = require("page_utils")
+local tracker = require("tracker")
 
 -- whether to return host statistics: on by default
 local host_stats           = _GET["host_stats"]
@@ -20,7 +22,7 @@ host = _GET["host"]
 
 if(host_info["host"] == nil) then
    sendHTTPContentTypeHeader('text/html')
-   ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
+   page_utils.print_header()
    dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
    print("<div class=\"alert alert-danger\"><img src=".. ntop.getHttpPrefix() .. "/img/warning.png> Host parameter is missing (internal error ?)</div>")
    return
@@ -70,7 +72,7 @@ host = interface.getHostInfo(host_info["host"], host_info["vlan"])
 
 if(host == nil) then
    sendHTTPContentTypeHeader('text/html')
-   ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
+   page_utils.print_header()
    dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
    print("<div class=\"alert alert-danger\"><img src=".. ntop.getHttpPrefix() .. "/img/warning.png> Host ".. host_info["host"] .. " Vlan" ..host_info["vlan"].." cannot be found (expired ?)</div>")
    return
@@ -122,5 +124,8 @@ else
       hj["flows_count"] = total
    end
    print(json.encode(hj, nil))
+
+   -- TRACKER HOOK
+   tracker.log("host_get_json", {host_info["host"], host_info["vlan"]})
 end
 
