@@ -8,6 +8,8 @@ package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 require "lua_utils"
 require "graph_utils"
 
+local page_utils = require("page_utils")
+
 page        = _GET["page"]
 hosts_ip     = _GET["hosts"]
 
@@ -23,16 +25,16 @@ show_aggregation = true
 
 active_page = "hosts"
 sendHTTPContentTypeHeader('text/html')
-ntop.dumpFile(dirs.installdir .. "/httpdocs/inc/header.inc")
-dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
+page_utils.print_header()
+
+dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
 if(hosts_ip == nil) then
    print("<div class=\"alert alert-danger\"><img src=".. ntop.getHttpPrefix() .. "/img/warning.png> "..i18n("hosts_comparison.hosts_parameter_missing_message").."</div>")
    return
 end
 
-
 print [[
   <nav class="navbar navbar-default" role="navigation">
   <div class="navbar-collapse collapse">
