@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 require "os"
@@ -9,6 +9,8 @@ print [[
       <p id="ntopng_update_available"></p>
    ]]
 
+local template = require "template_utils"
+local have_nedge = ntop.isnEdge()
 info = ntop.getInfo(true)
 
 print [[
@@ -26,14 +28,49 @@ printntopngRelease(info)
 
 print(" v."..info["version"])
 
-print("</br>User ")
-print('<a href="'..ntop.getHttpPrefix()..'/lua/admin/users.lua"><span class="label label-primary">'.._SESSION["user"].. '</span></a> Interface <a href="'..ntop.getHttpPrefix()..'/lua/if_stats.lua"><span class="label label-primary">')
+print("</br> ") print(i18n("please_wait_page.user")) print(" ")
+print('<a href="'..ntop.getHttpPrefix()..'/lua/admin/users.lua"><span class="label label-primary">'.._SESSION["user"].. '</span></a> ' .. i18n("interface") .. ' <a href="'..ntop.getHttpPrefix()..'/lua/if_stats.lua"><span class="label label-primary" title="'..ifname..'">')
 
-local alias = getInterfaceNameAlias(ifname)
+local alias = getHumanReadableInterfaceName(ifname)
 print(alias)
 
 print('</span></a>')
 
+if have_nedge then
+  print[[<form id="powerOffForm" method="post">
+    <input name="csrf" value="]] print(ntop.getRandomCSRFValue()) print[[" type="hidden" />
+    <input name="poweroff" value="" type="hidden" />
+  </form>
+  <form id="rebootForm" method="post">
+    <input name="csrf" value="]] print(ntop.getRandomCSRFValue()) print[[" type="hidden" />
+    <input name="reboot" value="" type="hidden" />
+  </form>]]
+
+  print(
+    template.gen("modal_confirm_dialog.html", {
+      dialog={
+        id      = "poweroff_dialog",
+        action  = "$('#powerOffForm').submit()",
+        title   = i18n("nedge.power_off"),
+        message = i18n("nedge.power_off_confirm"),
+        confirm = i18n("nedge.power_off"),
+      }
+    })
+  )
+
+  print(
+    template.gen("modal_confirm_dialog.html", {
+      dialog={
+        id      = "reboot_dialog",
+        action  = "$('#rebootForm').submit()",
+        title   = i18n("nedge.reboot"),
+        message = i18n("nedge.reboot_corfirm"),
+        confirm = i18n("nedge.reboot"),
+      }
+    })
+  )
+end
+
 if(info["pro.systemid"] and (info["pro.systemid"] ~= "")) then
    local do_show = false
 
@@ -42,17 +79,17 @@ if(info["pro.systemid"] and (info["pro.systemid"] ~= "")) then
       if(info["pro.demo_ends_at"] ~= nil) then
 	 local rest = info["pro.demo_ends_at"] - os.time()
 	 if(rest > 0) then
-	    print(' License expires in '.. secondsToTime(rest) ..'')
+	    print(" " .. i18n("about.licence_expires_in", {time=secondsToTime(rest)}))
 	 end
       end
    else
-      print('Upgrade to Professional version')
+      print(i18n("about.upgrade_to_professional"))
       do_show = true
    end
    print('</span></A>')
 
    if(info["pro.out_of_maintenance"] == true) then
-      print('<span class="badge badge-error">ntopng maintenance is expired</span>')
+      print('<span class="badge badge-error">') print(i18n("about.maintenance_expired", {product=info["product"]})) print('</span>')
    end
    
    if(do_show) then
@@ -65,12 +102,17 @@ print [[</font>
 </div> <!-- End column 1 -->
 	<div class="col-xs-4 v col-sm-4">
 	<div class="row">
-	 <div class="col-xs-6 col-sm-6">
 ]]
 
-if interface.isPcapDumpInterface() == false then
+if not have_nedge then
+  print[[	 <div class="col-xs-6 col-sm-6"> ]]
+else
+  print[[	 <div class="col-md-12"> ]]
+end
+
+if (interface.isPcapDumpInterface() == false) and (not have_nedge) then
    if(ifname ~= nil) then
-     maxSpeed = getInterfaceSpeed(_ifstats)
+     maxSpeed = getInterfaceSpeed(_ifstats.id)
    end
    -- io.write(maxSpeed)
    if((maxSpeed == "") or (maxSpeed == nil)) then
@@ -85,6 +127,7 @@ if interface.isPcapDumpInterface() == false then
       -- use the user-specified custom value for the speed
       maxSpeed = tonumber(maxSpeed)*1000000
    end
+
    addGauge('networkload', ntop.getHttpPrefix()..'/lua/if_stats.lua?ifid='..getInterfaceId(ifname).."&page=config", 100, 100, 50)
    print [[ <div class="text-center" title="All traffic detected by NTOP: Local2Local, Remote2Local, Local2Remote" id="gauge_text_allTraffic"></div> ]]
 
@@ -105,6 +148,19 @@ if interface.isPcapDumpInterface() == false then
 
 end -- closes interface.isPcapDumpInterface() == false 
 
+if have_nedge then
+   print [[  <a href="]]
+   print (ntop.getHttpPrefix())
+   print [[/lua/if_stats.lua">
+	    <table style="border-collapse:collapse; !important">
+	    <tr><td title="Upload"><i class="fa fa-cloud-upload"></i>&nbsp;</td><td class="network-load-chart-local2remote">0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0</td><td class="text-right" id="chart-local2remote-text"></td></tr>
+	    <tr><td title="Download"><i class="fa fa-cloud-download"></i>&nbsp;</td><td class="network-load-chart-remote2local">0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0</td><td class="text-right" id="chart-remote2local-text"></td></tr>
+	    </table>
+	    </div>
+	    <div class="col-xs-6 col-sm-4">
+	    </a>]]
+end
+
 print [[
       </div>
     </div>
@@ -115,33 +171,17 @@ print [[
     <div id="network-load">
   </div> <!-- End column 3 -->
 </div>
-</div>]]
-
--- Bridge wizard check
-local show_bridge_dialog = false
-
-if isAdministrator()
- and isBridgeInterface(_ifstats)
- and ntop.isEnterprise()
- and ((ntop.getCache(getBridgeInitializedKey(_ifstats.id)) ~= "1") or (_POST["show_wizard"] ~= nil)) then
-  show_bridge_dialog = true
-  dofile(dirs.installdir .. "/scripts/lua/inc/bridge_wizard.lua")
-end
-
-print[[<script>
-// Updating charts.
+</div>
+<script>
 ]]
 
-if show_bridge_dialog then
-  print("$('#bridgeWizardModal').modal();")
-  ntop.setCache(getBridgeInitializedKey(_ifstats.id), "1")
-end
+local traffic_peity_width = ternary(have_nedge, "140", "64")
 
 print('var is_historical = false;')
 print [[
 
-var updatingChart_local2remote = $(".network-load-chart-local2remote").peity("line", { width: 64, max: null });
-var updatingChart_remote2local = $(".network-load-chart-remote2local").peity("line", { width: 64, max: null, fill: "lightgreen"});
+var updatingChart_local2remote = $(".network-load-chart-local2remote").peity("line", { width: ]] print(traffic_peity_width) print[[, max: null });
+var updatingChart_remote2local = $(".network-load-chart-remote2local").peity("line", { width: ]] print(traffic_peity_width) print[[, max: null, fill: "lightgreen"});
 
 var prev_bytes   = 0;
 var prev_packets = 0;
@@ -165,13 +205,17 @@ print [[/lua/logout.lua");  }, */
 	  success: function(rsp) {
     
 	  try {
-            if(rsp.a_to_b_out_pkts) {
-              /* This is a bridge, charts must show actually bridged traffic
-                 and not the total ingress traffic. That is,
-                 the sum of out packets/bytes on each interface */
-              rsp.bytes   = rsp.a_to_b_out_bytes + rsp.b_to_a_out_bytes;
-              rsp.packets = rsp.a_to_b_out_pkts + rsp.b_to_a_out_pkts;
-            }
+]]
+
+if have_nedge then
+  -- Use bytes up / down on edge
+  print[[
+            rsp.local2remote = rsp.bytes_upload;
+            rsp.remote2local = rsp.bytes_download;
+  ]]
+end
+
+print[[
 
 	    if (prev_bytes > 0) {
 	      if (rsp.packets < prev_packets) {
@@ -216,7 +260,7 @@ print [[/lua/logout.lua");  }, */
                 }
 ]]
 
-   if interface.isPcapDumpInterface() == false then
+   if (interface.isPcapDumpInterface() == false) and (not have_nedge) then
       print[[
 
 		$('#gauge_text_allTraffic').html("<small>"+bitsToSize(Math.min(bps, ]] print(maxSpeed) print[[), 1000) + " [" + addCommas(pps) + " pps]</small>");
@@ -227,14 +271,18 @@ print [[/lua/logout.lua");  }, */
 		$('#networkload').html(v+"%");
 
 ]]
-
+   elseif have_nedge then
+     print[[
+		$('#chart-local2remote-text').html("&nbsp;"+bitsToSize(bps_local2remote, 1000));
+		$('#chart-remote2local-text').html("&nbsp;"+bitsToSize(bps_remote2local, 1000));
+     ]]
    end
 
 print[[
 }
 	      } /* closes if (prev_bytes > 0) */
 
-		var msg = "&nbsp;<i class=\"fa fa-clock-o\"></i> <small>"+rsp.localtime+" | Uptime: "+rsp.uptime+"</small>";
+		var msg = "&nbsp;<i class=\"fa fa-clock-o\"></i> <small>"+rsp.localtime+" | ]] print(i18n("about.uptime")) print[[: "+rsp.uptime+"</small>";
 
                 if(rsp.system_host_stats.mem_total !== undefined) {
                    var mem_total = rsp.system_host_stats.mem_total;
@@ -275,7 +323,7 @@ print[[
 print (ntop.getHttpPrefix())
 print [[/lua/show_alerts.lua\">"
 
-                   msg += "&nbsp;<span class=\"label " + label + "\">"+addCommas(rsp.engaged_alerts)+" <i class=\"fa fa-warning\"></i></span></A>"
+                   msg += "<span class=\"label " + label + "\">"+addCommas(rsp.engaged_alerts)+" <i class=\"fa fa-warning\"></i></span></A>"
                 }
 
 		if((rsp.engaged_alerts > 0 || rsp.alerts_stored == true) && $("#alerts-id").is(":visible") == false) {
@@ -287,15 +335,15 @@ print [[/lua/show_alerts.lua\">"
 		var alert = 0;     
 
 		if(rsp.num_local_hosts > 0) {
-		  msg += "<a style=\"margin-left:0.5em;\" href=\"]]
+		  msg += "&nbsp;<a href=\"]]
 print (ntop.getHttpPrefix())
 print [[/lua/hosts_stats.lua?mode=local\">";
 
 		  msg += "<span class=\"label label-success\">";
-		  msg += addCommas(rsp.num_local_hosts)+" <i class=\"fa fa-laptop\" aria-hidden=\"true\"></i></span></a> ";
+		  msg += addCommas(rsp.num_local_hosts)+" <i class=\"fa fa-laptop\" aria-hidden=\"true\"></i></span></a>";
 		}
 
-	    msg += "<a href=\"]]
+	    msg += "&nbsp;<a href=\"]]
 print (ntop.getHttpPrefix())
 print [[/lua/hosts_stats.lua?mode=remote\">";
 
@@ -311,12 +359,15 @@ print [[/lua/hosts_stats.lua?mode=remote\">";
 
 		msg += addCommas(rsp.num_hosts-rsp.num_local_hosts)+" <i class=\"fa fa-laptop\" aria-hidden=\"true\"></i></span></a> ";
 
-            msg += "<a href=\"]]
+	    if(typeof rsp.num_devices !== "undefined") {
+	      msg += "<a href=\"]]
 print (ntop.getHttpPrefix())
-print [[/lua/macs_stats.lua?devices_mode=host_macs_only\">";
+print [[/lua/macs_stats.lua?devices_mode=source_macs_only\">";
 		  msg += "<span class=\"label label-default\">";
 		msg += addCommas(rsp.num_devices)+" Devices</span></a> ";
+	    }
 
+	    if(typeof rsp.num_flows !== "undefined") {
     msg += "<a href=\"]]
 print (ntop.getHttpPrefix())
 print [[/lua/flows_stats.lua\">";
@@ -340,13 +391,52 @@ print [[/lua/if_stats.lua\"><i class=\"fa fa-warning\" style=\"color: #B94A48;\"
 
 		   msg += "</span></A> ";
 		}
+	    }
 
-		$('#network-load').html(msg);
+	    if((typeof rsp.num_live_captures !== "undefined") && (rsp.num_live_captures > 0)) {               
+   	        msg += "&nbsp;<a href=\"]]
+                print (ntop.getHttpPrefix())
+                print [[/lua/live_capture_stats.lua\">";
+ 	        msg += "<span class=\"label label-primary\">";
+		msg += addCommas(rsp.num_live_captures)+" <i class=\"fa fa-download fa-lg\"></i></A> </span> ";
+	    }
+
+	    if(typeof rsp.remote_assistance !== "undefined") {
+	      var status = rsp.remote_assistance.status;
+	      var status_label = (status == "active") ? "success" : "danger";
+	      
+	      msg += "&nbsp;<a href=\"]] print(ntop.getHttpPrefix()) print[[/lua/admin/remote_assistance.lua?tab=status\"><span class=\"label label-" + status_label + "\" title=\"]]
+	      print(i18n("remote_assistance.remote_assistance")) print[[\">";
+	      msg += "<i class=\"fa fa-commenting fa-lg\"></i></span></a>";
+	    }
+
+	    if(typeof rsp.traffic_recording !== "undefined") {
+                var status_label="primary"; 
+                var status_title="]] print(i18n("traffic_recording.recording")) print [[";
+                if (rsp.traffic_recording != "recording") { 
+                  status_label = "danger"; 
+                  status_title = "]] print(i18n("traffic_recording.failure")) print [["; 
+                }
+   	        msg += "&nbsp;<a href=\"]] print (ntop.getHttpPrefix()) print [[/lua/if_stats.lua?ifid=]] print(tostring(getInterfaceId(ifname))) print[[&page=traffic_recording&tab=status\">";
+ 	        msg += "<span class=\"label label-"+status_label+"\" title=\""+addCommas(status_title)+"\">";
+		msg += "<i class=\"fa fa-hdd-o fa-lg\"></i></a></span>";
+	    }
+
+	    if(typeof rsp.traffic_extraction !== "undefined") {
+                var status_title="]] print(i18n("traffic_recording.traffic_extraction_jobs")) print [["; 
+                var status_label = "default";
+                if (rsp.traffic_extraction == "ready") status_label="primary";
+   	        msg += "&nbsp;<a href=\"]] print (ntop.getHttpPrefix()) print [[/lua/if_stats.lua?ifid=]] print(tostring(getInterfaceId(ifname))) print[[&page=traffic_recording&tab=jobs\">";
+ 	        msg += "<span class=\"label label-"+status_label+"\" title=\""+addCommas(status_title)+"\">";
+		msg += rsp.traffic_extraction_num_tasks+" <i class=\"fa fa-tasks fa-lg\"></i></a></span>";
+	    }
+
+	    $('#network-load').html(msg);
 
 
-		if(alert) {
-		   $('#toomany').html("<div class='alert alert-warning'><h4>Warning</h4>You have too many hosts/flows for your ntopng configuration and this will lead to packet drops and high CPU load. Please restart ntopng increasing -x and -X.</div>");
-		}
+	    if(alert) {
+	        $('#toomany').html("<div class='alert alert-warning'><h4>]] print(i18n("warning")) print[[</h4>]] print(i18n("about.you_have_too_many_flows", {product=info["product"]})) print[[.</div>");
+	    }
 
 	    prev_bytes   = rsp.bytes;
 	    prev_packets = rsp.packets;
@@ -368,8 +458,24 @@ print [[/lua/logout.lua");  */
       });
 }
 
+$(document).ajaxError(function(err, response, ajaxSettings, thrownError) {
+  if((response.status == 403) && (response.responseText == "Login Required"))
+    window.location.href = "]] print(ntop.getHttpPrefix().."/lua/login.lua") print[[";
+});
+
 footerRefresh();  /* call immediately to give the UI a more responsive look */
-setInterval(footerRefresh, ]] print(getInterfaceRefreshRate(_ifstats.id).."") print[[ * 1000);  /* re-schedule every [interface-rate] seconds */
+setInterval(footerRefresh, ]]
+
+local footer_refresh_rate
+
+if have_nedge then
+  footer_refresh_rate = 5
+else
+  footer_refresh_rate = getInterfaceRefreshRate(_ifstats.id)
+end
+
+print(footer_refresh_rate.."")
+print[[ * 1000);  /* re-schedule every [interface-rate] seconds */
 
 //Enable tooltip without a fixer placement
 $(document).ready(function () { $("[rel='tooltip']").tooltip(); });
