@@ -1,5 +1,5 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -519,7 +519,7 @@ function handleCustomFlowField(key, value, snmpdevice)
    elseif((key == 'FLOW_USER_NAME') or (key == '57593')) then
       elems = string.split(value, ';')
 
-      if((elems ~= nil) and (table.getn(elems) == 6)) then
+      if((elems ~= nil) and (#elems == 6)) then
           r = '<table class="table table-bordered table-striped">'
 	  imsi = elems[1]
 	  mcc = string.sub(imsi, 1, 3)
@@ -576,11 +576,11 @@ function formatTcpFlags(flags)
    end
 
    rsp = "<A HREF=\"http://en.wikipedia.org/wiki/Transmission_Control_Protocol\">"
-   if(bit.band(flags, 1) == 2)  then rsp = rsp .. " SYN "  end
-   if(bit.band(flags, 16) == 16) then rsp = rsp .. " ACK "  end
-   if(bit.band(flags, 1) == 1)  then rsp = rsp .. " FIN "  end
-   if(bit.band(flags, 4) == 4)  then rsp = rsp .. " RST "  end
-   if(bit.band(flags, 8) == 8 )  then rsp = rsp .. " PUSH " end
+   if((flags & 1) == 2)  then rsp = rsp .. " SYN "  end
+   if((flags & 16) == 16) then rsp = rsp .. " ACK "  end
+   if((flags & 1) == 1)  then rsp = rsp .. " FIN "  end
+   if((flags & 4) == 4)  then rsp = rsp .. " RST "  end
+   if((flags & 8) == 8 )  then rsp = rsp .. " PUSH " end
 
    return(rsp .. "</A>")
 end
@@ -1162,6 +1162,9 @@ function getFlowLabel(flow, show_macs, add_hyperlinks)
       if(flow["cli.systemhost"] == true) then
 	 cli_name = cli_name.." <i class='fa fa-flag' aria-hidden='true'></i>"
       end
+      if(flow["cli.blacklisted"] == true) then
+	 cli_name = cli_name.." <i class='fa fa-ban' aria-hidden='true' title='Blacklisted'></i>"
+      end
       cli_name = cli_name.."</A>"
 
       srv_name = "<A HREF=\""..ntop.getHttpPrefix().."/lua/host_details.lua?"..hostinfo2url(flow,"srv") .. "\">"
@@ -1169,6 +1172,9 @@ function getFlowLabel(flow, show_macs, add_hyperlinks)
       if(flow["srv.systemhost"] == true) then
 	 srv_name = srv_name.." <i class='fa fa-flag' aria-hidden='true'></i>"
       end
+      if(flow["srv.blacklisted"] == true) then
+	 srv_name = srv_name.." <i class='fa fa-ban' aria-hidden='true' title='Blacklisted'></i>"
+      end
       srv_name = srv_name.."</A>"
 
       if cli_port then
@@ -1361,7 +1367,7 @@ function getSIPInfo(infoPar)
       if(((called_party == nil) or (called_party == "")) and ((calling_party == nil) or (calling_party == ""))) then
         returnString = ""
       else
-        returnString =  calling_party .. " <-> " .. called_party
+        returnString =  calling_party .. " <i class='fa fa-exchange fa-sm' aria-hidden='true'></i> " .. called_party
       end
     end
   end
@@ -1772,43 +1778,56 @@ end
 -- #######################
 
 function getFlowQuota(ifid, info, as_client)
-  local pool_id, quota_protocol, quota_is_category
+  local pool_id, quota_source
 
   if as_client then
     pool_id = info["cli.pool_id"]
-    quota_protocol = info["cli.quota_applied_proto"]
-    quota_is_category = info["cli.quota_is_category"]
+    quota_source = info["cli.quota_source"]
   else
     pool_id = info["srv.pool_id"]
-    quota_protocol = info["srv.quota_applied_proto"]
-    quota_is_category = info["srv.quota_is_category"]
+    quota_source = info["srv.quota_source"]
   end
 
   local master_proto, app_proto = splitProtocol(info["proto.ndpi"])
+  app_proto = app_proto or master_proto
+
   local pools_stats = interface.getHostPoolsStats()
   local pool_stats = pools_stats and pools_stats[tonumber(pool_id)]
+  local quota_and_protos = shaper_utils.getPoolProtoShapers(ifid, pool_id)
 
   if pool_stats ~= nil then
-    local application = ternary(quota_protocol == "master", master_proto, app_proto)
-    local key, category_stats, proto_stats
+    local key = nil
 
-    if quota_is_category then
-      -- the quota is being applied on the protocol category
-      key = interface.getnDPIProtoCategory(interface.getnDPIProtoId(application)).name
-      proto_stats = nil
-      category_stats = pool_stats.ndpi_categories
-    else
-      -- the quota is being applied on the protocol itself
-      key = application
-      proto_stats = pool_stats.ndpi
-      category_stats = nil
+    if quota_source == "policy_source_protocol" then
+	proto_stats = pool_stats.ndpi
+
+	-- determine if the quota is on the app or master proto
+	if(quota_and_protos[master_proto] ~= nil) then
+	    key = master_proto
+	else
+	    key = app_proto
+	end
+    elseif quota_source == "policy_source_category" then
+	key = flow["proto.ndpi_cat"]
+	proto_stats = nil
+	category_stats = pool_stats.ndpi_categories
+    elseif quota_source == "policy_source_pool" then
+	key = "Default"
+	proto_stats = nil
+	category_stats = {default = pool_stats.cross_application}
     end
 
-    local quota_and_protos = shaper_utils.getPoolProtoShapers(ifid, pool_id)
-    local proto_info = quota_and_protos[key]
+    if key ~= nil then
+	local proto_info = nil
+	if key ~= "Default" then
+	    proto_info = quota_and_protos[key]
+	else
+	    proto_info = shaper_utils.getCrossApplicationShaper(ifid, pool_id)
+	end
 
-    if proto_info ~= nil then
-      return proto_info, proto_stats, category_stats
+	if proto_info ~= nil then
+	  return proto_info, proto_stats, category_stats
+	end
     end
   end
 
@@ -1836,23 +1855,39 @@ function printFlowSNMPInfo(snmpdevice, input_idx, output_idx)
    if available_devices == nil then available_devices = {} end
 
    for dev, _ in pairs(available_devices) do
+      local snmp_device = require "snmp_device"
+      snmp_device.init(dev)
+
       if dev == snmpdevice then
 	 local community = get_snmp_community(dev)
 	 local port_indexes = get_snmp_device_port_indexes(dev, community)
 	 if port_indexes == nil then port_indexes = {} end
 
-	 local snmpurl = "<A HREF='" .. ntop.getHttpPrefix() .. "/lua/pro/enterprise/snmp_device_info.lua?ip="..dev.. "'>"..dev.."</A>"
+	 local snmpurl = "<A HREF='" .. ntop.getHttpPrefix() .. "/lua/pro/enterprise/snmp_device_details.lua?host="..dev.. "'>"..dev.."</A>"
 
-	 local inputurl = input_idx
-	 local outputurl = output_idx
-	 for _, idx in pairs(port_indexes) do
-	    if input_idx == idx then
-	       inputurl = "<A HREF='" .. ntop.getHttpPrefix() .. "/lua/pro/enterprise/snmp_device_info.lua?ip="..dev.."&ifIdx="..input_idx.."'>"..input_idx.." <span class=\"label label-default\">"..get_snmp_port_label(dev, community, input_idx).."</span></A>"
-	    end
-	    if output_idx == idx then
-	       outputurl = "<A HREF='" .. ntop.getHttpPrefix() .. "/lua/pro/enterprise/snmp_device_info.lua?ip="..dev.."&ifIdx="..input_idx.."'>"..output_idx.." <span class=\"label label-default\">"..get_snmp_port_label(dev, community, output_idx).."</span></A>"
+	 local snmp_interfaces = snmp_device.get_device()["interfaces"]
+	 local inputurl, outputurl
+
+	 local function prepare_interface_url(idx, port)
+	    local ifurl
+
+	    if port then
+	       local label = port["index"]
+
+	       if port["name"] and port["name"] ~= "" then
+		  label = shortenString(port["name"])
+	       end
+
+	       ifurl = "<A HREF='" .. ntop.getHttpPrefix() .. "/lua/pro/enterprise/snmp_interface_details.lua?host="..dev.."&snmp_port_idx="..port["index"].."'>"..label.."</A>"
+	    else
+	       ifurl = idx
 	    end
+
+	    return ifurl
 	 end
+ 
+	 local inputurl = prepare_interface_url(input_idx, snmp_interfaces[input_idx])
+	 local outputurl = prepare_interface_url(output_idx, snmp_interfaces[output_idx])
 
 	 print("<tr><th rowspan='2'>"..i18n("details.flow_snmp_localization").."</th><th>"..i18n("snmp.snmp_device").."</th><th>"..i18n("details.input_device_port").." / "..i18n("details.output_device_port").."</th></tr>")
 	 print("<tr><td>"..snmpurl.."</td><td>"..inputurl.." / "..outputurl.."</td></tr>")
@@ -1863,3 +1898,39 @@ function printFlowSNMPInfo(snmpdevice, input_idx, output_idx)
 end
 
 -- #######################
+
+function printBlockFlowJs()
+  print[[
+  var block_flow_csrf = "]] print(ntop.getRandomCSRFValue()) print[[";
+
+  function block_flow(flow_key) {
+    var url = "]] print(ntop.getHttpPrefix()) print[[/lua/pro/nedge/block_flow.lua";
+    $.ajax({
+      type: 'GET',
+      url: url,
+      cache: false,
+      data: {
+        csrf: block_flow_csrf,
+        flow_key: flow_key
+      },
+      success: function(content) {
+        var data = jQuery.parseJSON(content);
+        block_flow_csrf = data.csrf;
+        if (data.status == "BLOCKED") {
+          $('#'+flow_key+'_info').find('.block-badge')
+            .removeClass('label-default')
+            .addClass('label-danger')
+            .attr('title', ']] print(i18n("flow_details.flow_traffic_is_dropped")) print[[');
+          $('#'+flow_key+'_application, #'+flow_key+'_l4, #'+flow_key+'_client, #'+flow_key+'_server')
+            .css("text-decoration", "line-through");
+        }
+      },
+      error: function(content) {
+        console.log("error");
+      }
+    });
+  }
+  ]]
+end
+
+-- #######################
