@@ -1,5 +1,5 @@
 --
--- (C) 2017 - ntop.org
+-- (C) 2017-18 - ntop.org
 --
 dirs = ntop.getDirs()
 
@@ -11,14 +11,35 @@ local os_utils = require "os_utils"
 
 local host_pools_utils = {}
 host_pools_utils.DEFAULT_POOL_ID = "0"
-host_pools_utils.DEFAULT_ROUTING_POLICY_ID = "0"
+host_pools_utils.DEFAULT_ROUTING_POLICY_ID = "1"
 host_pools_utils.FIRST_AVAILABLE_POOL_ID = "1"
 host_pools_utils.DEFAULT_POOL_NAME = "Not Assigned"
 host_pools_utils.MAX_NUM_POOLS = 128 -- Note: keep in sync with C
 
-host_pools_utils.LIMITED_NUMBER_POOL_MEMBERS = ntop_info["constants.max_num_pool_members"] or 5
+--
+-- BEGIN NEDGE specific code
+--
+function host_pools_utils.usernameToPoolId(username)
+  local res = ntop.getPref("ntopng.user."..string.lower(username)..".host_pool_id")
+  return ternary(not isEmptyString(res), res, nil)
+end
+
+function host_pools_utils.poolIdToUsername(pool_id)
+  local ifid = getInterfaceId(ifname)
+  return host_pools_utils.getPoolName(ifid, pool_id)
+end
+
+function host_pools_utils.getUserUrl(pool_id)
+  return ntop.getHttpPrefix() .."/lua/pro/nedge/admin/nf_edit_user.lua?username=" ..
+    ternary(tostring(pool_id) == host_pools_utils.DEFAULT_POOL_ID, "", host_pools_utils.poolIdToUsername(pool_id))
+end
+--
+-- END NEDGE specific code
+--
+
+host_pools_utils.LIMITED_NUMBER_POOL_MEMBERS = ntop_info["constants.max_num_pool_members"]
 -- this takes into account the special pools
-host_pools_utils.LIMITED_NUMBER_TOTAL_HOST_POOLS = ntop_info["constants.max_num_host_pools"] or 5
+host_pools_utils.LIMITED_NUMBER_TOTAL_HOST_POOLS = ntop_info["constants.max_num_host_pools"]
 -- this does not take into account the special pools
 host_pools_utils.LIMITED_NUMBER_USER_HOST_POOLS = host_pools_utils.LIMITED_NUMBER_TOTAL_HOST_POOLS - 1
 
@@ -34,8 +55,8 @@ local function get_pool_details_key(ifid, pool_id)
   return "ntopng.prefs." .. ifid .. ".host_pools.details." .. pool_id
 end
 
-local function get_user_pool_dump_key(ifid)
-  return "ntopng.prefs." .. ifid .. ".host_pools.dump"
+local function get_pools_serialized_key(ifid)
+  return "ntopng.serialized_host_pools.ifid_" .. ifid
 end
 
 -- It is safe to call this multiple times
@@ -55,35 +76,69 @@ function host_pools_utils.setPoolDetail(ifid, pool_id, detail, value)
   return ntop.setHashCache(details_key, detail, tostring(value))
 end
 
-local function addMemberToRedisPool(members_key, member_key)
+local function traceHostPoolEvent(severity, event)
+    if ntop.getPref("ntopng.prefs.enable_host_pools_log") ~= "1" then
+       return
+    end
+
+    local f_name = debug.getinfo(2, "n").name
+    if f_name ~= nil then
+       f_name = string.format("[%s] ", f_name)
+    end
+
+    traceError(severity, TRACE_CONSOLE, string.format("%s%s", f_name or '', event))
+end
+
+local function addMemberToRedisPool(ifid, pool_id, member_key)
+  if pool_id == host_pools_utils.DEFAULT_POOL_ID then
+    -- avoid adding default pool members explicitly
+    traceHostPoolEvent(TRACE_NORMAL,
+		       string.format("Setting DEFAULT_POOL_ID (aka 'Not Assigned'). [pool_id: %d][member: %s]",
+				     host_pools_utils.DEFAULT_POOL_ID, member_key))
+    return true
+  end
+
+  local members_key = get_pool_members_key(ifid, pool_id)
   local n = table.len(ntop.getMembersCache(members_key) or {})
 
   if n >= host_pools_utils.LIMITED_NUMBER_POOL_MEMBERS then
+    traceHostPoolEvent(TRACE_ERROR, string.format("Unable to set host pool, maximum number of pool members hit. [max num pool members: %d][member: %s] [members_key: %s]", host_pools_utils.LIMITED_NUMBER_POOL_MEMBERS, member_key, members_key))
     return false
   end
 
   ntop.setMembersCache(members_key, member_key)
+  traceHostPoolEvent(TRACE_NORMAL, string.format("Member added to pool. [member: %s] [members_key: %s]", member_key, members_key))
   return true
 end
 
 --------------------------------------------------------------------------------
 
 function host_pools_utils.createPool(ifid, pool_id, pool_name, children_safe,
-				     enforce_quotas_per_pool_member, enforce_shapers_per_pool_member)
+				     enforce_quotas_per_pool_member, enforce_shapers_per_pool_member, ignore_exist)
   local details_key = get_pool_details_key(ifid, pool_id)
   local ids_key = get_pool_ids_key(ifid)
+  local members = ntop.getMembersCache(ids_key) or {}
 
-  local n = table.len(ntop.getMembersCache(ids_key) or {})
+  local n = table.len(members)
 
   if n >= host_pools_utils.LIMITED_NUMBER_TOTAL_HOST_POOLS then
     return false
   end
 
+  if not ignore_exist then
+    for _, m in pairs(members) do
+      if m == pool_id then
+        return true
+      end
+    end
+  end
+
   ntop.setMembersCache(ids_key, pool_id)
   ntop.setHashCache(details_key, "name", pool_name)
   ntop.setHashCache(details_key, "children_safe", tostring(children_safe or false))
   ntop.setHashCache(details_key, "enforce_quotas_per_pool_member",  tostring(enforce_quotas_per_pool_member  or false))
   ntop.setHashCache(details_key, "enforce_shapers_per_pool_member", tostring(enforce_shapers_per_pool_member or false))
+  ntop.setHashCache(details_key, "forge_global_dns", "true")
   return true
 end
 
@@ -92,12 +147,12 @@ function host_pools_utils.deletePool(ifid, pool_id)
   local ids_key = get_pool_ids_key(ifid)
   local details_key = get_pool_details_key(ifid, pool_id)
   local members_key = get_pool_members_key(ifid, pool_id)
-  local dump_key = get_user_pool_dump_key(ifid)
+  local serialized_key = get_pools_serialized_key(ifid)
 
   ntop.delMembersCache(ids_key, pool_id)
   ntop.delCache(details_key)
   ntop.delCache(members_key)
-  ntop.delHashCache(dump_key, pool_id)
+  ntop.delHashCache(serialized_key, pool_id)
   ntop.rmdir(rrd_base)
 end
 
@@ -146,6 +201,10 @@ end
 -- a pool.
 --
 function host_pools_utils.changeMemberPool(ifid, member_and_vlan, new_pool, info --[[optional]], strict_host_mode --[[optional]])
+  traceHostPoolEvent(TRACE_NORMAL,
+		     string.format("Pool change requested. [member: %s][new_pool: %s][strict_host_mode: %s]",
+				   member_and_vlan, new_pool, tostring(strict_host_mode)))
+
   if not strict_host_mode then
     local hostkey, is_network = host_pools_utils.getMemberKey(member_and_vlan)
 
@@ -156,10 +215,14 @@ function host_pools_utils.changeMemberPool(ifid, member_and_vlan, new_pool, info
         info = interface.getHostInfo(hostinfo["host"], hostinfo["vlan"])
       end
 
-      if not isEmptyString(info["mac"]) then
+      if not isEmptyString(info["mac"]) and (info["mac"] ~= "00:00:00:00:00:00") then
         local mac_has_pool, mac_pool_info = getMembershipInfo(info["mac"])
 
-        if mac_has_pool and mac_pool_info.existing_member_pool ~= host_pools_utils.DEFAULT_POOL_ID then
+        -- Two cases:
+        --  1. if we are moving to a well defined pool, we must set the mac pool
+        --  2. if we are moving to the default pool, we must set the mac pool only
+        --     if the mac already has a pool, otherwise we set the ip pool
+        if (new_pool ~= host_pools_utils.DEFAULT_POOL_ID) or mac_has_pool then
           -- we must change the MAC address in order to change the host pool
           member_and_vlan = info["mac"]
         end
@@ -167,7 +230,6 @@ function host_pools_utils.changeMemberPool(ifid, member_and_vlan, new_pool, info
     end
   end
 
-  local members_key = get_pool_members_key(ifid, new_pool)
   local member_exists, info = getMembershipInfo(member_and_vlan)
   local prev_pool
 
@@ -180,27 +242,42 @@ function host_pools_utils.changeMemberPool(ifid, member_and_vlan, new_pool, info
   end
 
   if prev_pool == new_pool then
+     traceHostPoolEvent(TRACE_ERROR,
+		     string.format("Pool did't change. Exiting. [member: %s][prev_pool: %s][new_pool: %s]",
+				   member_and_vlan, prev_pool, new_pool))
     return false
   end
 
+  traceHostPoolEvent(TRACE_NORMAL,
+		     string.format("Pool change prepared. [member: %s][info.key: %s][prev_pool: %s][new_pool: %s]",
+				   member_and_vlan, tostring(info.key), prev_pool, new_pool))
+
   host_pools_utils.deletePoolMember(ifid, prev_pool, info.key)
-  addMemberToRedisPool(members_key, info.key)
+  addMemberToRedisPool(ifid, new_pool, info.key)
   return true
 end
 
 function host_pools_utils.addPoolMember(ifid, pool_id, member_and_vlan)
-  local members_key = get_pool_members_key(ifid, pool_id)
+  traceHostPoolEvent(TRACE_NORMAL,
+		     string.format("Pool member addition requested. [member: %s][pool_id: %s]",
+				   member_and_vlan, pool_id))
+
   local member_exists, info = getMembershipInfo(member_and_vlan)
 
   if member_exists then
+     traceHostPoolEvent(TRACE_NORMAL, string.format("Member already in pool. [pool_id: %d] [member: %s]", pool_id, member_and_vlan))
     return false, info
   else
-    local rv = addMemberToRedisPool(members_key, info.key)
+    local rv = addMemberToRedisPool(ifid, pool_id, info.key)
     return rv, info
   end
 end
 
 function host_pools_utils.deletePoolMember(ifid, pool_id, member_and_vlan)
+  traceHostPoolEvent(TRACE_NORMAL,
+		     string.format("Pool member deletion requested. [member: %s][pool_id: %s]",
+				   member_and_vlan, pool_id))
+
   local members_key = get_pool_members_key(ifid, pool_id)
 
   -- Possible delete volatile member
@@ -325,9 +402,21 @@ function host_pools_utils.getChildrenSafe(ifid, pool_id)
   return toboolean(host_pools_utils.getPoolDetail(ifid, pool_id, "children_safe"))
 end
 
+function host_pools_utils.setChildrenSafe(ifid, pool_id, value)
+  host_pools_utils.setPoolDetail(ifid, pool_id, "children_safe", ternary(value, "true", "false"))
+end
+
+function host_pools_utils.getForgeGlobalDNS(ifid, pool_id)
+  return toboolean(host_pools_utils.getPoolDetail(ifid, pool_id, "forge_global_dns"))
+end
+
+function host_pools_utils.setForgeGlobalDNS(ifid, pool_id, value)
+  host_pools_utils.setPoolDetail(ifid, pool_id, "forge_global_dns", ternary(value, "true", "false"))
+end
+
 function host_pools_utils.getRoutingPolicyId(ifid, pool_id)
   local routing_policy_id = host_pools_utils.getPoolDetail(ifid, pool_id, "routing_policy_id")
-  if isEmptyString(routing_policy_id) then routing_policy_id = "0" end
+  if isEmptyString(routing_policy_id) then routing_policy_id = host_pools_utils.DEFAULT_ROUTING_POLICY_ID end
   return routing_policy_id
 end
 
@@ -351,7 +440,9 @@ function host_pools_utils.clearPools()
     if not ifstats.isView then
        local pools_list = host_pools_utils.getPoolsList(ifid)
        for _, pool in pairs(pools_list) do
-	  host_pools_utils.deletePool(ifid, pool["id"])
+          if pool["id"] ~= host_pools_utils.DEFAULT_POOL_ID then
+            host_pools_utils.deletePool(ifid, pool["id"])
+          end
        end
     end
 
@@ -405,9 +496,9 @@ function host_pools_utils.purgeExpiredPoolsMembers()
    for _, ifname in pairs(ifnames) do
       interface.select(ifname)
 
-      if isCaptivePortalActive() then
-        interface.purgeExpiredPoolsMembers()
-      end
+      -- Currently, volatile pool members are no longer used,
+      -- so there's no need to purge them.
+      -- interface.purgeExpiredPoolsMembers()
    end
 end
 
@@ -417,25 +508,26 @@ function host_pools_utils.getRRDBase(ifid, pool_id)
 end
 
 function host_pools_utils.updateRRDs(ifid, dump_ndpi, verbose)
+  local ts_utils = require "ts_utils"
+  require "ts_5min"
+
   -- NOTE: requires graph_utils
-  for pool_id, pool_stats in pairs(interface.getHostPoolsStats()) do
-    local pool_base = host_pools_utils.getRRDBase(ifid, pool_id)
+  for pool_id, pool_stats in pairs(interface.getHostPoolsStats() or {}) do
+    ts_utils.append("host_pool:traffic", {ifid=ifid, pool=pool_id,
+              bytes_sent=pool_stats["bytes.sent"], bytes_rcvd=pool_stats["bytes.rcvd"]}, when)
 
-    if(not(ntop.exists(pool_base))) then
-      ntop.mkdir(pool_base)
+    if pool_id ~= tonumber(host_pools_utils.DEFAULT_POOL_ID) then
+       local flows_dropped = pool_stats["flows.dropped"] or 0
+
+       ts_utils.append("host_pool:blocked_flows", {ifid=ifid, pool=pool_id,
+              num_flows=flows_dropped}, when)
     end
 
-    -- Traffic stats
-    local rrdpath = os_utils.fixPath(pool_base .. "/bytes.rrd")
-    createRRDcounter(rrdpath, 300, verbose)
-    ntop.rrd_update(rrdpath, nil, tolongint(pool_stats["bytes.sent"]), tolongint(pool_stats["bytes.rcvd"]))
-
     -- nDPI stats
     if dump_ndpi then
       for proto,v in pairs(pool_stats["ndpi"] or {}) do
-        local ndpiname = os_utils.fixPath(pool_base.."/"..proto..".rrd")
-        createRRDcounter(ndpiname, 300, verbose)
-        ntop.rrd_update(ndpiname, nil, tolongint(v["bytes.sent"]), tolongint(v["bytes.rcvd"]))
+        ts_utils.append("host_pool:ndpi", {ifid=ifid, pool=pool_id, protocol=proto,
+              bytes_sent=v["bytes.sent"], bytes_rcvd=v["bytes.rcvd"]}, when)
       end
     end
   end
@@ -464,8 +556,10 @@ function host_pools_utils.printQuotas(pool_id, host, page_params)
   end
 
   if empty then
+    local url = "/lua/pro/nedge/admin/nf_edit_user.lua?page=protocols&username=" .. host_pools_utils.poolIdToUsername(pool_id)
+
     print("<div class=\"alert alert alert-danger\"><img src=".. ntop.getHttpPrefix() .. "/img/warning.png>"..i18n("shaping.no_quota_data")..
-      ". Create new quotas <a href=\""..ntop.getHttpPrefix().."/lua/if_stats.lua?page=filtering&pool="..pool_id.."\">here</a>.</div>")
+      ". " .. i18n("host_pools.create_new_quotas_here", {url=ntop.getHttpPrefix()..url}) .. "</div>")
   else
     print[[
     <table class="table table-bordered table-striped">
@@ -525,4 +619,25 @@ function host_pools_utils.getFirstAvailablePoolId(ifid)
   return tostring(host_pool_id)
 end
 
+function host_pools_utils.resetPoolsQuotas(ifid, pool_filter)
+  local serialized_key = get_pools_serialized_key(ifid)
+  local keys_to_del
+
+  if pool_filter ~= nil then
+    keys_to_del = {[pool_filter]=1, }
+  else
+    keys_to_del = ntop.getHashKeysCache(serialized_key) or {}
+  end
+
+  -- Delete the redis serialization
+  for key in pairs(keys_to_del) do
+    ntop.delHashCache(serialized_key, tostring(key))
+  end
+
+  -- Delete the in-memory stats
+  interface.resetPoolsQuotas(pool_filter)
+end
+
+host_pools_utils.traceHostPoolEvent = traceHostPoolEvent
+
 return host_pools_utils
