@@ -1,33 +1,65 @@
 --
--- (C) 2013-17 - ntop.org
+-- (C) 2013-18 - ntop.org
 --
 
-dirs = ntop.getDirs()
+local dirs = ntop.getDirs()
 package.path = dirs.installdir .. "/scripts/lua/modules/?.lua;" .. package.path
 
 require "lua_utils"
-json = require("dkjson")
+local json = require("dkjson")
 
 --sendHTTPContentTypeHeader('text/html')
 sendHTTPHeader('application/json')
 
+local callback_utils = require("callback_utils")
+local recording_utils = require("recording_utils")
+local remote_assistance = require("remote_assistance")
+
+local function userHasRestrictions()
+   local allowed_nets = ntop.getPref("ntopng.user." .. (_SESSION["user"] or "") .. ".allowed_nets")
+
+   for _, net in pairs(split(allowed_nets, ",")) do
+      if not isEmptyString(net) and net ~= "0.0.0.0/0" and net ~= "::/0" then
+	 return true
+      end
+   end
+
+   return false
+end
+
+local function countHosts()
+   local res = {
+      local_hosts = 0,
+      hosts = 0,
+   }
+
+   for host, info in callback_utils.getHostsIterator(false --[[no details]]) do
+      if info.localhost then
+	 res.local_hosts = res.local_hosts + 1
+      end
+
+      res.hosts = res.hosts + 1
+   end
+
+   return res
+end
+
 function dumpInterfaceStats(interface_name)
    interface.select(interface_name)
 
    local ifstats = interface.getStats()
-   local stats = interface.getFlowsStats()
 
    local res = {}
    if(ifstats ~= nil) then
-      uptime = ntop.getUptime()
-      prefs = ntop.getPrefs()
+      local uptime = ntop.getUptime()
+      local prefs = ntop.getPrefs()
 
       -- Round up
-      hosts_pctg = math.floor(1+((ifstats.stats.hosts*100)/prefs.max_num_hosts))
-      flows_pctg = math.floor(1+((ifstats.stats.flows*100)/prefs.max_num_flows))
+      local hosts_pctg = math.floor(1+((ifstats.stats.hosts*100)/prefs.max_num_hosts))
+      local flows_pctg = math.floor(1+((ifstats.stats.flows*100)/prefs.max_num_flows))
 
       res["ifname"]  = interface_name
-      res["speed"]  = getInterfaceSpeed(ifstats)
+      res["speed"]  = getInterfaceSpeed(ifstats.id)
       -- network load is used by web pages that are shown to the user
       -- so we must return statistics since the latest (possible) reset
       res["packets"] = ifstats.stats_since_reset.packets
@@ -48,10 +80,16 @@ function dumpInterfaceStats(interface_name)
 	 res["alerts_stored"]      = alert_cache["alerts_stored"] or 0
       end
 
-      res["num_flows"]        = ifstats.stats.flows
-      res["num_hosts"]        = ifstats.stats.hosts
-      res["num_local_hosts"]  = ifstats.stats.local_hosts
-      res["num_devices"]      = ifstats.stats.devices
+      if not userHasRestrictions() then
+	 res["num_flows"]        = ifstats.stats.flows
+	 res["num_hosts"]        = ifstats.stats.hosts
+	 res["num_local_hosts"]  = ifstats.stats.local_hosts
+	 res["num_devices"]      = ifstats.stats.devices
+      else
+	 local num_hosts = countHosts()
+	 res["num_hosts"]        = num_hosts.hosts
+	 res["num_local_hosts"]  = num_hosts.local_hosts
+      end
 
       res["epoch"]      = os.time()
       res["localtime"]  = os.date("%H:%M:%S %z", res["epoch"])
@@ -63,9 +101,22 @@ function dumpInterfaceStats(interface_name)
       res["remote_bps"] = ifstats.remote_bps
       res["is_view"]    = ifstats.isView
 
+      if isAdministrator() then
+	 res["num_live_captures"]    = ifstats.stats.num_live_captures
+      end
+
       res["local2remote"] = ifstats["localstats"]["bytes"]["local2remote"]
       res["remote2local"] = ifstats["localstats"]["bytes"]["remote2local"]
 
+      if ntop.isnEdge() then
+        res["bytes_upload"] = ifstats["eth"]["egress"]["bytes"]
+        res["bytes_download"] = ifstats["eth"]["ingress"]["bytes"]
+      end
+
+      if ntop.isnEdge() and ifstats.type == "netfilter" and ifstats.netfilter then
+	 res["netfilter"] = ifstats.netfilter
+      end
+
       if(ifstats.zmqRecvStats ~= nil) then
 	 res["zmqRecvStats"] = {}
 	 res["zmqRecvStats"]["flows"] = ifstats.zmqRecvStats.flows
@@ -81,33 +132,39 @@ function dumpInterfaceStats(interface_name)
       res["tcpPacketStats"]["retransmissions"] = ifstats.tcpPacketStats.retransmissions
       res["tcpPacketStats"]["out_of_order"]    = ifstats.tcpPacketStats.out_of_order
       res["tcpPacketStats"]["lost"]            = ifstats.tcpPacketStats.lost
-      
-      if(ifstats["bridge.device_a"] ~= nil) then
-	 res["a_to_b_in_pkts"]       = ifstats["bridge.a_to_b.in_pkts"]
-	 res["a_to_b_in_bytes"]      = ifstats["bridge.a_to_b.in_bytes"]
-	 res["a_to_b_out_pkts"]      = ifstats["bridge.a_to_b.out_pkts"]
-	 res["a_to_b_out_bytes"]     = ifstats["bridge.a_to_b.out_bytes"]
-	 res["a_to_b_filtered_pkts"] = ifstats["bridge.a_to_b.filtered_pkts"]
-	 res["a_to_b_shaped_pkts"]   = ifstats["bridge.a_to_b.shaped_pkts"]
-
-	 res["b_to_a_in_pkts"]       = ifstats["bridge.b_to_a.in_pkts"]
-	 res["b_to_a_in_bytes"]      = ifstats["bridge.b_to_a.in_bytes"]
-	 res["b_to_a_out_pkts"]      = ifstats["bridge.b_to_a.out_pkts"]
-	 res["b_to_a_out_bytes"]     = ifstats["bridge.b_to_a.out_bytes"]
-	 res["b_to_a_filtered_pkts"] = ifstats["bridge.b_to_a.filtered_pkts"]
-	 res["b_to_a_shaped_pkts"]   = ifstats["bridge.b_to_a.shaped_pkts"]
-
-	 res["a_to_b_num_pkts_send_buffer_full"] = ifstats["bridge.a_to_b.num_pkts_send_buffer_full"]
-	 res["a_to_b_num_pkts_send_error"]       = ifstats["bridge.a_to_b.num_pkts_send_error"]
-	 res["b_to_a_num_pkts_send_buffer_full"] = ifstats["bridge.b_to_a.num_pkts_send_buffer_full"]
-	 res["b_to_a_num_pkts_send_error"]       = ifstats["bridge.b_to_a.num_pkts_send_error"] 
-      end
 
       if(ifstats["profiles"] ~= nil) then
 	 res["profiles"] = ifstats["profiles"]
       end
 
-      res["breed"] = stats["breeds"]
+      if remote_assistance.isAvailable() then
+	 if remote_assistance.isEnabled() then
+	    res["remote_assistance"] = {
+	       status = remote_assistance.getStatus(),
+	    }
+	 end
+      end
+
+      if recording_utils.isAvailable() then
+        if recording_utils.isEnabled(ifstats.id) then
+          if recording_utils.isActive(ifstats.id) then
+            res["traffic_recording"] = "recording"
+          else
+            res["traffic_recording"] = "failed"
+          end
+        end
+
+        if recording_utils.isEnabled(ifstats.id) then
+          local jobs_info = recording_utils.extractionJobsInfo(ifstats.id)
+          if jobs_info.ready > 0 then
+            res["traffic_extraction"] = "ready"
+          elseif jobs_info.total > 0 then
+            res["traffic_extraction"] = jobs_info.total
+          end
+          res["traffic_extraction_num_tasks"] = jobs_info.total
+        end
+      end
+
    end
    return res
 end
@@ -122,6 +179,8 @@ if(_GET["iffilter"] == "all") then
       -- its a lua array and will look for integers starting at one
       res[ifid..""] = dumpInterfaceStats(ifname)
    end
+elseif not isEmptyString(_GET["iffilter"]) then
+   res = dumpInterfaceStats(getInterfaceName(_GET["iffilter"]))
 else
    res = dumpInterfaceStats(ifname)
 end
