@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -108,15 +108,15 @@ void AlertCounter::lua(lua_State* vm, const char *table_key) {
   if(vm) {
     lua_newtable(vm);
 
-    lua_push_int_table_entry(vm, "max_num_hits_sec", max_num_hits_sec);
-    lua_push_int_table_entry(vm, "num_hits_since_first_alert", num_hits_since_first_alert);
-    lua_push_int_table_entry(vm, "over_threshold_duration_sec", over_threshold_duration_sec);
-    lua_push_int_table_entry(vm, "time_last_hit", time_last_hit);
-    lua_push_int_table_entry(vm, "time_last_alert_reported", time_last_alert_reported);
-    lua_push_int_table_entry(vm, "last_trespassed_threshold", last_trespassed_threshold);
-    lua_push_int_table_entry(vm, "num_trespassed_threshold", num_trespassed_threshold);
-    lua_push_int_table_entry(vm, "num_hits_rcvd_last_second", num_hits_rcvd_last_second);
-    lua_push_int_table_entry(vm, "last_trespassed_hits", last_trespassed_hits);
+    lua_push_uint64_table_entry(vm, "max_num_hits_sec", max_num_hits_sec);
+    lua_push_uint64_table_entry(vm, "num_hits_since_first_alert", num_hits_since_first_alert);
+    lua_push_uint64_table_entry(vm, "over_threshold_duration_sec", over_threshold_duration_sec);
+    lua_push_uint64_table_entry(vm, "time_last_hit", time_last_hit);
+    lua_push_uint64_table_entry(vm, "time_last_alert_reported", time_last_alert_reported);
+    lua_push_uint64_table_entry(vm, "last_trespassed_threshold", last_trespassed_threshold);
+    lua_push_uint64_table_entry(vm, "num_trespassed_threshold", num_trespassed_threshold);
+    lua_push_uint64_table_entry(vm, "num_hits_rcvd_last_second", num_hits_rcvd_last_second);
+    lua_push_uint64_table_entry(vm, "last_trespassed_hits", last_trespassed_hits);
     lua_push_bool_table_entry(vm, "threshold_trepassed", thresholdTrepassed);
 
     lua_pushstring(vm, table_key ? table_key : (char*)"alert_counter");
