@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *o
  * This program is free software; you can redistribute it and/or modify
@@ -41,10 +41,13 @@ AlertsManager::AlertsManager(int interface_id, const char *filename) : StoreMana
   sprintf(&filePath[base_offset], "%s", "alerts_v5.db");
   unlink(filePath);
   */
+  sprintf(&filePath[base_offset], "%s", "alerts_v6.db");
+  unlink(filePath);
   filePath[base_offset] = 0;
 
   /* open the newest */
   strncpy(fileName, filename, sizeof(fileName));
+  fileName[sizeof(fileName) - 1] = '\0';
   snprintf(fileFullPath, sizeof(fileFullPath), "%s/%d/alerts/%s",
 	   ntop->get_working_dir(), ifid, filename);
   ntop->fixPath(filePath);
@@ -167,7 +170,10 @@ int AlertsManager::openStore() {
 	   "cli_blacklisted  INTEGER NOT NULL DEFAULT 0, "
 	   "srv_blacklisted  INTEGER NOT NULL DEFAULT 0, "
 	   "cli_localhost    INTEGER NOT NULL DEFAULT 0, "
-	   "srv_localhost    INTEGER NOT NULL DEFAULT 0 "
+	   "srv_localhost    INTEGER NOT NULL DEFAULT 0, "
+	   "cli_host_pool_id INTEGER NOT NULL DEFAULT 0, "
+	   "srv_host_pool_id INTEGER NOT NULL DEFAULT 0, "
+	   "flow_status      INTEGER NOT NULL DEFAULT 0  "
 	   ");"
 	   "CREATE INDEX IF NOT EXISTS t3i_tstamp    ON %s(alert_tstamp); "
 	   "CREATE INDEX IF NOT EXISTS t3i_type      ON %s(alert_type); "
@@ -188,7 +194,10 @@ int AlertsManager::openStore() {
 	   "CREATE INDEX IF NOT EXISTS t3i_cport     ON %s(cli_port); "
 	   "CREATE INDEX IF NOT EXISTS t3i_sport     ON %s(srv_port); "
 	   "CREATE INDEX IF NOT EXISTS t3i_clocal    ON %s(cli_localhost); "
-	   "CREATE INDEX IF NOT EXISTS t3i_slocal    ON %s(srv_localhost); ",
+	   "CREATE INDEX IF NOT EXISTS t3i_slocal    ON %s(srv_localhost); "
+	   "CREATE INDEX IF NOT EXISTS t3i_cpool     ON %s(cli_host_pool_id); "
+	   "CREATE INDEX IF NOT EXISTS t3i_spool     ON %s(srv_host_pool_id); "
+	   "CREATE INDEX IF NOT EXISTS t3i_status    ON %s(flow_status); ",
 	   ALERTS_MANAGER_FLOWS_TABLE_NAME,
 	   NDPI_PROTOCOL_UNKNOWN,
 	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
@@ -200,7 +209,9 @@ int AlertsManager::openStore() {
 	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
 	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
 	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
-	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME);
+	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
+	   ALERTS_MANAGER_FLOWS_TABLE_NAME, ALERTS_MANAGER_FLOWS_TABLE_NAME,
+	   ALERTS_MANAGER_FLOWS_TABLE_NAME);
   m.lock(__FILE__, __LINE__);
   rc = exec_query(create_query, NULL, NULL);
   m.unlock(__FILE__, __LINE__);
@@ -210,8 +221,11 @@ int AlertsManager::openStore() {
 
 /* **************************************************** */
 
-bool AlertsManager::isAlertEngaged(AlertEngine alert_engine, AlertEntity alert_entity, const char *alert_entity_value, const char *engaged_alert_id,
-		  AlertType *alert_type, AlertLevel *alert_severity, char **alert_json, char **alert_source, char **alert_target, time_t *alert_tstamp) {
+bool AlertsManager::isAlertEngaged(AlertEngine alert_engine, AlertEntity alert_entity,
+				   const char *alert_entity_value, const char *engaged_alert_id,
+				   AlertType *alert_type, AlertLevel *alert_severity,
+				   char **alert_json, char **alert_source,
+				   char **alert_target, time_t *alert_tstamp) {
   char query[STORE_MANAGER_MAX_QUERY];
   sqlite3_stmt *stmt = NULL;
   int rc;
@@ -260,7 +274,6 @@ bool AlertsManager::isAlertEngaged(AlertEngine alert_engine, AlertEntity alert_e
 #endif
     } else if(rc == SQLITE_ERROR) {
       ntop->getTrace()->traceEvent(TRACE_INFO, "SQL Error: step");
-      rc = 1;
       goto out;
     }
   }
@@ -301,11 +314,12 @@ void AlertsManager::markForMakeRoom(bool on_flows) {
 int AlertsManager::engageAlert(AlertEngine alert_engine, AlertEntity alert_entity, const char *alert_entity_value,
 			       const char *engaged_alert_id,
 			       AlertType alert_type, AlertLevel alert_severity, const char *alert_json,
-			       const char *alert_origin, const char *alert_target) {
-  if(!ntop->getPrefs()->are_alerts_disabled()) {
+			       const char *alert_origin, const char *alert_target, bool ignore_disabled) {
+  if(ignore_disabled || !ntop->getPrefs()->are_alerts_disabled()) {
     char query[STORE_MANAGER_MAX_QUERY];
     sqlite3_stmt *stmt = NULL;
     int rc = 0;
+    time_t now = time(NULL);
 
     if(!store_initialized || !store_opened)
       return -1;
@@ -329,7 +343,7 @@ int AlertsManager::engageAlert(AlertEngine alert_engine, AlertEntity alert_entit
       if(sqlite3_prepare(db, query, -1,  &stmt, 0)
 	 || sqlite3_bind_text(stmt,  1,  engaged_alert_id, -1, SQLITE_STATIC)
 	 || sqlite3_bind_int(stmt,   2,  static_cast<int>(alert_engine))
-	 || sqlite3_bind_int64(stmt, 3,  static_cast<long int>(time(NULL)))
+	 || sqlite3_bind_int64(stmt, 3,  static_cast<long int>(now))
 	 || sqlite3_bind_int(stmt,   4,  static_cast<int>(alert_type))
 	 || sqlite3_bind_int(stmt,   5,  static_cast<int>(alert_severity))
 	 || sqlite3_bind_int(stmt,   6,  static_cast<int>(alert_entity))
@@ -355,14 +369,9 @@ int AlertsManager::engageAlert(AlertEngine alert_engine, AlertEntity alert_entit
       if(stmt) sqlite3_finalize(stmt);
       m.unlock(__FILE__, __LINE__);
 
-      notifySlack(alert_entity, alert_entity_value, engaged_alert_id,
+      notifyAlert(alert_entity, alert_entity_value, engaged_alert_id,
 		  alert_type, alert_severity, alert_json,
-		  alert_origin, alert_target, true);
-
-#ifndef WIN32
-      if(ntop->getPrefs()->are_alerts_syslog_enabled())
-	syslog(LOG_WARNING, "[Alert] [ENGAGED] %s", alert_json ? alert_json : (char*)"");
-#endif
+		  alert_origin, alert_target, true, now, NULL);
     }
 
     return rc;
@@ -374,8 +383,8 @@ int AlertsManager::engageAlert(AlertEngine alert_engine, AlertEntity alert_entit
 
 int AlertsManager::releaseAlert(AlertEngine alert_engine,
 				AlertEntity alert_entity, const char *alert_entity_value,
-				const char *engaged_alert_id) {
-  if(!ntop->getPrefs()->are_alerts_disabled()) {
+				const char *engaged_alert_id, bool ignore_disabled) {
+  if(ignore_disabled || !ntop->getPrefs()->are_alerts_disabled()) {
     char query[STORE_MANAGER_MAX_QUERY];
     sqlite3_stmt *stmt = NULL;
     int rc = 0;
@@ -397,14 +406,9 @@ int AlertsManager::releaseAlert(AlertEngine alert_engine,
     if(getNetworkInterface())
       getNetworkInterface()->decAlertLevel();
 
-#ifndef WIN32
-    if(ntop->getPrefs()->are_alerts_syslog_enabled())
-      syslog(LOG_WARNING, "[Alert] [RELEASED] %s", alert_json ? alert_json : (char*)"");
-#endif
-
-    notifySlack(alert_entity, alert_entity_value, engaged_alert_id,
+    notifyAlert(alert_entity, alert_entity_value, engaged_alert_id,
           alert_type, alert_severity, alert_json,
-          alert_origin, alert_target, false);
+          alert_origin, alert_target, false, time(NULL), NULL);
   
     /* Move the alert from engaged to closed */
     snprintf(query, sizeof(query),
@@ -486,178 +490,94 @@ int AlertsManager::releaseAlert(AlertEngine alert_engine,
 
 /* **************************************************** */
 
-const char* AlertsManager::getAlertEntity(AlertEntity alert_entity) {
-  switch (alert_entity) {
-  case alert_entity_interface:   return("#interface");
-  case alert_entity_host:        return("#host");
-  case alert_entity_network:     return("#network");
-  case alert_entity_snmp_device: return("#snmp");
-  case alert_entity_flow:        return("#flow");
-  }
-
-  return(""); /* NOTREACHED */
-}
-
-/* **************************************************** */
-
-const char* AlertsManager::getAlertLevel(AlertLevel alert_severity) {
-  switch(alert_severity) {
-  case alert_level_info:    return(":information_source:");
-  case alert_level_warning: return(":warning:");
-  case alert_level_error:   return(":exclamation:");
-  }
-
-  return(""); /* NOTREACHED */
-}
-
-/* **************************************************** */
-
-/* Note: keep in sync with lua  */
-const char* AlertsManager::getAlertType(AlertType alert_type) {
-  switch(alert_type) {
-  case alert_none:                   return("No alert");
-  case alert_syn_flood:              return("TCP SYN Flood");
-  case alert_flow_flood:             return("Flows Flood");
-  case alert_threshold_exceeded:     return("Threshold Cross");
-  case alert_dangerous_host:         return("Blacklisted Host");
-  case alert_periodic_activity:      return("Periodic activity");
-  case alert_quota:                  return("Quota Exceeded");
-  case alert_malware_detection:      return("Malware Detected");
-  case alert_host_under_attack:      return("Under Attack");
-  case alert_host_attacker:          return("Ongoing Attacker");
-  case alert_app_misconfiguration:   return("Misconfigured App");
-  case alert_suspicious_activity:    return("Suspicious Activity");
-  case alert_too_many_alerts:        return("Too Many Alerts");
-  case alert_db_misconfiguration:    return("MySQL open_files_limit too small");
-  case alert_interface_alerted:      return("Interface Alerted");
-  case alert_flow_misbehaviour:      return("Flow Misbehaviour");
-  }
-
-  return(""); /* NOTREACHED */
-}
-
-/* **************************************************** */
-
-SlackNotificationChoice AlertsManager::getSlackNotificationChoice(char* choice) {
-  if(strcmp(choice, "only_errors") == 0)         return notify_errors_only;
-  if(strcmp(choice, "errors_and_warnings") == 0) return notify_errors_and_warnings;
-
-  return notify_all_alerts; /* default choice */
-}
-
-/* **************************************************** */
-
-void AlertsManager::notifyAlert(AlertEntity alert_entity, const char *alert_entity_value,
+bool AlertsManager::notifyAlert(AlertEntity alert_entity, const char *alert_entity_value,
 				const char *engaged_alert_id,
 				AlertType alert_type, AlertLevel alert_severity,
 				const char *alert_json,
 				const char *alert_origin, const char *alert_target,
-        bool engage) {
-  if(!ntop->getPrefs()->are_alerts_disabled()) {
+				bool engage, time_t when, Flow *flow) {
+  bool rv = false;
+
+  if(!ntop->getPrefs()->are_alerts_disabled()
+	  && ntop->getPrefs()->are_ext_alerts_notifications_enabled()) {
     json_object *notification;
-    char alert_sender_name[64], message[1024], notification_username[96];
-    const char *json_alert, *level;
+    const char *json_alert;
 
-    if((notification = json_object_new_object()) == NULL) return;
+    if((notification = json_object_new_object()) != NULL) {
+      /* Mandatory information */
+      json_object_object_add(notification, "ifid", json_object_new_int(iface->get_id()));
+      json_object_object_add(notification, "entity_type", json_object_new_int(alert_entity));
+      json_object_object_add(notification, "entity_value", json_object_new_string(alert_entity_value));
+      json_object_object_add(notification, "type", json_object_new_int(alert_type));
+      json_object_object_add(notification, "severity", json_object_new_int(alert_severity));
+      json_object_object_add(notification, "message", json_object_new_string(alert_json));
+      json_object_object_add(notification, "tstamp",  json_object_new_int64(when));
+      json_object_object_add(notification, "action",
+			     json_object_new_string(
+						    engaged_alert_id ? (engage ? ALERT_ACTION_ENGAGE : ALERT_ACTION_RELEASE)
+						    : ALERT_ACTION_STORE)
+			     );
+      
+      /* optional */
+      if(alert_origin) json_object_object_add(notification, "origin", json_object_new_string(alert_origin));
+      if(alert_target) json_object_object_add(notification, "target", json_object_new_string(alert_target));
+      if(engaged_alert_id) json_object_object_add(notification, "alert_key", json_object_new_string(engaged_alert_id));
 
-    json_object_object_add(notification, "channel",
-			   json_object_new_string(getAlertEntity(alert_entity)));
-    json_object_object_add(notification, "icon_emoji",
-			   json_object_new_string(getAlertLevel(alert_severity)));
+      /* flow only - only put relevant information for message generation */
+      if(flow) {
+	json_object *flow_obj;
 
-    if(ntop->getRedis()->get((char*)ALERTS_MANAGER_SENDER_USERNAME,
-			     alert_sender_name, sizeof(alert_sender_name)) >= 0) {    
-      switch(alert_severity) {
-      case alert_level_error:   level = "ERROR";   break;
-      case alert_level_warning: level = "WARNING"; break;
-      case alert_level_info:    level = "INFO";    break;
+	if((flow_obj = json_object_new_object()) == NULL) {
+	  ntop->getTrace()->traceEvent(TRACE_ERROR, "json_object_new_object: Not enough memory");
+	  goto notify_return;
+	}
+
+	char cli_ip_buf[64], srv_ip_buf[64];
+	char *cli_ip = NULL, *srv_ip = NULL;
+	Host *cli = flow->get_cli_host();
+	Host *srv = flow->get_srv_host();
+
+	if(cli && cli->get_ip())
+	  cli_ip = cli->get_ip()->print(cli_ip_buf, sizeof(cli_ip_buf));
+	if(srv && srv->get_ip())
+	  srv_ip = srv->get_ip()->print(srv_ip_buf, sizeof(srv_ip_buf));
+
+	/* mandatory */
+	json_object_object_add(flow_obj, "cli_port", json_object_new_int(flow->get_cli_port()));
+	json_object_object_add(flow_obj, "srv_port", json_object_new_int(flow->get_srv_port()));
+	json_object_object_add(flow_obj, "cli_blacklisted", json_object_new_int((cli && cli->isBlacklisted()) ? 1 : 0));
+	json_object_object_add(flow_obj, "srv_blacklisted", json_object_new_int((srv && srv->isBlacklisted()) ? 1 : 0));
+	json_object_object_add(flow_obj, "vlan_id", json_object_new_int(flow->get_vlan_id()));
+	json_object_object_add(flow_obj, "proto", json_object_new_int(flow->get_protocol()));
+	json_object_object_add(flow_obj, "flow_status", json_object_new_int((int)flow->getFlowStatus()));
+	json_object_object_add(flow_obj, "l7_proto", json_object_new_int(flow->get_detected_protocol().app_protocol));
+
+	/* optional */
+	if(cli_ip) json_object_object_add(flow_obj, "cli_addr", json_object_new_string(cli_ip));
+	if(srv_ip) json_object_object_add(flow_obj, "srv_addr", json_object_new_string(srv_ip));
+
+	json_object_object_add(notification, "flow", flow_obj);
       }
 
-      snprintf(notification_username, sizeof(notification_username),
-	       "%s [%s]", alert_sender_name, level);
+      json_alert = json_object_to_json_string(notification);
 
-      json_object_object_add(notification, "username",
-			     json_object_new_string(notification_username));
-    }
+      if(ntop->getRedis()->rpush(ALERTS_MANAGER_NOTIFICATION_QUEUE_NAME,
+				 (char*)json_alert, ALERTS_MANAGER_MAX_ENTITY_ALERTS) < 0)
+	ntop->getTrace()->traceEvent(TRACE_WARNING,
+				     "An error occurred when pushing alert %s to redis list %s.",
+				     json_alert, ALERTS_MANAGER_NOTIFICATION_QUEUE_NAME);
+      else
+	rv = true;
 
-    /* Initial information */
-    size_t len = snprintf(message, sizeof(message), "%s[%s] ",
-       engaged_alert_id ? (engage ? "Alert Engaged " : "Alert Released ") : "",
-       getAlertType(alert_type));
+notify_return:
 
-    if ((alert_json != NULL) && (alert_json[0] != '\0')) {
-      /* A message has been provided - convert HTML characters to slack format */
-      int open_tags = 0;
-      int j = len;
-
-      for (int i=0; (alert_json[i] != '\0') && (j < (int)sizeof(message)); i++) {
-        if (alert_json[i] == '<')
-          open_tags++;
-        else if (alert_json[i] == '>')
-          open_tags--;
-        else if (! open_tags)
-          message[j++] = alert_json[i];
-      }
-
-      message[j] = '\0';
-    } else {
-      /* Use a standard message */
-      snprintf(&message[len], sizeof(message)-len, "[%s][%s][Origin: %s][Target: %s]",
-        alert_entity_value ? alert_entity_value : "",
-        engaged_alert_id ? engaged_alert_id : "",
-        alert_origin ? alert_origin : "",
-        alert_target ? alert_target : "");
-    }
-
-    json_object_object_add(notification, "text", json_object_new_string(message));
-
-    json_alert = json_object_to_json_string(notification);
-
-    if(ntop->getRedis()->lpush(ALERTS_MANAGER_NOTIFICATION_QUEUE_NAME,
-			       (char*)json_alert, ALERTS_MANAGER_MAX_ENTITY_ALERTS) < 0)
-      ntop->getTrace()->traceEvent(TRACE_WARNING,
-				   "An error occurred when pushing alert %s to redis list %s.",
-				   json_alert, ALERTS_MANAGER_NOTIFICATION_QUEUE_NAME);
-
-    /* Free memory */
-    json_object_put(notification);
+      /* Free memory */
+      json_object_put(notification);
+    } else
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "json_object_new_object: Not enough memory");
   }
-}
 
-/* **************************************************** */
-
-void AlertsManager::notifySlack(AlertEntity alert_entity, const char *alert_entity_value,
-				const char *engaged_alert_id,
-				AlertType alert_type, AlertLevel alert_severity,
-				const char *alert_json,
-				const char *alert_origin, const char *alert_target,
-				bool engage) {
-  if(!ntop->getPrefs()->are_alerts_disabled()) {
-    char choice[32];
-    bool alert_to_be_notified = false;
-    SlackNotificationChoice notification_choice;
-
-    if(ntop->getPrefs()->are_slack_notification_enabled()) {
-      ntop->getRedis()->get((char*) ALERTS_MANAGER_NOTIFICATION_SEVERITY, choice, sizeof(choice));
-
-      notification_choice = getSlackNotificationChoice(choice);
-
-      if(notification_choice == notify_all_alerts)
-	alert_to_be_notified=true;
-      else if(notification_choice == notify_errors_and_warnings) {
-	if((alert_severity == alert_level_error) || (alert_severity == alert_level_warning))
-	  alert_to_be_notified = true;
-      } else {
-	if((notification_choice == notify_errors_only) && (alert_severity == alert_level_error))
-	  alert_to_be_notified = true;
-      }
-
-      if(alert_to_be_notified)
-	notifyAlert(alert_entity, alert_entity_value, engaged_alert_id,
-		    alert_type, alert_severity, alert_json,
-		    alert_origin, alert_target, engage);
-    }
-  }
+  return rv;
 }
 
 /* **************************************************** */
@@ -666,7 +586,7 @@ int AlertsManager::storeAlert(AlertEntity alert_entity, const char *alert_entity
 			      AlertType alert_type, AlertLevel alert_severity,
 			      const char *alert_json,
 			      const char *alert_origin, const char *alert_target,
-			      bool check_maximum) {
+			      bool check_maximum, time_t when) {
   if(!ntop->getPrefs()->are_alerts_disabled()) {
     char query[STORE_MANAGER_MAX_QUERY];
     sqlite3_stmt *stmt = NULL;
@@ -677,6 +597,10 @@ int AlertsManager::storeAlert(AlertEntity alert_entity, const char *alert_entity
     else if(check_maximum)
       markForMakeRoom(false);
 
+    notifyAlert(alert_entity, alert_entity_value, NULL,
+	  alert_type, alert_severity, alert_json,
+	  NULL, NULL, false, when, NULL);
+
     /* This alert is being engaged */
     snprintf(query, sizeof(query),
 	     "INSERT INTO %s "
@@ -688,7 +612,7 @@ int AlertsManager::storeAlert(AlertEntity alert_entity, const char *alert_entity
     m.lock(__FILE__, __LINE__);
 
     if(sqlite3_prepare(db, query, -1, &stmt, 0)
-       || sqlite3_bind_int64(stmt, 1, static_cast<long int>(time(NULL)))
+       || sqlite3_bind_int64(stmt, 1, static_cast<long int>(when))
        || sqlite3_bind_int(stmt,   2, static_cast<int>(alert_type))
        || sqlite3_bind_int(stmt,   3, static_cast<int>(alert_severity))
        || sqlite3_bind_int(stmt,   4, static_cast<int>(alert_entity))
@@ -715,11 +639,6 @@ int AlertsManager::storeAlert(AlertEntity alert_entity, const char *alert_entity
     if(stmt) sqlite3_finalize(stmt);
     m.unlock(__FILE__, __LINE__);
 
-#ifndef WIN32
-    if(ntop->getPrefs()->are_alerts_syslog_enabled())
-      syslog(LOG_WARNING, "[Alert] %s", alert_json ? alert_json : (char*)"");
-#endif
-
     return rc;
   } else
     return(-1);
@@ -727,31 +646,49 @@ int AlertsManager::storeAlert(AlertEntity alert_entity, const char *alert_entity
 
 /* **************************************************** */
 
-int AlertsManager::storeFlowAlert(Flow *f, AlertType alert_type,
-				  AlertLevel alert_severity,
-				  const char *alert_json) {
+int AlertsManager::storeFlowAlert(Flow *f) {
   if(!ntop->getPrefs()->are_alerts_disabled()) {
+    const char *alert_json;
+    char cli_ip_buf[64], srv_ip_buf[64];
     char query[STORE_MANAGER_MAX_QUERY];
     sqlite3_stmt *stmt = NULL;
     int rc = 0;
     Host *cli, *srv;
-    char *cli_ip = NULL, *cli_ip_buf = NULL, *srv_ip = NULL, *srv_ip_buf = NULL,
-      cb[64], cb1[64];
-    
+    char *cli_ip = NULL, *srv_ip = NULL;
+    char cb[64], cb1[64];
+    const char *msg;
+    AlertType alert_type;
+    AlertLevel alert_severity;
+    time_t now = time(NULL);
+    char *info;
+    json_object *alert_json_obj;
+ 
     if(!store_initialized || !store_opened || !f)
       return(-1);
 
     markForMakeRoom(true);
 
-    cli = f->get_cli_host(), srv = f->get_srv_host();
-    if(cli && cli->get_ip() && (cli_ip_buf = (char*)malloc(sizeof(char) * 256)))
-      cli_ip = cli->get_ip()->print(cli_ip_buf, 256);
-    if(srv && srv->get_ip() && (srv_ip_buf = (char*)malloc(sizeof(char) * 256)))
-      srv_ip = srv->get_ip()->print(srv_ip_buf, 256);
+    info = f->getFlowInfo();
 
-    notifySlack(alert_entity_flow, "flow", NULL,
+    msg = Utils::flowStatus2str(f->getFlowStatus(), &alert_type, &alert_severity);
+    cli = f->get_cli_host(), srv = f->get_srv_host();
+    if(cli && cli->get_ip())
+      cli_ip = cli->get_ip()->print(cli_ip_buf, sizeof(cli_ip_buf));
+    if(srv && srv->get_ip())
+      srv_ip = srv->get_ip()->print(srv_ip_buf, sizeof(srv_ip_buf));
+
+    json_object *status_info = f->flow2statusinfojson();
+
+    if((alert_json_obj = json_object_new_object()) == NULL)
+      return(-1);
+
+    json_object_object_add(alert_json_obj, "info", json_object_new_string(info ? info : (char*)""));
+    json_object_object_add(alert_json_obj, "status_info", status_info ? status_info : json_object_new_object());
+    alert_json = json_object_to_json_string(alert_json_obj);
+
+    notifyAlert(alert_entity_flow, "flow", NULL,
 		alert_type, alert_severity, alert_json,
-		cli_ip, srv_ip, false);
+		cli_ip, srv_ip, false, now, f);
 
     // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s %s", cli_ip, srv_ip);
     /* TODO: implement check maximum for flow alerts
@@ -769,8 +706,8 @@ int AlertsManager::storeFlowAlert(Flow *f, AlertType alert_type,
 	     "cli2srv_bytes, srv2cli_bytes, "
 	     "cli2srv_packets, srv2cli_packets, "
 	     "cli2srv_tcpflags, srv2cli_tcpflags, cli_blacklisted, srv_blacklisted, "
-	     "cli_localhost, srv_localhost) "
-	     "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ",
+	     "cli_localhost, srv_localhost, cli_host_pool_id, srv_host_pool_id, flow_status) "
+	     "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?); ",
 	     ALERTS_MANAGER_FLOWS_TABLE_NAME);
 
     m.lock(__FILE__, __LINE__);
@@ -781,7 +718,7 @@ int AlertsManager::storeFlowAlert(Flow *f, AlertType alert_type,
       goto out;
     }
 
-    if(sqlite3_bind_int64(stmt, 1, static_cast<long int>(time(NULL)))
+    if(sqlite3_bind_int64(stmt, 1, static_cast<long int>(now))
        || sqlite3_bind_int(stmt,   2, (int)(alert_type))
        || sqlite3_bind_int(stmt,   3, (int)(alert_severity))
        || sqlite3_bind_text(stmt,  4, alert_json, -1, SQLITE_STATIC)
@@ -810,6 +747,9 @@ int AlertsManager::storeFlowAlert(Flow *f, AlertType alert_type,
        || sqlite3_bind_int(stmt,  27, (srv && srv->isBlacklisted()) ? 1 : 0)
        || sqlite3_bind_int(stmt,  28, (cli && cli->isLocalHost()) ? 1 : 0)
        || sqlite3_bind_int(stmt,  29, (srv && srv->isLocalHost()) ? 1 : 0)
+       || sqlite3_bind_int(stmt,  30, cli ? cli->get_host_pool() : 0)
+       || sqlite3_bind_int(stmt,  31, srv ? srv->get_host_pool() : 0)
+       || sqlite3_bind_int(stmt,  32, (int)f->getFlowStatus())
        ) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to bind to arguments to %s", query);
       rc = 2;
@@ -828,18 +768,14 @@ int AlertsManager::storeFlowAlert(Flow *f, AlertType alert_type,
     alerts_stored = true;
     rc = 0;
   out:
-    if(cli_ip_buf) free(cli_ip_buf);
-    if(srv_ip_buf) free(srv_ip_buf);
 
     if(stmt) sqlite3_finalize(stmt);
     m.unlock(__FILE__, __LINE__);
 
     f->setFlowAlerted();
-  
-#ifndef WIN32
-    if(ntop->getPrefs()->are_alerts_syslog_enabled())
-      syslog(LOG_WARNING, "[Alert] %s", alert_json ? alert_json : (char*)"");
-#endif
+
+    ntop->getTrace()->traceEvent(TRACE_INFO, "[%s] %s", msg, alert_json);
+    json_object_put(alert_json_obj);
 
     return rc;
   } else
@@ -868,7 +804,7 @@ int AlertsManager::engageReleaseHostAlert(const char *host_ip, u_int16_t host_vl
 					  const char *engaged_alert_id,
 					  AlertType alert_type, AlertLevel alert_severity, const char *alert_json,
 					  const char *alert_origin, const char *alert_target,
-					  bool engage) {
+					  bool engage, bool ignore_disabled) {
   char counters_key[64], ipbuf_id[64], rsp[16];
   int rc;
   Host *h;
@@ -876,13 +812,17 @@ int AlertsManager::engageReleaseHostAlert(const char *host_ip, u_int16_t host_vl
   snprintf(ipbuf_id, sizeof(ipbuf_id), "%s@%d", host_ip, host_vlan);
   int num_alerts;
 
+  // If alerts are disabled, we must return now
+  if(!ignore_disabled && ntop->getPrefs()->are_alerts_disabled())
+    return 0;
+
   if(engage) {
     rc = engageAlert(alert_engine, alert_entity_host, ipbuf_id,
 		     engaged_alert_id, alert_type, alert_severity, alert_json,
-		     alert_origin, alert_target);
+		     alert_origin, alert_target, ignore_disabled);
   } else {
     rc = releaseAlert(alert_engine, alert_entity_host, ipbuf_id,
-		      engaged_alert_id);
+		      engaged_alert_id, ignore_disabled);
   }
 
   if (rc != 0)
@@ -924,57 +864,6 @@ int AlertsManager::engageReleaseHostAlert(const char *host_ip, u_int16_t host_vl
 
 /* ******************************************* */
 
-int AlertsManager::engageReleaseNetworkAlert(const char *cidr,
-					     AlertEngine alert_engine,
-					     const char *engaged_alert_id,
-					     AlertType alert_type, AlertLevel alert_severity,
-					     const char *alert_json, bool engage) {
-  struct in_addr addr4;
-  struct in6_addr addr6;
-  char ip_buf[256];
-  char *slash;
-
-  if(!cidr) return -1;
-
-  strncpy(ip_buf, cidr, sizeof(ip_buf));
-  if((slash = strchr(ip_buf, '/')) == NULL) return -2;
-  slash[0] = '\0';
-
-  if(inet_pton(AF_INET, ip_buf, &addr4) != 1 && inet_pton(AF_INET6, ip_buf, &addr6) != 1) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Error parsing network %s\n", cidr);
-    return -2; /* not a valid network */
-  }
-
-  if(engage)
-    return engageAlert(alert_engine, alert_entity_network, cidr,
-		       engaged_alert_id, alert_type, alert_severity, alert_json, NULL, NULL);
-  else
-    return releaseAlert(alert_engine, alert_entity_network, cidr,
-			engaged_alert_id);
-};
-
-/* ******************************************* */
-
-int AlertsManager::engageReleaseInterfaceAlert(NetworkInterface *n,
-					       AlertEngine alert_engine,
-					       const char *engaged_alert_id,
-					       AlertType alert_type, AlertLevel alert_severity, const char *alert_json,
-					       bool engage) {
-  char id_buf[8];
-  if(!n) return -1;
-
-  snprintf(id_buf, sizeof(id_buf), "iface_%u", n -> get_id());
-
-  if(engage)
-    return engageAlert(alert_engine, alert_entity_interface, id_buf,
-		       engaged_alert_id, alert_type, alert_severity, alert_json, NULL, NULL);
-  else
-    return releaseAlert(alert_engine, alert_entity_interface, id_buf,
-			engaged_alert_id);
-};
-
-/* ******************************************* */
-
 int AlertsManager::storeHostAlert(Host *h,
 				  AlertType alert_type, AlertLevel alert_severity, const char *alert_json,
 				  Host *alert_origin, Host *alert_target) {
@@ -983,10 +872,10 @@ int AlertsManager::storeHostAlert(Host *h,
   if(!isValidHost(h, ipbuf_id, sizeof(ipbuf_id)))
     return -1;
 
-  return storeAlert(alert_entity_host, ipbuf_id, alert_type, alert_severity, alert_json,
+  return(storeAlert(alert_entity_host, ipbuf_id, alert_type, alert_severity, alert_json,
 		    isValidHost(alert_origin, ipbuf_origin, sizeof(ipbuf_origin)) ? ipbuf_origin : NULL,
 		    isValidHost(alert_target, ipbuf_target, sizeof(ipbuf_target)) ? ipbuf_target : NULL,
-		    true);
+		    true, time(NULL)));
 };
 
 /* ******************************************* */
@@ -1022,7 +911,7 @@ static int getAlertsCallback(void *data, int argc, char **argv, char **azColName
     lua_push_str_table_entry(vm, azColName[i], argv[i]);
   }
 
-  lua_pushnumber(vm, ++ar->current_offset);
+  lua_pushinteger(vm, ++ar->current_offset);
   lua_insert(vm, -2);
   lua_settable(vm, -3);
 
@@ -1122,7 +1011,7 @@ int AlertsManager::getNumFlowAlerts(const char *sql_where_clause) {
 int AlertsManager::getCachedNumAlerts(lua_State *vm) {
   lua_newtable(vm);
 
-  lua_push_int_table_entry(vm, "num_alerts_engaged", num_alerts_engaged);
+  lua_push_uint64_table_entry(vm, "num_alerts_engaged", num_alerts_engaged);
   lua_push_bool_table_entry(vm, "alerts_stored", alerts_stored);
 
   return 0;
