@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2015-17 - ntop.org
+ * (C) 2015-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,17 +27,26 @@
 
 HostPools::HostPools(NetworkInterface *_iface) {
   tree = tree_shadow = NULL;
+#ifdef NTOPNG_PRO
+  children_safe = forge_global_dns = NULL;
+  routing_policy_id = NULL;
+#endif
 
 #ifdef NTOPNG_PRO
   if((children_safe = (bool*)calloc(MAX_NUM_HOST_POOLS, sizeof(bool))) == NULL)
     throw 1;
 
+  if((forge_global_dns = (bool*)calloc(MAX_NUM_HOST_POOLS, sizeof(bool))) == NULL)
+    throw 1;
+
   if((routing_policy_id = (u_int8_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int8_t))) == NULL)
     throw 1;
 
+  for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) routing_policy_id[i] = DEFAULT_ROUTING_TABLE_ID;
+
   stats = stats_shadow = NULL;
 
-  if((volatile_members = (volatile_members_t**)calloc(MAX_NUM_HOST_POOLS, sizeof(volatile_members_t))) == NULL
+  if((volatile_members = (volatile_members_t**)calloc(MAX_NUM_HOST_POOLS, sizeof(volatile_members_t*))) == NULL
      || (volatile_members_lock            = new Mutex*[MAX_NUM_HOST_POOLS]) == NULL
      || (pool_shaper = (u_int16_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int16_t))) == NULL
      || (schedule_bitmap = (u_int32_t*)calloc(MAX_NUM_HOST_POOLS, sizeof(u_int32_t))) == NULL
@@ -77,20 +86,6 @@ HostPools::HostPools(NetworkInterface *_iface) {
 
 /* *************************************** */
 
-void HostPools::deleteTree(AddressTree ***at) {
-  if(at) {
-    if(*at) {
-      for(int i = 0; i < MAX_NUM_VLAN; i++)
-	if((*at)[i])
-	  delete (*at)[i];
-      delete [] *at;
-      *at = NULL;
-    }
-  }
-}
-
-/* *************************************** */
-
 #ifdef NTOPNG_PRO
 
 void HostPools::deleteStats(HostPoolStats ***hps) {
@@ -119,12 +114,13 @@ HostPools::~HostPools() {
   if(num_active_l2_devices_offline)
     free(num_active_l2_devices_offline);
 
-  if(tree_shadow)   deleteTree(&tree_shadow);
-  if(tree)          deleteTree(&tree);
+  if(tree_shadow)   delete tree_shadow;
+  if(tree)          delete tree;
   if(swap_lock)     delete swap_lock;
 
 #ifdef NTOPNG_PRO
   if(children_safe)     free(children_safe);
+  if(forge_global_dns)  free(forge_global_dns);
   if(routing_policy_id) free(routing_policy_id);
 
   dumpToRedis();
@@ -173,9 +169,9 @@ HostPools::~HostPools() {
 /* *************************************** */
 
 #ifdef NTOPNG_PRO
-void HostPools::swap(AddressTree **new_trees, HostPoolStats **new_stats) {
+void HostPools::swap(VlanAddressTree *new_trees, HostPoolStats **new_stats) {
 #else
-void HostPools::swap(AddressTree **new_trees) {
+void HostPools::swap(VlanAddressTree *new_trees) {
 #endif
   swap_lock->lock(__FILE__, __LINE__);
 
@@ -200,7 +196,7 @@ void HostPools::swap(AddressTree **new_trees) {
   /* Swap address trees */
   if(new_trees) {
     if(tree) {
-      if(tree_shadow) deleteTree(&tree_shadow); /* Invokes the destructor */
+      if(tree_shadow) delete tree_shadow;
       tree_shadow = tree;
     }
 
@@ -215,7 +211,7 @@ void HostPools::swap(AddressTree **new_trees) {
 
 #ifdef NTOPNG_PRO
 
-void HostPools::reloadVolatileMembers(AddressTree **_trees) {
+void HostPools::reloadVolatileMembers(VlanAddressTree *_trees) {
   volatile_members_t *current, *tmp;
   char *at, *member;
   bool rc;
@@ -241,19 +237,17 @@ void HostPools::reloadVolatileMembers(AddressTree **_trees) {
 	} else
 	  vlan_id = 0;
 
-	if(_trees[vlan_id]  || (_trees[vlan_id] = new AddressTree())) {
-	  if(!(rc = _trees[vlan_id]->addAddress(member, pool_id))
+	if(!(rc = _trees->addAddress(vlan_id, member, pool_id))
 #ifdef HOST_POOLS_DEBUG
-	     || true
+	    || true
 #endif
 	     )
-	    ntop->getTrace()->traceEvent(TRACE_NORMAL,
-					 "%s VOLATILE tree node for %s [vlan %i] [host pool: %i]",
-					 rc ? "Successfully added" : "Unable to add",
-					 member, vlan_id,
-					 pool_id);
+	  ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				       "%s VOLATILE tree node for %s [vlan %i] [host pool: %i]",
+				       rc ? "Successfully added" : "Unable to add",
+				       member, vlan_id,
+				       pool_id);
 
-	}
 
 	free(member);
       }
@@ -267,6 +261,12 @@ void HostPools::reloadVolatileMembers(AddressTree **_trees) {
 
     volatile_members_lock[pool_id]->unlock(__FILE__, __LINE__);
   }
+
+#ifdef HAVE_NEDGE
+  /* Note: we must re-evaluate the active flows as a captive portal host may be blocked now */
+  if(iface && (iface->getIfType() == interface_type_NETFILTER))
+    ((NetfilterInterface *) iface)->setPolicyChanged();
+#endif
 };
 
 /* *************************************** */
@@ -306,10 +306,10 @@ void HostPools::dumpToRedis() {
 
   if((!redis) || (! stats) || (! iface)) return;
 
-  snprintf(key, sizeof(key), HOST_POOL_DUMP_KEY, iface->get_id());
+  snprintf(key, sizeof(key), HOST_POOL_SERIALIZED_KEY, iface->get_id());
 
   for(int i = 0; i<MAX_NUM_HOST_POOLS; i++) {
-    if(stats[i]) {
+    if(stats[i] && !stats[i]->needsReset()) {
       snprintf(buf, sizeof(buf), "%d", i);
       char *value = stats[i]->serialize(iface);
 
@@ -319,6 +319,11 @@ void HostPools::dumpToRedis() {
       }
     }
   }
+
+  // Save the deadline time for quota expiration, assuming quota is reset at midnight
+  snprintf(buf, sizeof(buf), "%u",
+	   Utils::roundTime(time(0), 86400, ntop->get_time_offset()) - 86400);
+  redis->hashSet(key, (char *)"deadline", buf);
 }
 
 /* *************************************** */
@@ -328,11 +333,19 @@ void HostPools::loadFromRedis() {
   json_object *obj;
   enum json_tokener_error jerr = json_tokener_success;
   Redis *redis = ntop->getRedis();
+  time_t deadline = 0;
 
-  snprintf(key, sizeof(key), HOST_POOL_DUMP_KEY, iface->get_id());
+  snprintf(key, sizeof(key), HOST_POOL_SERIALIZED_KEY, iface->get_id());
 
   if((!redis) || (!stats) || (!iface)) return;
 
+  if(redis->hashGet(key, (char *)"deadline", buf, sizeof(buf)) == 0) {
+    sscanf(buf, "%lu", &deadline);
+
+    if(time(0) > deadline)
+      return; /* Expired */
+  }
+
   if((value = (char *) malloc(POOL_MAX_SERIALIZED_LEN)) == NULL) {
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to allocate memory to deserialize %s", key);
     return;
@@ -377,7 +390,8 @@ void HostPools::incPoolStats(u_int32_t when, u_int16_t host_pool_id, u_int16_t n
   if(!hps) return;
 
   /* Important to use the assigned hps as a swap can make stats[host_pool_id] NULL */
-  hps->incStats(when, ndpi_proto, category_id, sent_packets, sent_bytes, rcvd_packets, rcvd_bytes);
+  hps->incStats(when, ndpi_proto, sent_packets, sent_bytes, rcvd_packets, rcvd_bytes);
+  hps->incCategoryStats(when, category_id, sent_bytes, rcvd_bytes);
 };
 
 /* *************************************** */
@@ -415,15 +429,36 @@ void HostPools::luaStats(lua_State *vm) {
 
 /* *************************************** */
 
-void HostPools::resetPoolsStats() {
-    HostPoolStats *hps;
+void HostPools::resetPoolsStats(u_int16_t pool_filter) {
+  HostPoolStats *hps;
+
+  if(stats) {
+    if(pool_filter != (u_int16_t)-1) {
+      if((hps = getPoolStats(pool_filter)))
+	hps->resetStats();
+    } else {
+      for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
+	if((hps = stats[i])) {
+	  /* Must use the assigned hps as stats can be swapped
+	     and accesses such as stats[i] could yield a NULL value */
+	  hps->resetStats();
+	}
+      }
+    }
+  }
+}
+
+/* *************************************** */
+
+void HostPools::checkPoolsStatsReset() {
+  HostPoolStats *hps;
 
   if(stats) {
     for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
       if((hps = stats[i])) {
 	/* Must use the assigned hps as stats can be swapped
 	   and accesses such as stats[i] could yield a NULL value */
-	hps->resetStats();
+	hps->checkStatsReset();
       }
     }
   }
@@ -575,15 +610,13 @@ void HostPools::lua(lua_State *vm) {
   lua_newtable(vm);
 
   for(int i = 0; i < MAX_NUM_HOST_POOLS; i++) {
-    if(i == NO_HOST_POOL_ID) continue;
-
     if((cur_hosts = getNumPoolHosts(i)))  hosts += cur_hosts;
     if((cur_l2 = getNumPoolL2Devices(i))) l2_devices += cur_l2;
 
     if(cur_hosts || cur_l2) {
       lua_newtable(vm);
-      lua_push_int_table_entry(vm, "num_hosts", cur_hosts);
-      lua_push_int_table_entry(vm, "num_l2_devices", cur_l2);
+      lua_push_uint64_table_entry(vm, "num_hosts", cur_hosts);
+      lua_push_uint64_table_entry(vm, "num_l2_devices", cur_l2);
       snprintf(buf, sizeof(buf), "%d", i);
 
       lua_pushstring(vm, buf);
@@ -599,9 +632,9 @@ void HostPools::lua(lua_State *vm) {
   lua_settable(vm, -3);
 
   lua_newtable(vm);
-  lua_push_int_table_entry(vm, "num_hosts", hosts);
-  lua_push_int_table_entry(vm, "num_l2_devices", l2_devices);
-  lua_push_int_table_entry(vm, "num_active_pools", active_pools);
+  lua_push_uint64_table_entry(vm, "num_hosts", hosts);
+  lua_push_uint64_table_entry(vm, "num_l2_devices", l2_devices);
+  lua_push_uint64_table_entry(vm, "num_active_pools", active_pools);
 
   lua_pushstring(vm, "num_members");
   lua_insert(vm, -2);
@@ -613,9 +646,9 @@ void HostPools::lua(lua_State *vm) {
 void HostPools::reloadPools() {
   char kname[CONST_MAX_LEN_REDIS_KEY];
   char **pools, **pool_members, *at, *member;
-  int num_pools, num_members;
+  int num_pools, num_members, actual_num_members;
   u_int16_t _pool_id, vlan_id;
-  AddressTree **new_tree;
+  VlanAddressTree *new_tree;
 #ifdef NTOPNG_PRO
   HostPoolStats **new_stats;
 #endif
@@ -624,7 +657,7 @@ void HostPools::reloadPools() {
   if(!iface || (iface->get_id() == -1))
     return;
 
-  if((new_tree = new AddressTree*[MAX_NUM_VLAN]) == NULL
+  if((new_tree = new VlanAddressTree) == NULL
 #ifdef NTOPNG_PRO
      || (new_stats = new HostPoolStats*[MAX_NUM_HOST_POOLS]) == NULL
 #endif
@@ -633,9 +666,6 @@ void HostPools::reloadPools() {
     return;
   }
 
-  for(u_int32_t i = 0; i < MAX_NUM_VLAN; i++)
-    new_tree[i] = NULL;
-
 #ifdef NTOPNG_PRO
   for(u_int32_t i = 0; i < MAX_NUM_HOST_POOLS; i++)
     new_stats[i] = NULL;
@@ -655,14 +685,16 @@ void HostPools::reloadPools() {
   num_pools = redis->smembers(kname, &pools);
 
   for(int i = 0; i < num_pools; i++) {
-
-
     if(!pools[i])
       continue;
 
     _pool_id = (u_int16_t)atoi(pools[i]);
-    if(_pool_id >= MAX_NUM_HOST_POOLS)
+    if(_pool_id >= MAX_NUM_HOST_POOLS) {
+      ntop->getTrace()->traceEvent(TRACE_WARNING, "Ignoring pool [pool id: %2d]. "
+				   "Maximum number of host pools for this license is %u, inclusive of the Not Assigned pool.",
+				   _pool_id, MAX_NUM_HOST_POOLS);
       continue;
+    }
 
 #ifdef NTOPNG_PRO
     if(_pool_id != 0) { /* Pool id 0 stats already updated */
@@ -673,34 +705,40 @@ void HostPools::reloadPools() {
     }
 #endif
 
-    snprintf(kname, sizeof(kname), HOST_POOL_DETAILS_KEY, iface->get_id(), i);
+    snprintf(kname, sizeof(kname), HOST_POOL_DETAILS_KEY, iface->get_id(), _pool_id);
 
 #ifdef NTOPNG_PRO
     char rsp[16] = { 0 };
 
-    children_safe[i] = ((redis->hashGet(kname, (char*)CONST_CHILDREN_SAFE, rsp, sizeof(rsp)) != -1)
+    children_safe[_pool_id] = ((redis->hashGet(kname, (char*)CONST_CHILDREN_SAFE, rsp, sizeof(rsp)) != -1)
 			&& (!strcmp(rsp, "true")));
 
-    routing_policy_id[i] = (redis->hashGet(kname, (char*)CONST_ROUTING_POLICY_ID, rsp, sizeof(rsp)) != -1) ? atoi(rsp) : DEFAULT_ROUTING_TABLE_ID;
-    pool_shaper[i] = (redis->hashGet(kname, (char*)CONST_POOL_SHAPER_ID, rsp, sizeof(rsp)) != -1) ? atoi(rsp) : DEFAULT_SHAPER_ID;
-    schedule_bitmap[i] = (redis->hashGet(kname, (char*)CONST_SCHEDULE_BITMAP, rsp, sizeof(rsp)) != -1) ? strtol(rsp, NULL, 16) : DEFAULT_TIME_SCHEDULE;
+    forge_global_dns[_pool_id] = ((redis->hashGet(kname, (char*)CONST_FORGE_GLOBAL_DNS, rsp, sizeof(rsp)) != -1)
+    			&& (!strcmp(rsp, "true")));
 
-    enforce_quotas_per_pool_member[i]   = ((redis->hashGet(kname, (char*)CONST_ENFORCE_QUOTAS_PER_POOL_MEMBER, rsp, sizeof(rsp)) != -1)
+    routing_policy_id[_pool_id] = (redis->hashGet(kname, (char*)CONST_ROUTING_POLICY_ID, rsp, sizeof(rsp)) != -1) ? atoi(rsp) : DEFAULT_ROUTING_TABLE_ID;
+    pool_shaper[_pool_id] = (redis->hashGet(kname, (char*)CONST_POOL_SHAPER_ID, rsp, sizeof(rsp)) != -1) ? atoi(rsp) : DEFAULT_SHAPER_ID;
+    schedule_bitmap[_pool_id] = (redis->hashGet(kname, (char*)CONST_SCHEDULE_BITMAP, rsp, sizeof(rsp)) != -1) ? strtol(rsp, NULL, 16) : DEFAULT_TIME_SCHEDULE;
+
+    enforce_quotas_per_pool_member[_pool_id]   = ((redis->hashGet(kname, (char*)CONST_ENFORCE_QUOTAS_PER_POOL_MEMBER, rsp, sizeof(rsp)) != -1)
 					 && (!strcmp(rsp, "true")));;
-    enforce_shapers_per_pool_member[i]   = ((redis->hashGet(kname, (char*)CONST_ENFORCE_SHAPERS_PER_POOL_MEMBER, rsp, sizeof(rsp)) != -1)
+    enforce_shapers_per_pool_member[_pool_id]   = ((redis->hashGet(kname, (char*)CONST_ENFORCE_SHAPERS_PER_POOL_MEMBER, rsp, sizeof(rsp)) != -1)
 					 && (!strcmp(rsp, "true")));;
 
 #ifdef HOST_POOLS_DEBUG
     redis->hashGet(kname, (char*)"name", rsp, sizeof(rsp));
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Loading pool [name: %s]"
+    ntop->getTrace()->traceEvent(TRACE_NORMAL, "Loading pool [iteration: %u][pool_id: %u][name: %s]"
 				 "[children_safe: %i]"
+				 "[forge_global_dns: %i]"
 				 "[pool_shaper: %i]"
 				 "[schedule_bitmap: %i]"
 				 "[enforce_quotas_per_pool_member: %i]"
 				 "[enforce_shapers_per_pool_member: %i]",
-				 rsp, children_safe[i], pool_shaper[i], schedule_bitmap[i],
-				 enforce_quotas_per_pool_member[i],
-				 enforce_shapers_per_pool_member[i]);
+				 i, _pool_id,
+				 rsp, children_safe[_pool_id], forge_global_dns[_pool_id],
+				 pool_shaper[_pool_id], schedule_bitmap[_pool_id],
+				 enforce_quotas_per_pool_member[_pool_id],
+				 enforce_shapers_per_pool_member[_pool_id]);
 #endif
 
 #endif /* NTOPNG_PRO */
@@ -710,9 +748,15 @@ void HostPools::reloadPools() {
     /* Pool members are the elements of the list */
     if((num_members = redis->smembers(kname, &pool_members)) > 0) {
       // NOTE: the auto-assigned host_pool must not be limited as it receives devices assigments automatically
-      num_members = min_val((u_int32_t)num_members, ((i == ntop->getPrefs()->get_auto_assigned_pool_id()) ? MAX_NUM_INTERFACE_HOSTS : MAX_NUM_POOL_MEMBERS));
+      actual_num_members = min_val((u_int32_t)num_members, ((_pool_id == ntop->getPrefs()->get_auto_assigned_pool_id()) ? MAX_NUM_INTERFACE_HOSTS : MAX_NUM_POOL_MEMBERS));
 
-      for(int k = 0; k < num_members; k++) {
+      if(actual_num_members < num_members) {
+	ntop->getTrace()->traceEvent(TRACE_WARNING, "Too many members [pool id: %2d][pool members: %d]. "
+				     "Maximum number of pool members for this license is %u, so %u pool members will be ignored.",
+				     _pool_id, num_members, actual_num_members, num_members - actual_num_members, actual_num_members);
+      }
+
+      for(int k = 0; k < actual_num_members; k++) {
 	member = pool_members[k];
 
 	if(!member) continue;
@@ -723,21 +767,19 @@ void HostPools::reloadPools() {
 	} else
 	  vlan_id = 0;
 
-	if(new_tree[vlan_id] || (new_tree[vlan_id] = new AddressTree())) {
-	  bool rc;
+	bool rc;
 
-	  if(!(rc = new_tree[vlan_id]->addAddress(member, _pool_id))
+	if(!(rc = new_tree->addAddress(vlan_id, member, _pool_id))
 #ifdef HOST_POOLS_DEBUG
-	     || true
+	    || true
 #endif
 	     )
 
-	    ntop->getTrace()->traceEvent(TRACE_NORMAL,
-					 "%s tree node for %s [vlan %i] [host pool: %s]",
-					 rc ? "Successfully added" : "Unable to add",
-					 member, vlan_id,
-					 pools[i]);
-	}
+	  ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				       "%s tree node for %s [vlan %i] [host pool: %s]",
+				       rc ? "Successfully added" : "Unable to add",
+				       member, vlan_id,
+				       pools[i]);
 
 	free(member);
       }
@@ -763,14 +805,14 @@ void HostPools::reloadPools() {
 
 /* *************************************** */
 
-bool HostPools::findMacPool(u_int8_t *mac, u_int16_t vlan_id, u_int16_t *found_pool) {
-  AddressTree *cur_tree; /* must use this as tree can be swapped */
+bool HostPools::findMacPool(u_int8_t *mac, u_int16_t *found_pool) {
+  VlanAddressTree *cur_tree; /* must use this as tree can be swapped */
   int16_t ret;
 
-  if(!tree || !(cur_tree = tree[vlan_id]))
+  if(!tree || !(cur_tree = tree))
     return(false);
 
-  ret = cur_tree->findMac(mac);
+  ret = cur_tree->findMac(0, mac);
 
   if(ret != -1) {
     *found_pool = (u_int16_t)ret;
@@ -783,31 +825,24 @@ bool HostPools::findMacPool(u_int8_t *mac, u_int16_t vlan_id, u_int16_t *found_p
 /* *************************************** */
 
 bool HostPools::findMacPool(Mac *mac, u_int16_t *found_pool) {
-  bool found;
-
   if(mac->isSpecialMac())
     return(false);
 
-  found = findMacPool(mac->get_mac(), mac->get_vlan_id(), found_pool);
-
-  if((!found) && (mac->get_vlan_id() != 0))
-    found = findMacPool(mac->get_mac(), 0 /* No VLAN */, found_pool);
-
-  return(found);
+  return findMacPool(mac->get_mac(), found_pool);
 }
 
 /* *************************************** */
 
 bool HostPools::findIpPool(IpAddress *ip, u_int16_t vlan_id, u_int16_t *found_pool, patricia_node_t **found_node) {
-  AddressTree *cur_tree; /* must use this as tree can be swapped */
+  VlanAddressTree *cur_tree; /* must use this as tree can be swapped */
 #ifdef HOST_POOLS_DEBUG
   char buf[128];
 #endif
 
-  if(!tree || !(cur_tree = tree[vlan_id]))
+  if(!tree || !(cur_tree = tree))
     return(false);
 
-  *found_node = (patricia_node_t*)ip->findAddress(cur_tree);
+  *found_node = (patricia_node_t*)ip->findAddress(cur_tree->getAddressTree(vlan_id));
 
   if(*found_node) {
 #ifdef HOST_POOLS_DEBUG
