@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -23,6 +23,15 @@
 
 /* ******************************* */
 
+static void* resolverCheckFctn(void* ptr) {
+  MDNS *m = (MDNS*)ptr;
+
+  m->initializeResolver();
+  return(NULL);
+}
+
+/* ******************************* */
+
 MDNS::MDNS(NetworkInterface *iface) {
   if(((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
      || ((batch_udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1))
@@ -31,6 +40,21 @@ MDNS::MDNS(NetworkInterface *iface) {
   /* Multicast group is 224.0.0.251 */
   gatewayIPv4 = Utils::findInterfaceGatewayIPv4(iface->get_name());
 
+  pthread_create(&resolverCheck, NULL, resolverCheckFctn, (void*)this);
+}
+
+/* ******************************* */
+
+MDNS::~MDNS() {
+  if(udp_sock != -1)       close(udp_sock);
+  if(batch_udp_sock != -1) close(batch_udp_sock);
+
+  pthread_join(resolverCheck, NULL);
+}
+
+/* ******************************* */
+
+void MDNS::initializeResolver() {
   if(gatewayIPv4) {
     /* Let's check if this resolver is active */
     u_int dns_query_len;
@@ -65,13 +89,6 @@ MDNS::MDNS(NetworkInterface *iface) {
 
 /* ******************************* */
 
-MDNS::~MDNS() {
-  if(udp_sock != -1)       close(udp_sock);
-  if(batch_udp_sock != -1) close(batch_udp_sock);
-}
-
-/* ******************************* */
-
 u_int16_t MDNS::buildMDNSRequest(char *query, u_int8_t query_type,
 				 char *mdnsbuf, u_int mdnsbuf_len,
 				 u_int16_t tid) {
