@@ -1,6 +1,6 @@
- /*
+/*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -41,71 +41,77 @@ MacManufacturers::MacManufacturers(const char * const home) {
 /* *************************************** */
 
 void MacManufacturers::init() {
+#ifdef WIN32
+  struct _stat64 buf;
+#else
   struct stat buf;
+#endif
   FILE *fd;
-  char line[256], *manuf = NULL, *cr;
+  char line[256], *cr;
   int _mac[3];
-  u_int8_t mac[3];
-  char short_name[9];
   mac_manufacturers_t *s;
 
   if(!(stat(manufacturers_file, &buf) == 0) && (S_ISREG(buf.st_mode)))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "File %s doesn't exists or is not readable", manufacturers_file);
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "File %s doesn't exists or is not readable",
+				 manufacturers_file);
 
   if((fd = fopen(manufacturers_file, "r")) == NULL)
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to read %s", manufacturers_file);
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to read %s",
+				 manufacturers_file);
 
   if(fd) {
     while(fgets(line, sizeof(line), fd)) {
-      char *tab = strchr(line, '\t');
+      char *tmp;
+      char *mac = strtok_r(line, "\t", &tmp);
+      char *shortmanuf, *manuf;
 
-      if((sscanf(line, "%02x:%02x:%02x", &_mac[0], &_mac[1], &_mac[2]) == 3)
-	 && (tab != NULL)) {
+      if(!mac)
+	continue;
+      else
+	shortmanuf = strtok_r(NULL, "\t", &tmp);
+
+      if(!shortmanuf)
+	continue;
+      else {
+	manuf = strtok_r(NULL, "\t", &tmp);
+	if(!manuf) manuf = shortmanuf;
+      }
+
+#ifdef MANUF_DEBUG
+      ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s [short: %s][full: %s]", mac, shortmanuf, manuf);
+#endif
+				   
+      if(sscanf(mac, "%02x:%02x:%02x", &_mac[0], &_mac[1], &_mac[2]) == 3) {	
 	// printf("Retrieved line of length %zu :\n", read);
 	// printf("%s", line);
 
 	/* Lines are like:
 	   00:05:02        Apple                  # Apple, Inc.
 	   So it is possible to use '# ' as the full manufacturer name separator
-	 */
-	if((manuf = strstr(line, "# ")) == NULL)
-	  continue;
-	manuf += 2;
+	*/
+	tmp = strstr(manuf, "# ");
+	if(tmp)
+	  manuf = &tmp[2];
 
 	if((cr = strchr(manuf, '\n')))
-	   *cr = '\0';
+	  *cr = '\0';
 
 	mac[0] = (u_int8_t)_mac[0], mac[1] = (u_int8_t)_mac[1], mac[2] = (u_int8_t)_mac[2];
 	HASH_FIND(hh, mac_manufacturers, mac, 3, s);
 	if(!s) {
 	  if((s = (mac_manufacturers_t*)calloc(1, sizeof(mac_manufacturers_t))) != NULL) {
-	    int i, j;
-	    
 	    memcpy(s->mac_manufacturer, mac, 3);
-	    s->manufacturer_name = (char*)malloc(strlen(manuf)+1);
+
+	    s->manufacturer_name = (char*)calloc(strlen(manuf) + 1, sizeof(char));
+	    strcpy(s->manufacturer_name, manuf);
+	    Utils::purifyHTTPparam(s->manufacturer_name, false, false, false);
+
+	    s->short_name = (char*)calloc(strlen(shortmanuf) + 1, sizeof(char));
+	    strcpy(s->short_name, shortmanuf);
+	    Utils::purifyHTTPparam(s->short_name, false, false, false);
 
 	    /* TODO: reduce memory usage for recurrent manufacturers */
-	    if(s->manufacturer_name) {
-	      /* 
-		 We need to zap chars like ' and " that
-		 can corrupt html
-	      */
-	      for(i=0, j=0; manuf[i] != '\0'; i++) {
-		switch(manuf[i]) {
-		case '"':
-		case '\'':
-		  break;
-		  
-		default:
-		  s->manufacturer_name[j++] = manuf[i];
-		}
-	      }
 
-	      s->manufacturer_name[j++] = '\0';
-	    }
-	    
-	    sscanf(tab, "%8s", short_name);
-	    s->short_name = strdup(short_name);
 	    HASH_ADD(hh, mac_manufacturers, mac_manufacturer, 3, s);
 
 #ifdef MANUF_DEBUG
