@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@
 #include "ntop_includes.h"
 
 #ifdef HAVE_MYSQL
+
 /* **************************************************** */
 
 static void* queryLoop(void* ptr) {
@@ -43,7 +44,7 @@ void* MySQLDB::queryLoop() {
   if(ntop->getGlobals()->isShutdown() || !connectToDB(&mysql_alt, true))
     return(NULL);
 
-  while(!ntop->getGlobals()->isShutdown()) {
+  while(isRunning()) {
     int rc = r->lpop(CONST_SQL_QUEUE, sql, sizeof(sql));
 
     if(rc == 0) {
@@ -472,7 +473,8 @@ bool MySQLDB::createNprobeDBView() {
 /* ******************************************* */
 
 MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
-  mysqlDroppedFlows = 0;
+  mysqlDroppedFlows = mysqlConsumerDroppedFlows = 0;
+  mysqlEnqueuedFlows = 0;
   mysqlExportedFlows = 0, mysqlLastExportedFlows = 0;
   mysqlExportRate = 0;
   checkpointDroppedFlows = checkpointExportedFlows = 0;
@@ -483,15 +485,27 @@ MySQLDB::MySQLDB(NetworkInterface *_iface) : DB(_iface) {
 /* ******************************************* */
 
 MySQLDB::~MySQLDB() {
+  shutdown();
   disconnectFromDB(&mysql);
 }
 
 /* ******************************************* */
 
 void MySQLDB::startDBLoop() {
+  running = true;
   pthread_create(&queryThreadLoop, NULL, ::queryLoop, (void*)this);
 }
 
+/* ******************************************* */
+
+void MySQLDB::shutdown() {
+  if(running) {
+    void *res;
+
+    DB::shutdown();
+    pthread_join(queryThreadLoop, &res);
+  }
+}
 
 /* ******************************************* */
 
@@ -515,10 +529,10 @@ void MySQLDB::updateStats(const struct timeval *tv) {
 /* ******************************************* */
 
 void MySQLDB::lua(lua_State *vm, bool since_last_checkpoint) const {
-  lua_push_int_table_entry(vm, "flow_export_count",
+  lua_push_uint64_table_entry(vm, "flow_export_count",
 			   mysqlExportedFlows - (since_last_checkpoint ? checkpointExportedFlows : 0));
   lua_push_int32_table_entry(vm, "flow_export_drops",
-			     mysqlDroppedFlows - (since_last_checkpoint ? checkpointDroppedFlows : 0));
+			     (mysqlDroppedFlows + mysqlConsumerDroppedFlows) - (since_last_checkpoint ? checkpointDroppedFlows : 0));
   lua_push_float_table_entry(vm, "flow_export_rate",
 			     mysqlExportRate >= 0 ? mysqlExportRate : 0);
 }
@@ -808,6 +822,22 @@ int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_
 
   if(m) m->lock(__FILE__, __LINE__);
 
+
+  if(ntop->getPrefs()->is_sql_log_enabled() && log_fd && sql) {
+#ifndef WIN32
+    char log_date[32];
+    time_t log_time = time(NULL);
+    struct tm result;
+
+    strftime(log_date, sizeof(log_date),
+	     "%d/%b/%Y %H:%M:%S", localtime_r(&log_time, &result));
+    fprintf(log_fd, "%s ", log_date);
+#endif
+
+    fprintf(log_fd, "%s\n", sql);
+    fflush(log_fd);
+  }
+  
   if((rc = mysql_query(&mysql, sql)) != 0) {
     /* retry */
     disconnectFromDB(&mysql);
@@ -859,10 +889,18 @@ int MySQLDB::exec_sql_query(lua_State *vm, char *sql, bool limitRows, bool wait_
       for(int i = 0; i < num_fields; i++)
 	lua_push_str_table_entry(vm, (const char*)fields[i], row[i] ? row[i] : (char*)"");
 
-      lua_pushnumber(vm, ++num);
+      lua_pushinteger(vm, ++num);
       lua_insert(vm, -2);
       lua_settable(vm, -3);
 
+      if(num > MYSQL_MAX_NUM_ROWS) {
+	static bool warning_shown = false;
+	if(!warning_shown) {
+	  ntop->getTrace()->traceEvent(TRACE_WARNING, "Too many results returned from MySQL, reduce query result set");
+	  warning_shown = true;
+	}
+      }
+
       if(limitRows && num >= MYSQL_MAX_NUM_ROWS) break;
     }
 
