@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,25 +27,23 @@ NetworkDiscovery::NetworkDiscovery(NetworkInterface *_iface) {
   char errbuf[PCAP_ERRBUF_SIZE];
   iface = _iface;
 
-  if((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) != -1) {
-    int rc;
-    char *ifname  = iface->altDiscoverableName();
+  char *ifname  = iface->altDiscoverableName();
+  if(ifname == NULL)
+    ifname = iface->get_name();
 
-    if(ifname == NULL)
-      ifname = iface->get_name();
-
-    errno = 0;
-    rc = Utils::bindSockToDevice(udp_sock, AF_INET, ifname);
-
-    if((rc < 0) && (errno != 0)) {
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to bind socket to %s [%d/%s]",
-				   ifname, errno, strerror(errno));
-    }
-  } else
-    throw("Unable to start network discovery");
-
-  if((pd = pcap_open_live(iface->get_name(), 128 /* snaplen */, 0 /* no promisc */, 5, errbuf)) == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create pcap socket [%d/%s]", errno, strerror(errno));
+#if ! defined(__arm__)
+  if((pd = pcap_open_live(ifname, 128 /* snaplen */, 0 /* no promisc */, 5, errbuf)) == NULL) {
+#else
+  /* pcap_next can really block a lot if we do not activate immediate mode! See https://github.com/mfontanini/libtins/issues/180 */
+  if(((pd = pcap_create(ifname, errbuf)) == NULL) ||
+		(pcap_set_timeout(pd, 5) != 0) ||
+		(pcap_set_snaplen(pd, 128) != 0) ||
+		(pcap_set_immediate_mode(pd, 1) != 0) || /* enable immediate mode */
+		(pcap_activate(pd) != 0)) {
+#endif
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create pcap socket on %s [%d/%s]", ifname, errno, strerror(errno));
+	udp_sock = -1;
+	throw("Unable to start network discovery");
   } else {
     const char* bpfFilter = "arp && arp[6:2] = 2";  // arp[x:y] - from byte 6 for 2 bytes (arp.opcode == 2 -> reply)
     struct bpf_program fcode;
@@ -54,6 +52,20 @@ NetworkDiscovery::NetworkDiscovery(NetworkInterface *_iface) {
     if(pcap_compile(pd, &fcode, bpfFilter, 1, 0xFFFFFF00) == 0)
       pcap_setfilter(pd, &fcode);
   }
+
+  if ((udp_sock = socket(AF_INET, SOCK_DGRAM, 0)) != -1) {
+	  int rc;
+
+	  errno = 0;
+	  rc = Utils::bindSockToDevice(udp_sock, AF_INET, ifname);
+
+	  if ((rc < 0) && (errno != 0)) {
+		  ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to bind socket to %s [%d/%s]",
+			  ifname, errno, strerror(errno));
+	  }
+  }
+  else
+	  throw("Unable to start network discovery");
 }
 
 /* ******************************* */
@@ -95,7 +107,7 @@ u_int16_t NetworkDiscovery::in_cksum(u_int8_t *buf, u_int16_t buf_len, u_int32_t
 */
 void NetworkDiscovery::arpScan(lua_State* vm) {
   bpf_u_int32 netp, maskp;
-  u_int32_t first_ip, last_ip, host_ip, sender_ip = Utils::readIPv4(iface->get_name());
+  u_int32_t first_ip, last_ip, host_ip, sender_ip;
   char macbuf[32], ipbuf[32], mdnsbuf[256];
   u_char mdnsreply[1500];
   const u_int max_num_ips = 1024;
@@ -107,16 +119,41 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
   ndpi_dns_packet_header *dns_h;
   u_int dns_query_len;
   struct sockaddr_in mdns_dest;
+  int fd = -1;
+  char *ifname  = iface->altDiscoverableName();
+
+#ifndef WIN32
+  fd = pcap_get_selectable_fd(pd);
+#endif
+
+  if(ifname == NULL)
+    ifname = iface->get_name();
 
   if(!pd) return;
+  sender_ip = Utils::readIPv4(ifname);
 
   lua_newtable(vm);
-  
+
   iface->getIPv4Address(&netp, &maskp);
-    
+
   /* Purge existing packets */
-  while(pcap_next(pd, &h) != NULL) ;
+
+  while(!ntop->getGlobals()->isShutdown()) {
+    fd_set rset;
+    struct timeval tv;
+    
+    FD_ZERO(&rset);
+    FD_SET(fd, &rset);
+    
+    tv.tv_sec = 0, tv.tv_usec = 0;
+    if(select(fd + 1, &rset, NULL, NULL, &tv) > 0)
+      pcap_next(pd, &h);
+    else
+      break;
+  }
   
+  if(ntop->getGlobals()->isShutdown()) return;
+
   if((mdns_sock = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create MDNS socket");
   else {
@@ -124,7 +161,7 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
     u_int last_dot = 0;
     char *queries;
 
-    if(mdns_sock > max_sock) max_sock = mdns_sock;    
+    if(mdns_sock > max_sock) max_sock = mdns_sock;
     dns_h = (struct ndpi_dns_packet_header*)mdnsbuf;
     dns_h->tr_id = 0;
     dns_h->flags = 0 /* query */;
@@ -153,6 +190,8 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
     dns_query_len += sizeof(struct ndpi_dns_packet_header);
   }
 
+  if(ntop->getGlobals()->isShutdown()) return;
+
   netp = ntohl(netp), maskp = ntohl(maskp);
   first_ip = netp & maskp, last_ip = netp + (~maskp);
   first_ip++, last_ip--;
@@ -160,33 +199,33 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
   if((last_ip - first_ip) > max_num_ips)
     last_ip = first_ip + max_num_ips;
 
-  Utils::readMac(iface->get_name(), arp.arp_sha);
+  Utils::readMac(ifname, arp.arph.arp_sha);
 
   memset(arp.dst_mac, 0xFF, sizeof(arp.dst_mac));
-  memcpy(arp.src_mac, arp.arp_sha, sizeof(arp.src_mac));
+  memcpy(arp.src_mac, arp.arph.arp_sha, sizeof(arp.src_mac));
   arp.proto  = htons(0x0806 /* ARP */);
-  arp.ar_hrd = htons(1);
-  arp.ar_pro = htons(0x0800);
-  arp.ar_hln = 6;
-  arp.ar_pln = 4;
-  arp.ar_op = htons(1 /* ARP Request */);
-  arp.arp_spa = sender_ip;
-  memset(arp.arp_tha, 0, sizeof(arp.arp_tha));
+  arp.arph.ar_hrd = htons(1);
+  arp.arph.ar_pro = htons(0x0800);
+  arp.arph.ar_hln = 6;
+  arp.arph.ar_pln = 4;
+  arp.arph.ar_op = htons(1 /* ARP Request */);
+  arp.arph.arp_spa = sender_ip;
+  memset(arp.arph.arp_tha, 0, sizeof(arp.arph.arp_tha));
 
   /* Let's add myself */
   lua_push_str_table_entry(vm,
-			   Utils::formatMac(arp.arp_sha, macbuf, sizeof(macbuf)),
-			   Utils::intoaV4(ntohl(arp.arp_spa), ipbuf, sizeof(ipbuf)));
+			   Utils::formatMac(arp.arph.arp_sha, macbuf, sizeof(macbuf)),
+			   Utils::intoaV4(ntohl(arp.arph.arp_spa), ipbuf, sizeof(ipbuf)));
 
   mdns_dest.sin_family = AF_INET, mdns_dest.sin_port = htons(5353);
-    
+
   for(int num_runs=0; num_runs<2; num_runs++) {
     for(host_ip = first_ip; host_ip <last_ip; host_ip++) {
       int sel_rc = 0;
-      
-      arp.arp_tpa = ntohl(host_ip);
 
-      if(arp.arp_tpa == arp.arp_spa)
+      arp.arph.arp_tpa = ntohl(host_ip);
+
+      if(arp.arph.arp_tpa == arp.arph.arp_spa)
 	continue; /* I know myself already */
 
       // Inject packet
@@ -194,23 +233,33 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
 	break;
 
       FD_ZERO(&rset);
+
+      if(fd != -1) {
+	FD_SET(fd, &rset);
+	if(fd > max_sock) max_sock = fd;
+      }
       if(mdns_sock != -1) FD_SET(mdns_sock, &rset);
 
       tv.tv_sec = 0, tv.tv_usec = 0; /* Don't wait at all */
 
-      if(mdns_sock != -1)
+      if(max_sock != 0)
 	sel_rc = select(max_sock + 1, &rset, NULL, NULL, &tv);
-	
-      reply = (struct arp_packet*)pcap_next(pd, &h);      
+
+      if((fd == -1) || FD_ISSET(fd, &rset))
+	reply = (struct arp_packet*)pcap_next(pd, &h);
+      else
+	reply = NULL;
+
       if(reply) {
 	lua_push_str_table_entry(vm,
-				 Utils::formatMac(reply->arp_sha, macbuf, sizeof(macbuf)),
-				 Utils::intoaV4(ntohl(reply->arp_spa), ipbuf, sizeof(ipbuf)));
-	
-	ntop->getTrace()->traceEvent(TRACE_INFO, "Received ARP reply from %s",  Utils::intoaV4(ntohl(reply->arp_spa), ipbuf, sizeof(ipbuf)));
-	
+				 Utils::formatMac(reply->arph.arp_sha, macbuf, sizeof(macbuf)),
+				 Utils::intoaV4(ntohl(reply->arph.arp_spa), ipbuf, sizeof(ipbuf)));
+
+	ntop->getTrace()->traceEvent(TRACE_INFO, "Received ARP reply from %s",
+				     Utils::intoaV4(ntohl(reply->arph.arp_spa), ipbuf, sizeof(ipbuf)));
+
 	if(mdns_sock != -1) {
-	  mdns_dest.sin_addr.s_addr = reply->arp_spa, dns_h->tr_id++;
+	  mdns_dest.sin_addr.s_addr = reply->arph.arp_spa, dns_h->tr_id++;
 	  if(sendto(mdns_sock, mdnsbuf, dns_query_len, 0, (struct sockaddr *)&mdns_dest, sizeof(struct sockaddr_in)) < 0)
 	    ntop->getTrace()->traceEvent(TRACE_ERROR, "MDNS Send error [%d/%s]", errno, strerror(errno));
 	}
@@ -223,13 +272,13 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
 
 	if(len > 0) {
 	  char outbuf[1024];
-	    
+
 	  dissectMDNS(mdnsreply, len, outbuf, sizeof(outbuf));
-	    
+
 	  if(outbuf[0] != '\0')
 	    lua_push_str_table_entry(vm,
 				     Utils::intoaV4(ntohl(from.sin_addr.s_addr), ipbuf, sizeof(ipbuf)),
-				     outbuf);	  
+				     outbuf);
 	}
       }
 
@@ -239,24 +288,37 @@ void NetworkDiscovery::arpScan(lua_State* vm) {
 
   /* Query myself mith MDNS */
   mdns_dest.sin_addr.s_addr = sender_ip, dns_h->tr_id++;
-  if(sendto(mdns_sock, mdnsbuf, dns_query_len, 0, (struct sockaddr *)&mdns_dest, sizeof(struct sockaddr_in)) < 0)
+  errno = 0;
+  if((sendto(mdns_sock, mdnsbuf, dns_query_len, 0, (struct sockaddr *)&mdns_dest, sizeof(struct sockaddr_in)) < 0) && (errno != 0))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Send error [%d/%s]", errno, strerror(errno));
 
   /* Final rush */
   while(true) {
+    if(fd != -1) {
+      fd_set rset;
+      struct timeval tv;
+      
+      FD_ZERO(&rset);
+      FD_SET(fd, &rset);
+      
+      tv.tv_sec = 0, tv.tv_usec = 0;
+      if(select(fd + 1, &rset, NULL, NULL, &tv) <= 0)
+	break;
+    }
+
     if((reply = (struct arp_packet*)pcap_next(pd, &h)) != NULL) {
       lua_push_str_table_entry(vm,
-			       Utils::formatMac(reply->arp_sha, macbuf, sizeof(macbuf)),
-			       Utils::intoaV4(ntohl(reply->arp_spa), ipbuf, sizeof(ipbuf)));
-      
+			       Utils::formatMac(reply->arph.arp_sha, macbuf, sizeof(macbuf)),
+			       Utils::intoaV4(ntohl(reply->arph.arp_spa), ipbuf, sizeof(ipbuf)));
+
       ntop->getTrace()->traceEvent(TRACE_INFO, "Received ARP reply from %s",
-				   Utils::intoaV4(ntohl(reply->arp_spa), ipbuf, sizeof(ipbuf)));
-      mdns_dest.sin_addr.s_addr = reply->arp_spa, dns_h->tr_id++;
+				   Utils::intoaV4(ntohl(reply->arph.arp_spa), ipbuf, sizeof(ipbuf)));
+      mdns_dest.sin_addr.s_addr = reply->arph.arp_spa, dns_h->tr_id++;
       if(sendto(mdns_sock, mdnsbuf, dns_query_len, 0, (struct sockaddr *)&mdns_dest, sizeof(struct sockaddr_in)) < 0)
 	ntop->getTrace()->traceEvent(TRACE_ERROR, "Send error [%d/%s]", errno, strerror(errno));
     } else
       break;
-  }  
+  }
 
   if(mdns_sock != -1) {
     while(true) {
@@ -407,11 +469,11 @@ void NetworkDiscovery::dissectMDNS(u_char *buf, u_int buf_len,
 
     if(data_len < buf_len) {
       u_int l;
-      
+
       offset += 3;
 
       memset(rspbuf, 0, sizeof(rspbuf));
-      
+
       for(idx = 0; idx<data_len; idx++, offset++) {
 	if(queries[offset] < 32) {
 	  rspbuf[idx] = '.';
@@ -421,7 +483,7 @@ void NetworkDiscovery::dissectMDNS(u_char *buf, u_int buf_len,
 
 	    offset++;
 	    // ntop->getTrace()->traceEvent(TRACE_ERROR, "new_offset=%u", new_offset);
-	    
+
 	    while((idx < sizeof(rspbuf)) && (buf[new_offset] != 0)){
 	      if(buf[new_offset] < 32)
 		rspbuf[idx] = '.';
@@ -463,6 +525,10 @@ void NetworkDiscovery::discover(lua_State* vm, u_int timeout) {
   struct timeval tv = { (time_t)timeout /* sec */, 0 };
   fd_set fdset;
 
+  char *ifname  = iface->altDiscoverableName();
+  if(ifname == NULL)
+    ifname = iface->get_name();
+
   lua_newtable(vm);
 
   if(!pd) return;
@@ -509,9 +575,9 @@ void NetworkDiscovery::discover(lua_State* vm, u_int timeout) {
       NULL
     };
     int i;
-    u_int32_t sender_ip = Utils::readIPv4(iface->get_name());
+    u_int32_t sender_ip = Utils::readIPv4(ifname);
 
-    Utils::readMac(iface->get_name(), sender_mac);
+    Utils::readMac(ifname, sender_mac);
 
     for(i=0; query_list[i] != NULL; i++) {
       u_int16_t len = buildMDNSDiscoveryDatagram(query_list[i], sender_ip, sender_mac, msg, sizeof(msg));
@@ -529,11 +595,11 @@ void NetworkDiscovery::discover(lua_State* vm, u_int timeout) {
   FD_SET(udp_sock, &fdset);
 
   while(select(udp_sock + 1, &fdset, NULL, NULL, &tv) > 0) {
-    struct sockaddr_in from;
-    socklen_t s;
+    struct sockaddr_in from = { 0 };
+    socklen_t s = sizeof(from);
     char ipbuf[32];
-    int len = recvfrom(udp_sock, (char*)msg, sizeof(msg), 0, (struct sockaddr*)&from, &s);
-
+    int len = recvfrom(udp_sock, (char*)msg, sizeof(msg), 0, (sockaddr*)&from, &s);
+    
     ntop->getTrace()->traceEvent(TRACE_INFO, "Received SSDP packet from %s:%u",
 				 Utils::intoaV4(ntohl(from.sin_addr.s_addr), ipbuf, sizeof(ipbuf)),
 				 ntohs(from.sin_port));
@@ -557,6 +623,7 @@ void NetworkDiscovery::discover(lua_State* vm, u_int timeout) {
 	  }
 	}
       }
-    }
+    } else
+      break;
   }
 }
