@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,9 @@ ParserInterface::ParserInterface(const char *endpoint, const char *custom_interf
   zmq_remote_stats = zmq_remote_stats_shadow = NULL;
   zmq_remote_initial_exported_flows = 0;
   map = NULL, once = false;
+#ifdef NTOPNG_PRO
+  custom_app_maps = NULL;
+#endif
 
   addMapping("IN_BYTES", 1);
   addMapping("IN_PKTS", 2);
@@ -465,6 +468,9 @@ ParserInterface::~ParserInterface() {
 
   if(zmq_remote_stats)        free(zmq_remote_stats);
   if(zmq_remote_stats_shadow) free(zmq_remote_stats_shadow);
+#ifdef NTOPNG_PRO
+  if(custom_app_maps)         delete(custom_app_maps);
+#endif
 }
 
 /* **************************************************** */
@@ -502,7 +508,7 @@ u_int8_t ParserInterface::parseEvent(char *payload, int payload_size,
 
   // payload[payload_size] = '\0';
 
-  //ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", payload);
+  // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", payload);
   o = json_tokener_parse_verbose(payload, &jerr);
 
   if(o && (zrs = (ZMQ_RemoteStats*)calloc(1, sizeof(ZMQ_RemoteStats)))) {
@@ -543,8 +549,8 @@ u_int8_t ParserInterface::parseEvent(char *payload, int payload_size,
     }
 
     if(json_object_object_get_ex(o, "drops", &w)) {
-      if(json_object_object_get_ex(w, "export_queue_too_long", &z))
-	zrs->export_queue_too_long = (u_int32_t)json_object_get_int64(z);
+      if(json_object_object_get_ex(w, "export_queue_full", &z))
+	zrs->export_queue_full = (u_int32_t)json_object_get_int64(z);
 
       if(json_object_object_get_ex(w, "too_many_flows", &z))
 	zrs->too_many_flows = (u_int32_t)json_object_get_int64(z);
@@ -626,9 +632,13 @@ void ParserInterface::parseSingleFlow(json_object *o,
   IpAddress ip_aux; /* used to check empty IPs */
   struct json_object_iterator it = json_object_iter_begin(o);
   struct json_object_iterator itEnd = json_object_iter_end(o);
-
+  u_int8_t flow_ip_version = 0;
+  bool invalid_flow = false;
+  
   /* Reset data */
   memset(&flow, 0, sizeof(flow));
+  flow.core.l7_proto.master_protocol = flow.core.l7_proto.app_protocol = NDPI_PROTOCOL_UNKNOWN;
+  flow.core.l7_proto.category = NDPI_PROTOCOL_CATEGORY_UNSPECIFIED;
   flow.additional_fields = json_object_new_object();
   flow.core.pkt_sampling_rate = 1; /* 1:1 (no sampling) */
   flow.core.source_id = source_id, flow.core.vlan_id = 0;
@@ -696,6 +706,10 @@ void ParserInterface::parseSingleFlow(json_object *o,
 					 "Check exported fields");
 	}
 	break;
+      case IP_PROTOCOL_VERSION:
+	flow_ip_version = atoi(value);
+      break;
+      
       case IPV4_DST_ADDR:
       case IPV6_DST_ADDR:
 	if(flow.core.dst_ip.isEmpty()) {
@@ -728,7 +742,22 @@ void ParserInterface::parseSingleFlow(json_object *o,
 	  flow.core.vlan_id = atoi(value);
 	break;
       case L7_PROTO:
-	flow.core.l7_proto = atoi(value);
+	if(!strchr(value, '.')) {
+	  /* Old behaviour, only the app protocol */
+	  flow.core.l7_proto.app_protocol = atoi(value);
+	} else {
+	  char *proto_dot;
+
+	  flow.core.l7_proto.master_protocol = (u_int16_t)strtoll(value, &proto_dot, 10);
+	  flow.core.l7_proto.app_protocol    = (u_int16_t)strtoll(proto_dot + 1, NULL, 10);
+	}
+
+#if 0
+	ntop->getTrace()->traceEvent(TRACE_NORMAL, "[value: %s][master: %u][app: %u]",
+				     value,
+				     flow.core.l7_proto.master_protocol,
+				     flow.core.l7_proto.app_protocol);
+#endif
 	break;
       case PROTOCOL:
 	flow.core.l4_proto = atoi(value);
@@ -833,6 +862,7 @@ void ParserInterface::parseSingleFlow(json_object *o,
 	iface->enable_sprobe(); /* We're collecting system flows */
 	flow.src_process.pid = atoi(value);
 	break;
+#if 0
       case SRC_PROC_NAME:
 	iface->enable_sprobe(); /* We're collecting system flows */
 	snprintf(flow.src_process.name, sizeof(flow.src_process.name), "%s", value);
@@ -861,10 +891,12 @@ void ParserInterface::parseSingleFlow(json_object *o,
       case SRC_PROC_PCTG_IOWAIT:
 	flow.src_process.percentage_iowait_time = ((float)atol(value))/((float)100);
 	break;
+#endif
       case DST_PROC_PID:
 	iface->enable_sprobe(); /* We're collecting system flows */
 	flow.dst_process.pid = atoi(value);
 	break;
+#if 0
       case DST_PROC_NAME:
 	iface->enable_sprobe(); /* We're collecting system flows */
 	snprintf(flow.dst_process.name, sizeof(flow.dst_process.name), "%s", value);
@@ -893,6 +925,7 @@ void ParserInterface::parseSingleFlow(json_object *o,
       case DST_PROC_PCTG_IOWAIT:
 	flow.dst_process.percentage_iowait_time = ((float)atol(value))/((float)100);
 	break;
+#endif
       case DNS_QUERY:
 	flow.dns_query = strdup(value);
 	break;
@@ -919,6 +952,10 @@ void ParserInterface::parseSingleFlow(json_object *o,
 	flow.core.vrfId = atoi(value);
 	break;
       default:
+#ifdef NTOPNG_PRO
+	if(custom_app_maps || (custom_app_maps = new(std::nothrow) CustomAppMaps()))
+	  custom_app_maps->checkCustomApp(key, value, &flow);
+#endif
 	ntop->getTrace()->traceEvent(TRACE_DEBUG, "Not handled ZMQ field %u/%s", key_id, key);
 	add_to_additional_fields = true;
 	break;
@@ -935,8 +972,31 @@ void ParserInterface::parseSingleFlow(json_object *o,
     json_object_iter_next(&it);
   } // while json_object_iter_equal
 
-  /* Process Flow */
-  iface->processFlow(&flow);
+  /* Handle zero IPv4/IPv6 discrepacies */
+  if(flow_ip_version == 0) {
+    if(flow.core.src_ip.getVersion() != flow.core.dst_ip.getVersion()) {
+      if(flow.core.dst_ip.isIPv4() && flow.core.src_ip.isIPv6() && flow.core.src_ip.isEmpty())
+	flow.core.src_ip.setVersion(4);
+      else if(flow.core.src_ip.isIPv4() && flow.core.dst_ip.isIPv6() && flow.core.dst_ip.isEmpty())
+	flow.core.dst_ip.setVersion(4);
+      else if(flow.core.dst_ip.isIPv6() && flow.core.src_ip.isIPv4() && flow.core.src_ip.isEmpty())
+	flow.core.src_ip.setVersion(6);
+      else if(flow.core.src_ip.isIPv6() && flow.core.dst_ip.isIPv4() && flow.core.dst_ip.isEmpty())
+	flow.core.dst_ip.setVersion(6);
+      else {
+	invalid_flow = true;
+	ntop->getTrace()->traceEvent(TRACE_WARNING,
+				     "IP version mismatch: client:%d server:%d - flow will be ignored",
+				     flow.core.src_ip.getVersion(), flow.core.dst_ip.getVersion());
+      }
+    }
+  } else
+    flow.core.src_ip.setVersion(flow_ip_version), flow.core.dst_ip.setVersion(flow_ip_version);
+  
+  if(!invalid_flow) {
+    /* Process Flow */
+    iface->processFlow(&flow);
+  }
 
   /* Dispose memory */
   if(flow.dns_query) free(flow.dns_query);
@@ -1099,6 +1159,14 @@ void ParserInterface::setRemoteStats(ZMQ_RemoteStats *zrs) {
 
 /* **************************************************** */
 
+#ifdef NTOPNG_PRO
+bool ParserInterface::getCustomAppDetails(u_int32_t remapped_app_id, u_int32_t *const pen, u_int32_t *const app_field, u_int32_t *const app_id) {
+  return custom_app_maps && custom_app_maps->getCustomAppDetails(remapped_app_id, pen, app_field, app_id);
+}
+#endif
+
+/* **************************************************** */
+
 void ParserInterface::lua(lua_State* vm) {
   ZMQ_RemoteStats *zrs = zmq_remote_stats;
 
@@ -1114,11 +1182,14 @@ void ParserInterface::lua(lua_State* vm) {
     if(zrs->remote_probe_public_address[0] != '\0')
       lua_push_str_table_entry(vm, "probe.public_ip", zrs->remote_probe_public_address);
 
-    lua_push_int_table_entry(vm, "zmq.num_flow_exports", zrs->num_flow_exports - zmq_remote_initial_exported_flows);
-    lua_push_int_table_entry(vm, "zmq.num_exporters", zrs->num_exporters);
+    lua_push_uint64_table_entry(vm, "zmq.num_flow_exports", zrs->num_flow_exports - zmq_remote_initial_exported_flows);
+    lua_push_uint64_table_entry(vm, "zmq.num_exporters", zrs->num_exporters);
 
-    lua_push_int_table_entry(vm, "timeout.lifetime", zrs->remote_lifetime_timeout);
-    lua_push_int_table_entry(vm, "timeout.idle", zrs->remote_idle_timeout);
+    if(zrs->export_queue_full > 0)
+      lua_push_uint64_table_entry(vm, "zmq.drops.export_queue_full", zrs->export_queue_full);
+
+    lua_push_uint64_table_entry(vm, "timeout.lifetime", zrs->remote_lifetime_timeout);
+    lua_push_uint64_table_entry(vm, "timeout.idle", zrs->remote_idle_timeout);
   }
 }
 
