@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-17 - ntop.org
+ * (C) 2013-18 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -35,12 +35,18 @@ Redis::Redis(const char *_redis_host, const char *_redis_password, u_int16_t _re
   redis_host = _redis_host ? strdup(_redis_host) : NULL;
   redis_password = _redis_password ? strdup(_redis_password) : NULL;
   redis_port = _redis_port, redis_db_id = _redis_db_id;
+#ifdef __linux__
+  is_socket_connection = false;
+#endif
 
   num_requests = num_reconnections = 0;
   redis = NULL, operational = false;
+  initializationCompleted = false;
   reconnectRedis();
   stringCache = NULL, numCached = 0;
   l = new Mutex();
+
+  getRedisVersion();
 }
 
 /* **************************************** */
@@ -50,8 +56,10 @@ Redis::~Redis() {
   redisFree(redis);
   // flushCache();
   delete l;
+  
   if(redis_host)     free(redis_host);
   if(redis_password) free(redis_password);
+  if(redis_version)  free(redis_version);
 }
 
 /* **************************************** */
@@ -67,8 +75,14 @@ void Redis::reconnectRedis() {
     ntop->getTrace()->traceEvent(TRACE_NORMAL, "Redis has disconnected: reconnecting...");
     redisFree(redis);
   }
+#ifdef __linux__
+  struct stat buf;
 
-  redis = redisConnectWithTimeout(redis_host, redis_port, timeout);
+  if(!stat(redis_host, &buf) && S_ISSOCK(buf.st_mode))
+    redis = redisConnectUnixWithTimeout(redis_host, timeout), is_socket_connection = true;
+  else
+#endif
+    redis = redisConnectWithTimeout(redis_host, redis_port, timeout);
 
   if(redis_password) {
     num_requests++;
@@ -108,9 +122,16 @@ void Redis::reconnectRedis() {
       goto redis_error_handler;
     } else {
       freeReplyObject(reply);
-      ntop->getTrace()->traceEvent(TRACE_NORMAL,
-				   "Successfully connected to redis %s:%u@%u",
-				   redis_host, redis_port, redis_db_id);
+#ifdef __linux__
+      if(!is_socket_connection)
+	ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				     "Successfully connected to redis %s:%u@%u",
+				     redis_host, redis_port, redis_db_id);
+      else
+#endif
+	ntop->getTrace()->traceEvent(TRACE_NORMAL,
+				     "Successfully connected to redis %s@%u",
+				     redis_host, redis_db_id);
       operational = true;
     }
   }
@@ -144,8 +165,9 @@ int Redis::expire(char *key, u_int expire_secs) {
 
 /* **************************************** */
 
-bool Redis::isCacheable(char *key) {
+bool Redis::isCacheable(const char * const key) {
   if((strstr(key, "ntopng.cache."))
+     || (strstr(key, "ntopng.prefs."))
      || (strstr(key, "ntopng.user.") && (!strstr(key, ".password"))))
     return(true);
 
@@ -173,9 +195,26 @@ bool Redis::expireCache(char *key, u_int expire_secs) {
 
 /* **************************************** */
 
+void Redis::checkDumpable(const char * const key) {
+  if(!initializationCompleted) return;
+
+  /* We use this function to check and possibly request a preference dump to a file.
+     This ensures settings persistance also upon redis flushes */
+  if(!strncmp(key, "ntopng.prefs.", 13)
+     || !strncmp(key, "ntopng.user.", 12)) {
+    ntop->getPrefs()->reloadPrefsFromRedis();
+    /* Tell housekeeping.lua to dump prefs to disk */
+    ntop->getRedis()->set((char*)PREFS_CHANGED, (char*)"true");
+    // ntop->getTrace()->traceEvent(TRACE_NORMAL, "Going to refresh after change of: %s", key);
+  }
+}
+
+/* **************************************** */
+
 /* NOTE: We assume that the addToCache() caller locks this instance */
-void Redis::addToCache(char *key, char *value, u_int expire_secs) {
+void Redis::addToCache(const char * const key, const char * const value, u_int expire_secs) {
   StringCache_t *cached = NULL;
+  if(!initializationCompleted) return;
 
 #ifdef CACHE_DEBUG
   printf("**** Caching %s=%s\n", key, value ? value : "<NULL>");
@@ -219,7 +258,14 @@ int Redis::get(char *key, char *rsp, u_int rsp_len, bool cache_it) {
 
   HASH_FIND_STR(stringCache, key, cached);
   if(cached) {
-    snprintf(rsp, rsp_len, "%s", cached->value);
+    if((cached->expire > 0) && (time(NULL) >= cached->expire)) {
+#ifdef CACHE_DEBUG
+      printf("**** Cache expired %s\n", key);
+#endif
+      HASH_DEL(stringCache, cached);
+      rsp[0] = '\0';
+    } else
+      snprintf(rsp, rsp_len, "%s", cached->value);
 
 #ifdef CACHE_DEBUG
     printf("**** Read from cache %s=%s\n", key, rsp);
@@ -244,15 +290,36 @@ int Redis::get(char *key, char *rsp, u_int rsp_len, bool cache_it) {
   } else
     rsp[0] = 0, rc = -1;
 
-  if(cache_it || cacheable)
-    addToCache(key, rsp, 0);
+  if(cache_it || cacheable) {
+    u_int expire_sec = 0;
+
+    if(reply) freeReplyObject(reply);
+    num_requests++;
+    reply = (redisReply*)redisCommand(redis, "TTL %s", key);
+    if(!reply) reconnectRedis();
+    if(reply && (reply->type != REDIS_REPLY_INTEGER))
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+    if(reply && (((int32_t)reply->integer)) >= 0)
+      expire_sec = reply->integer;
+
+#ifdef CACHE_DEBUG
+    printf("**** ADD TO CACHE %s=%s [expire_sec=%u]\n", key, rsp, expire_sec);
+#endif
+
+    addToCache(key, rsp, expire_sec);
+  }
 
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
   if(cacheable && (rc == -1)) {
+    /* Don't fill redis with default empty strings.
+       Those empty strings have already been set
+       into the memory cache by addToCache, leave
+       them out from redis */
     /* Add default */
-    set(key, (char*)"", 0);
+    // set(key, (char*)"", 0);
   }
 
   return(rc);
@@ -288,12 +355,14 @@ int Redis::del(char *key){
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
+  if(reply) checkDumpable(key);
+
   return(rc);
 }
 
 /* **************************************** */
 
-int Redis::hashGet(char *key, char *field, char *rsp, u_int rsp_len) {
+int Redis::hashGet(const char * const key, const char * const field, char * const rsp, u_int rsp_len) {
   int rc;
   redisReply *reply;
 
@@ -316,7 +385,7 @@ int Redis::hashGet(char *key, char *field, char *rsp, u_int rsp_len) {
 
 /* **************************************** */
 
-int Redis::hashSet(char *key, char *field, char *value) {
+int Redis::hashSet(const char * const key, const char * const field, const char * const value) {
   int rc = 0;
   redisReply *reply;
 
@@ -329,12 +398,14 @@ int Redis::hashSet(char *key, char *field, char *value) {
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
+  if(reply) checkDumpable(key);
+
   return(rc);
 }
 
 /* **************************************** */
 
-int Redis::hashDel(char *key, char *field) {
+int Redis::hashDel(const char * const key, const char * const field) {
   int rc;
   redisReply *reply;
 
@@ -351,15 +422,24 @@ int Redis::hashDel(char *key, char *field) {
     rc = -1;
   l->unlock(__FILE__, __LINE__);
 
+  if(reply) checkDumpable(key);
+
   return(rc);
 }
 
 /* **************************************** */
 
-int Redis::set(char *key, char *value, u_int expire_secs) {
+int Redis::set(const char * const key, const char * const value, u_int expire_secs) {
   int rc;
   redisReply *reply;
 
+  if((value == NULL) || (value[0] == '\0')) {    
+    if(strncmp(key, NTOPNG_PREFS_PREFIX, sizeof(NTOPNG_PREFS_PREFIX)) == 0) {
+      /* This is an empty preference value that we can discard*/
+      ntop->getTrace()->traceEvent(TRACE_WARNING, "Discarding empty prefence value %s", key);
+    }
+  }
+  
   l->lock(__FILE__, __LINE__);
 
   if(isCacheable(key))
@@ -382,128 +462,8 @@ int Redis::set(char *key, char *value, u_int expire_secs) {
   }
   l->unlock(__FILE__, __LINE__);
 
-  return(rc);
-}
-
-/* **************************************** */
-
-char* Redis::popSet(char *pop_name, char *rsp, u_int rsp_len) {
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "SPOP %s", pop_name);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply && reply->str) {
-    snprintf(rsp, rsp_len, "%s", reply->str);
-  } else
-    rsp[0] = 0;
-
-  if(reply) freeReplyObject(reply);
-  l->unlock(__FILE__, __LINE__);
-
-  return(rsp);
-}
-
-/* **************************************** */
-
-/*
-  Increment a key and return its update value
-*/
-u_int32_t Redis::incrKey(char *key) {
-  u_int rc;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "INCR %s", key);
-  if(reply) {
-    if(reply->type == REDIS_REPLY_ERROR)
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???"), rc = 0;
-    else
-      rc = (u_int)reply->integer;
-
-    freeReplyObject(reply);
-  } else {
-    if(!reply) reconnectRedis();
-    rc = -1;
-  }
-
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-
-/* **************************************** */
-
-/*
-  Increment key.member of +value and keeps at most trim_len elements
-*/
-int Redis::zIncr(char *key, char *member) {
-  int rc;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "ZINCRBY %s 1 %s", key, member);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-
-/* **************************************** */
-
-/*
-  Increment key.member of +value and keeps at most trim_len elements
-*/
-int Redis::zTrim(char *key, u_int trim_len) {
-  int rc;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "ZREMRANGEBYRANK %s 0 %d", key, -1*trim_len);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-
-/* **************************************** */
-
-int Redis::zRevRange(const char *pattern, char ***keys_p) {
-  int rc = 0;
-  u_int i;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "ZREVRANGE %s 0 -1 WITHSCORES", pattern);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply && (reply->type == REDIS_REPLY_ARRAY)) {
-    (*keys_p) = (char**) malloc(reply->elements * sizeof(char*));
-    rc = (int)reply->elements;
-
-    for(i = 0; i < reply->elements; i++) {
-      (*keys_p)[i] = strdup(reply->element[i]->str);
-    }
-  }
-
-  if(reply) freeReplyObject(reply);
-  l->unlock(__FILE__, __LINE__);
+  if(reply && expire_secs == 0)
+    checkDumpable(key);
 
   return(rc);
 }
@@ -617,75 +577,12 @@ int Redis::hashGetAll(const char *key, char ***keys_p, char ***values_p) {
 
 /* **************************************** */
 
-int Redis::oneOperator(const char *operation, char *key) {
-  int rc;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "%s %s", operation, key);
-  if(!reply) reconnectRedis();
-
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-
-/* **************************************** */
-
-int Redis::twoOperators(const char *operation, char *op1, char *op2) {
-  int rc;
-  redisReply *reply;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "%s %s %s", operation, op1, op2);
-  if(!reply) reconnectRedis();
-
-  if(reply
-     && (reply->type == REDIS_REPLY_ERROR)
-     && strcmp(reply->str, "ERR no such key"))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-
-/* **************************************** */
-
-int Redis::pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existence, bool localHost) {
-  if(ntop->getPrefs()->is_httpbl_enabled()) {
-    if(hostname == NULL) return(-1);
-    return(pushHost(TRAFFIC_FILTERING_CACHE, TRAFFIC_FILTERING_TO_RESOLVE,
-		    hostname, dont_check_for_existence, localHost));
-  } else
-    return(0);
-}
-
-/* **************************************** */
-
 int Redis::pushHostToResolve(char *hostname, bool dont_check_for_existence, bool localHost) {
   if(!ntop->getPrefs()->is_dns_resolution_enabled()) return(0);
   if(hostname == NULL) return(-1);
 
-  if(!ntop->getPrefs()->is_dns_resolution_enabled_for_all_hosts()) {
-    /*
-      In case only local addresses need to be resolved, skip
-      remote hosts
-    */
-    IpAddress ip;
-    int16_t network_id;
-
-    ip.set(hostname);
-    if(!ip.isLocalHost(&network_id))
-      return(-1);
-  }
+  if(!Utils::shouldResolveHost(hostname))
+    return(-1);
 
   return(pushHost(DNS_CACHE, DNS_TO_RESOLVE, hostname, dont_check_for_existence, localHost));
 }
@@ -747,12 +644,6 @@ int Redis::pushHost(const char* ns_cache, const char* ns_list, char *hostname,
 
 /* **************************************** */
 
-int Redis::popHostToTrafficFiltering(char *hostname, u_int hostname_len) {
-  return(popHost(TRAFFIC_FILTERING_TO_RESOLVE, hostname, hostname_len));
-}
-
-/* **************************************** */
-
 int Redis::popHostToResolve(char *hostname, u_int hostname_len) {
   return(popHost(DNS_TO_RESOLVE, hostname, hostname_len));
 }
@@ -765,62 +656,6 @@ int Redis::popHost(const char* ns_list, char *hostname, u_int hostname_len) {
 
 /* **************************************** */
 
-char* Redis::getTrafficFilteringCategory(char *numeric_ip, char *buf,
-					 u_int buf_len, bool categorize_if_unknown) {
-  char key[CONST_MAX_LEN_REDIS_KEY];
-  redisReply *reply;
-
-  if(!ntop->getPrefs()->is_httpbl_enabled())
-    return(NULL);
-
-  buf[0] = '\0';
-  l->lock(__FILE__, __LINE__);
-
-  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
-
-  /*
-    Add only if the ip has not been checked against the blacklist
-  */
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "GET %s", key);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply && reply->str) {
-    snprintf(buf, buf_len, "%s", reply->str);
-    freeReplyObject(reply);
-  } else {
-    buf[0] = '\0';
-
-    if(categorize_if_unknown) {
-      num_requests++;
-
-      if(ntop->getPrefs()->is_httpbl_enabled()) {
-	reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", TRAFFIC_FILTERING_TO_RESOLVE, numeric_ip);
-	if(!reply) reconnectRedis();
-	if(reply && (reply->type == REDIS_REPLY_ERROR))
-	  ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-	if(reply) freeReplyObject(reply);
-      }
-    }
-  }
-
-  l->unlock(__FILE__, __LINE__);
-
-  return(buf);
-}
-
-/* **************************************** */
-
-#ifdef NOTUSED
-int Redis::popDomainToCategorize(char *domainname, u_int domainname_len) {
-  return(lpop(DOMAIN_TO_CATEGORIZE, domainname, domainname_len));
-}
-#endif
-
-/* **************************************** */
-
 void Redis::setDefaults() {
   char *admin_md5 = (char*)"21232f297a57a5a743894a0e4a801fc3";
   char *value;
@@ -836,7 +671,13 @@ void Redis::setDefaults() {
   if(get((char*)"ntopng.user.admin.password", value,
 	 CONST_MAX_LEN_REDIS_VALUE) < 0) {
     set((char*)"ntopng.user.admin.password", admin_md5);
-    set((char*)"ntopng.user.admin.full_name", (char*)"ntopng Administrator");
+    set((char*)"ntopng.user.admin.full_name",
+#ifdef HAVE_NEDGE
+		(char*)((ntop->getPro()->is_oem()) ? "Administrator" : "ntopng Administrator")
+#else
+		(char*)"ntopng Administrator"
+#endif
+    );
     set((char*)"ntopng.user.admin.group", (char*)CONST_USER_GROUP_ADMIN);
     set((char*)"ntopng.user.admin.allowed_nets", (char*)"0.0.0.0/0,::/0");
   } else if(strncmp(value, admin_md5, strlen(admin_md5))) {
@@ -873,36 +714,6 @@ int Redis::flushDb() {
 
 /* **************************************** */
 
-int Redis::getAddressTrafficFiltering(char *numeric_ip,
-				      NetworkInterface *iface,
-				      char *rsp, u_int rsp_len,
-				      bool queue_if_not_found) {
-  char key[CONST_MAX_LEN_REDIS_KEY];
-  int rc;
-
-  rsp[0] = '\0';
-  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
-
-  rc = get(key, rsp, rsp_len);
-
-  if(rc != 0) {
-    if(queue_if_not_found) {
-      char buf[64];
-
-      snprintf(buf, sizeof(buf), "%s@%s", numeric_ip, iface->get_name());
-      pushHostToTrafficFiltering(buf, true, false);
-    }
-  } else {
-    /* We need to extend expire */
-
-    expire(numeric_ip, TRAFFIC_FILTERING_CACHE_DURATION /* expire */);
-  }
-
-  return(rc);
-}
-
-/* **************************************** */
-
 int Redis::getAddress(char *numeric_ip, char *rsp,
 		      u_int rsp_len, bool queue_if_not_found) {
   char key[CONST_MAX_LEN_REDIS_KEY];
@@ -927,15 +738,6 @@ int Redis::getAddress(char *numeric_ip, char *rsp,
 
 /* **************************************** */
 
-int Redis::setTrafficFilteringAddress(char *numeric_ip, char *httpbl) {
-  char key[CONST_MAX_LEN_REDIS_KEY];
-
-  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
-  return(set(key, httpbl, TRAFFIC_FILTERING_CACHE_DURATION));
-}
-
-/* **************************************** */
-
 int Redis::setResolvedAddress(char *numeric_ip, char *symbolic_ip) {
   char key[CONST_MAX_LEN_REDIS_KEY], numeric[256], *w, *h;
   int rc = 0;
@@ -960,9 +762,11 @@ int Redis::setResolvedAddress(char *numeric_ip, char *symbolic_ip) {
 
 /* **************************************** */
 
-char* Redis::getVersion(char *str, u_int str_len) {
+char* Redis::getRedisVersion() {
   redisReply *reply;
-
+  char str[32];
+  int a, b, c;
+  
   l->lock(__FILE__, __LINE__);
   num_requests++;
   reply = (redisReply*)redisCommand(redis, "INFO");
@@ -970,7 +774,7 @@ char* Redis::getVersion(char *str, u_int str_len) {
   if(reply && (reply->type == REDIS_REPLY_ERROR))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
 
-  snprintf(str, str_len, "%s" , "????");
+  snprintf(str, sizeof(str), "%s" , "????");
 
   if(reply) {
     if(reply->str) {
@@ -980,7 +784,7 @@ char* Redis::getVersion(char *str, u_int str_len) {
 
       while(line != NULL) {
 	if(!strncmp(line, tofind, tofind_len)) {
-	  snprintf(str, str_len, "%s" , &line[tofind_len]);
+	  snprintf(str, sizeof(str), "%s" , &line[tofind_len]);
 	  break;
 	}
 
@@ -990,59 +794,29 @@ char* Redis::getVersion(char *str, u_int str_len) {
 
     freeReplyObject(reply);
   }
+  
   l->unlock(__FILE__, __LINE__);
+  redis_version = strdup(str);
+  sscanf(redis_version, "%d.%d.%d", &a, &b, &c);
+  num_redis_version = (a << 16) + (b << 8) + c;
 
-  return(str);
-}
-
-/* **************************************** */
-
-#ifdef NOTUSED
-int Redis::hashIncr(char *key, char *field, u_int32_t value) {
-  int rc;
-  redisReply *reply;
-
-  if(key == NULL || field == NULL) return 0;
-
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "HINCRBY %s %s %u", key, field, value);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
-  l->unlock(__FILE__, __LINE__);
-
-  return(rc);
-}
-#endif
-
-/* **************************************** */
-
-/*
-  Hosts:       [name == NULL] && [ip != NULL]
-  StringHosts: [name != NULL] && [ip == NULL]
-*/
-int Redis::addHostToDBDump(NetworkInterface *iface, IpAddress *ip, char *name) {
-  char buf[64], daybuf[32], *what;
-  time_t when = time(NULL);
-  bool new_key;
-
-  strftime(daybuf, sizeof(daybuf), CONST_DB_DAY_FORMAT, localtime(&when));
-  what = ip ? ip->print(buf, sizeof(buf)) : name;
-  return(host_to_id(iface, daybuf, what, &new_key));
+  return(redis_version);
 }
 
 /* ******************************************* */
 
 int Redis::sadd(const char *set_name, char *item) {
-  return(msg_push("SADD", set_name, item, 0 /* do not trim, this is not a queue */));
+  int ret = msg_push("SADD", set_name, item, 0 /* do not trim, this is not a queue */);
+  checkDumpable(set_name);
+  return ret;
 }
 
 /* ******************************************* */
 
 int Redis::srem(const char *set_name, char *item) {
-  return(msg_push("SREM", set_name, item, 0 /* do not trim, this is not a queue */));
+  int ret = msg_push("SREM", set_name, item, 0 /* do not trim, this is not a queue */);
+  checkDumpable(set_name);
+  return ret;
 }
 
 /* **************************************** */
@@ -1117,115 +891,28 @@ int Redis::smembers(const char *set_name, char ***members) {
   return(rc);
 }
 
-/* *************************************** */
-
-void Redis::setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_int32_t id) {
-  char buf[32], keybuf[384], host_id[16], _daybuf[32], value[32];
-  //redisReply *reply;
-
-  if(daybuf == NULL) {
-    time_t when = time(NULL);
-
-    strftime(_daybuf, sizeof(_daybuf), CONST_DB_DAY_FORMAT, localtime(&when));
-    daybuf = _daybuf;
-  }
-
-  snprintf(keybuf, sizeof(keybuf), "%s|%s", iface->get_name(), host_name);
-
-  /* Set the data */
-  snprintf(buf, sizeof(buf), "ntopng.%s.hostkeys", daybuf);
-  snprintf(host_id, sizeof(host_id), "%u", id);
-  hashSet(buf, keybuf, host_id); /* Forth */
-  hashSet(buf, host_id, keybuf); /* ...and back */
-  snprintf(value, sizeof(value), "%s|%u", iface->get_name(), id);
-
-#if 0
-  l->lock(__FILE__, __LINE__);
-  num_requests++;
-  reply = (redisReply*)redisCommand(redis, "SADD %s.keys %s", daybuf, value);
-  if(!reply) reconnectRedis();
-
-  if(reply) {
-    if(reply->type == REDIS_REPLY_INTEGER) {
-      if(reply->integer != 1)
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "'SADD %s.keys %s|%u' returned %lld",
-				     daybuf, iface->get_name(), id, reply->integer);
-    } else if(reply->type == REDIS_REPLY_ERROR)
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-    else
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "Invalid reply type [%d]", reply->type);
-  }
-
-  ntop->getTrace()->traceEvent(TRACE_INFO, "Dumping %u", host_id);
-
-  if(reply) freeReplyObject(reply);
-  l->unlock(__FILE__, __LINE__);
-#endif
-}
-
-/* *************************************** */
-
-u_int32_t Redis::host_to_id(NetworkInterface *iface, char *daybuf,
-			    char *host_name, bool *new_key) {
-  u_int32_t id;
-  int rc;
-  char buf[32], keybuf[384], *rsp;
-
-  if(iface == NULL) return(-1);
-
-  if((rsp = (char*)malloc(CONST_MAX_LEN_REDIS_VALUE)) == NULL)
-    return(CONST_LUA_PARAM_ERROR);
-  
-  snprintf(keybuf, sizeof(keybuf), "%s|%s", iface->get_name(), host_name);
-
-  /* Add host key if missing */
-  snprintf(buf, sizeof(buf), "ntopng.%s.hostkeys", daybuf);
-  rc = hashGet(buf, keybuf, rsp, CONST_MAX_LEN_REDIS_VALUE);
-
-  if(rc == -1) {
-    /* Not found */
-    char host_id[16];
-
-    snprintf(host_id, sizeof(host_id), "%u", id = incrKey((char*)NTOP_HOSTS_SERIAL));
-    setHostId(iface, daybuf, host_name, id);
-    *new_key = true;
-  } else
-    id = atol(rsp), *new_key = false;
-
-  free(rsp);
-  
-  return(id);
-}
-
-/* *************************************** */
-
-int Redis::id_to_host(char *daybuf, char *host_idx, char *buf, u_int buf_len) {
-  char key[CONST_MAX_LEN_REDIS_KEY];
-
-  /* Add host key if missing */
-  snprintf(key, sizeof(key), "ntopng.%s.hostkeys", daybuf);
-  return(hashGet(key, host_idx, buf, buf_len));
-}
-
 /* ******************************************* */
 
-int Redis::lpush(const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors) {
+/*  Add at the top of queue */
+int Redis::lpush(const char * const queue_name, const char * const msg, u_int queue_trim_size, bool trace_errors) {
   return(msg_push("LPUSH", queue_name, msg, queue_trim_size, trace_errors));
 }
 
 /* ******************************************* */
 
-int Redis::rpush(const char *queue_name, char *msg, u_int queue_trim_size) {
-  return(msg_push("RPUSH", queue_name, msg, queue_trim_size));
+/* Add at the bottom of the queue */
+int Redis::rpush(const char * const queue_name, const char * const msg, u_int queue_trim_size) {
+  return(msg_push("RPUSH", queue_name, msg, queue_trim_size, true, false));
 }
 
 /* ******************************************* */
 
-int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size, bool trace_errors) {
+int Redis::msg_push(const char * const cmd, const char * const queue_name, const char * const msg,
+          u_int queue_trim_size, bool trace_errors, bool head_trim) {
   redisReply *reply;
   int rc = 0;
 
-  l->lock(__FILE__, __LINE__);
+  l->lock(__FILE__, __LINE__, trace_errors);
   /* Put the latest messages on top so old messages (if any) will be discarded */
   num_requests++;
   reply = (redisReply*)redisCommand(redis, "%s %s %s", cmd,  queue_name, msg);
@@ -1241,7 +928,10 @@ int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int qu
 
     if(queue_trim_size > 0) {
       num_requests++;
-      reply = (redisReply*)redisCommand(redis, "LTRIM %s 0 %u", queue_name, queue_trim_size - 1);
+      if(head_trim)
+        reply = (redisReply*)redisCommand(redis, "LTRIM %s 0 %u", queue_name, queue_trim_size - 1);
+      else
+        reply = (redisReply*)redisCommand(redis, "LTRIM %s -%u -1", queue_name, queue_trim_size);
       if(!reply) reconnectRedis();
       if(reply) {
 	if(reply->type == REDIS_REPLY_ERROR && trace_errors)
@@ -1254,10 +944,64 @@ int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int qu
   } else
     rc = -1;
 
-  l->unlock(__FILE__, __LINE__);
+  l->unlock(__FILE__, __LINE__, trace_errors);
   return(rc);
 }
 
+/* **************************************** */
+
+u_int Redis::len(const char * const key) {
+  redisReply *reply;
+  u_int num = 0;
+
+  l->lock(__FILE__, __LINE__);
+
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "STRLEN %s", key);
+
+  if(!reply) reconnectRedis();
+  if(reply) {
+    if(reply->type == REDIS_REPLY_ERROR)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    else
+      num = (u_int)reply->integer;
+  }
+
+  l->unlock(__FILE__, __LINE__);
+  if(reply) freeReplyObject(reply);
+
+  return(num);
+
+}
+
+/* **************************************** */
+
+#if 0 /* Only available since Redis 3.2.0 */
+u_int Redis::hstrlen(const char * const key, const char * const value) {
+  redisReply *reply;
+  u_int num = 0;
+
+  l->lock(__FILE__, __LINE__);
+
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "HSTRLEN %s %s", key, value);
+
+  if(!reply) reconnectRedis();
+  if(reply) {
+    if(reply->type == REDIS_REPLY_ERROR)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    else
+      num = (u_int)reply->integer;
+  }
+
+  l->unlock(__FILE__, __LINE__);
+  if(reply) freeReplyObject(reply);
+
+  return(num);
+
+}
+#endif
+
 /* ******************************************* */
 
 u_int Redis::llen(const char *queue_name) {
@@ -1268,10 +1012,12 @@ u_int Redis::llen(const char *queue_name) {
   num_requests++;
   reply = (redisReply*)redisCommand(redis, "LLEN %s", queue_name);
   if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-  else
-    num = (u_int)reply->integer;
+  if(reply) {
+    if(reply->type == REDIS_REPLY_ERROR)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    else
+      num = (u_int)reply->integer;
+  }
   l->unlock(__FILE__, __LINE__);
   if(reply) freeReplyObject(reply);
 
@@ -1449,11 +1195,53 @@ int Redis::lrange(const char *list_name, char ***elements, int start_offset, int
 
 /* **************************************** */
 
+int Redis::ltrim(const char *queue_name, int start_idx, int end_idx) {
+  int rc = 0;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+
+  reply = (redisReply*)redisCommand(redis, "LTRIM %s %d %d", queue_name, start_idx, end_idx);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    rc = -1, ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
+/* ******************************************* */
+
+u_int Redis::incr(const char *key) {
+  redisReply *reply;
+  u_int num = 0;
+
+  l->lock(__FILE__, __LINE__);
+  num_requests++;
+  reply = (redisReply*)redisCommand(redis, "INCR %s", key);
+  if(!reply) reconnectRedis();
+  if(reply) {
+    if(reply->type == REDIS_REPLY_ERROR)
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    else
+      num = (u_int)reply->integer;
+  }
+  l->unlock(__FILE__, __LINE__);
+  if(reply) freeReplyObject(reply);
+
+  return(num);
+}
+
+/* **************************************** */
+
 void Redis::lua(lua_State *vm) {
   lua_newtable(vm);
 
-  lua_push_int_table_entry(vm, "num_requests", num_requests);
-  lua_push_int_table_entry(vm, "num_reconnections", num_reconnections);
+  lua_push_uint64_table_entry(vm, "num_requests", num_requests);
+  lua_push_uint64_table_entry(vm, "num_reconnections", num_reconnections);
 
   lua_pushstring(vm, "redis");
   lua_insert(vm, -2);
@@ -1475,7 +1263,7 @@ void Redis::flushCache() {
     free(current);
   }
 
-  numCached = 0;
+  stringCache = NULL, numCached = 0;
   l->unlock(__FILE__, __LINE__);
 
 #ifdef CACHE_DEBUG
@@ -1546,8 +1334,8 @@ int Redis::restore(char *key, char *buf) {
   int rc;
   redisReply *reply;
   char *buf_bin = (char*)malloc(strlen(buf));
-  const char * argv[4] = {"RESTORE", key, "0", buf_bin};
-  size_t argvlen[4] = {7, 0, 0, 0};
+  const char * argv[5] = {"RESTORE", key, "0", buf_bin};
+  size_t argvlen[5] = {7, 0, 0, 0, 7};
 
   if(buf_bin == NULL)
     return(-1);
@@ -1557,13 +1345,28 @@ int Redis::restore(char *key, char *buf) {
   l->lock(__FILE__, __LINE__);
   num_requests++;
 
-  argvlen[1] = strlen(argv[1]);
-  argvlen[2] = strlen(argv[2]);
-  argvlen[3] = strlen(buf) / 2;
+  /* Delete the key first */
+  reply = (redisReply*)redisCommand(redis, "DEL %s", key);
+  if(!reply) reconnectRedis();
 
-  reply = (redisReply*)redisCommandArgv(redis, 4, argv, argvlen);
+  if(reply && (reply->type == REDIS_REPLY_ERROR)) {
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    rc = -1;
+  } else if(reply) {
+    freeReplyObject(reply);
 
-  rc = reply ? 0 : -1;
+    argvlen[1] = strlen(argv[1]);
+    argvlen[2] = strlen(argv[2]);
+    argvlen[3] = strlen(buf) / 2;
+
+    reply = (redisReply*)redisCommandArgv(redis, 4, argv, argvlen);
+
+    rc = reply ? 0 : -1;
+
+    if(reply && (reply->type == REDIS_REPLY_ERROR))
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s [RESTORE %s]", reply->str ? reply->str : "???", key), rc = -1;
+  } else
+    rc = -1;
 
   if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
